
; Homebrew.
; (C) 2009 Jonathan Cauldwell.
; Project started 07-05-2009 @21:33.
; Project completed 23-08-2009 @16:05.

; NB: Coordinates in my games are non-standard.  Throughout the code
; x refers to the number of pixels from the top of the screen, and
; y to the number of pixels from the left edge.



; Constant declarations.

SPRSIZ equ 4               ; bytes per sprite datum.
NUMSPR equ 10              ; number of sprites including player.
LEN    equ SPRSIZ * NUMSPR ; size of each table.
NUMBUL equ 5               ; number of bullets.
BULSIZ equ 6               ; bullet table entry size.


; Sprite types.

SPAWN  equ 16
APPLE  equ 20
PEAR   equ 21
STRAW  equ 22
NANA   equ 23
GRAPE  equ 24
MELON  equ 25
LEMON  equ 26
CHERRY equ 27
ORANGE equ 28
PLUM   equ 29
FRUIT  equ 32              ; fruit below here.
CORE   equ FRUIT
TIN    equ CORE + 1
FISH   equ TIN + 1
NUT    equ FISH + 1
BAD    equ NUT + 1         ; bad fruit below here.
WASP   equ BAD
WASPR  equ WASP + 4
BAT    equ WASPR + 4
BATR   equ BAT + 4
BIRD   equ BATR + 4
BIRDR  equ BIRD + 4
DRAG   equ BIRDR + 4
DRAGR  equ DRAG + 4
FUZZ   equ DRAGR + 4
ROBOT  equ FUZZ + 4
STAR   equ ROBOT + 4
PICKUP equ STAR + 4
CAKE   equ PICKUP
BURG   equ CAKE + 1
GEM    equ BURG + 1
EXLIFE equ GEM + 1
EXPLOD equ PICKUP + 4      ; explosion.
LASTEX equ EXPLOD + 6

ENDCHR equ 255             ; end-of-buffer marker.


       org 24500

; Set up the colours.

       xor a               ; zero is code for black.
       call 8859           ; set the permanent border.
       ld hl,23693         ; attributes.
       ld (hl),71          ; set to bright white on black.
waitk  call 654            ; get keypress.
       inc e               ; anything pressed?
       jr z,waitk          ; no, wait until something pressed.
       call 3503           ; clear the screen.
       jp menu             ; go straight into the menu code.

; Delay routine.

delay  halt                ; wait for an interrupt.
       djnz delay          ; repeat until b is zero.
       ret

stpos  defb 0,0,80,120     ; starting sprite frame and position.
gfram  defb 0              ; frame counter.
pretim defb 0              ; previous time setting.
txbuf  defw buffer         ; pointer to buffer position.
nament defb 22,16,12       ; position string for name.
buffer defb '        '     ; name entry buffer.
       defb ENDCHR         ; buffer end marker.

       org 24576           ; align on a 256-byte boundary.

; Pointers to sprite types.

       defw barrel,barr01,barr02,barr03,barr04,barr05,barr06,barr07
       defw barrel,barr01,barr02,barr03,barr04,barr05,barr06,barr07
       defw spaw00,spaw01,spaw02,spaw03
       defw appl00,pear00,strw00,nana00
       defw grape,melon,lemon,cherry,orange,plum,z80,z80
       defw core,can,fish,nut
       defw bee00,bee01,bee02,bee03
       defw bee04,bee05,bee06,bee07
       defw bat4,bat5,bat6,bat7,bat0,bat1,bat2,bat3
       defw bird4,bird5,bird6,bird5,bird0,bird1,bird2,bird1
       defw drag4,drag5,drag6,drag7,drag0,drag1,drag2,drag3
       defw fuzz0,fuzz1,fuzz2,fuzz3,robot0,robot1,robot2,robot3
       defw star0,star1,star2,star3
       defw cake,burg,gem,z80
       defw expl0,expl1,expl2,expl3,expl4,expl5

; Colours for each fruit.

frtcol defb 68,68,66,70,67,67,70,66,66,65,7,7  ; good fruits.
       defb 68,5,71,71                         ; bad items.


; Some beeper sound effects.

beep   ld b,2              ; iterations.
beep2  push bc             ; outer loop.
       ld b,e              ; duration.
       inc b               ; at least one iteation.
beep1  xor a               ; zeroes in speaker and border bits.
       out (254),a
       ld a,l              ; pitch.
beep3  dec a               ; one less.
       jr nz,beep3         ; inner pause.
       ld a,248            ; set speaker bit.
       out (254),a         ; toggle speaker.
       ld a,l              ; pitch.
beep0  dec a               ; one less.
       jr nz,beep0         ; inner pause.
       djnz beep1          ; repeat for duration.
       pop bc              ; outer loop counter.
       djnz beep2          ; repeat once.
       ret

; Chirp.

chirp  ld hl,70            ; initial pitch.
       ld de,0             ; minimum duration.
       ld b,45             ; length of chirp.
chirp0 push bc             ; store counter on stack.
       call beep           ; call beep routine.
       pop bc              ; retrieve counter.
       dec hl              ; raise pitch.
       djnz chirp0         ; repeat.
       ret



; Synch to interrupts.  May as well do beeper noise here while we wait.

haltr  ld hl,pretim        ; previous clock setting.
       ld a,(sndtyp)       ; type of sound to play.
       and a               ; any sound at all?
       jr z,haltr2         ; no, just wait.
       dec a               ; firing noise?
       jr z,haltf          ; play that sound then.
       dec a               ; chirp sound?
       jp z,snd4           ; yes, play it.
       dec a               ; bang sound?
       jr z,snd2           ; yes, play it.

; Don't need to set bullet colours, as background is awash with different
; attributes - but we'll extend the multicoloured bullet effect by
; setting a single bullet colour every frame.  Any more and the clash
; might be too much.  Bullets are tiny, after all.

haltr2 call random         ; pick a random bullet.
       and 7               ; in range 0 - 7.
       cp NUMBUL           ; is this a valid bullet?
       jr nc,haltr1        ; no.
       rlca                ; multiply bullet number by 2.
       ld hl,bultab        ; bullets table.
       call addac          ; add to table address.
       rlca                ; multiple of 4.
       call addac          ; find displacement to bullet (bullet * 6).
       ld a,(hl)           ; get bullet status.
       rla                 ; is it on?
       jr c,haltr1         ; no, nothing to do then.
       inc hl              ; 3 bytes on to find x integer.
       inc hl
       inc hl
       ld de,dispx         ; x coordinate.
       ldi                 ; set up display x.
       inc hl              ; skip y fraction byte.
       ldi                 ; set up display y.
       ld ix,dispx-2       ; ataddp routine uses ix+2 and ix+3 for coords.
       call ataddp         ; find attribute address.
       push hl             ; store address.
       call random         ; random number.
       and 3               ; 4 values.
       add a,68            ; green through white.
       pop hl              ; restore attribute address.
       ld (hl),a           ; set bullet attributes.

haltr1 ld hl,pretim        ; previous clock setting.
       ld a,(23672)        ; current clock setting.
       sub (hl)            ; subtract last reading.
       cp 2                ; reached elapsed time already?
       jr nc,haltr0        ; yes, no more processing please.
       jp haltr1
haltf  ld c,0              ; value to send to speaker.
       ld b,25             ; frequency.
haltf0 ld a,c              ; speaker value.
       xor 248             ; toggle bits.
       ld c,a              ; new speaker value.
       out (254),a         ; send to speaker.
       push bc             ; store value and loop.
haltf2 djnz haltf2         ; delay loop.
       pop bc              ; restore value.
       ld a,(23672)        ; clock setting.
       sub (hl)            ; subtract previous value.
       cp 2                ; two frames passed already?
       jr nc,haltr0        ; yes, job done.
       djnz haltf0
       jr haltr1           ; done, so now we have to wait.

snd2   ld b,40             ; outer loop.
       ld c,0              ; value to write to speaker.
snd2c  call random         ; get speaker value.
       and 248             ; toggle all except the border bits.
       out (254),a         ; write to speaker.
       push bc             ; store loop counter.
snd2a  ld hl,pretim        ; previous clock setting.
       ld a,(23672)        ; current clock setting.
       sub (hl)            ; subtract last reading.
       cp 2                ; reached elapsed time already?
       jp nc,snd2b         ; yes, no more processing please.
       djnz snd2a          ; loop.
       pop bc              ; restore loop counter.
       djnz snd2c          ; continue noise.
       jr haltr1           ; make sure we wait for interrupt.

; Time up, cannot complete the sound.

snd2b  pop bc              ; restore loop counter.
       jr haltr0

snd4   ld b,40             ; outer loop.
       ld c,0              ; value to write to speaker.
snd4c  ld a,c              ; get speaker value.
       out (254),a         ; write to speaker.
       xor 248             ; toggle all except the border bits.
       ld c,a              ; store value for next time.
       push bc             ; store loop counter.
snd4a  ld hl,pretim        ; previous clock setting.
       ld a,(23672)        ; current clock setting.
       sub (hl)            ; subtract last reading.
       cp 2                ; reached elapsed time already?
       jp nc,snd4b         ; yes, no more processing please.
       djnz snd4a          ; loop.
       pop bc              ; restore loop counter.
       djnz snd4c          ; continue noise.
       jr haltr0

; Time up, cannot complete the sound.

snd4b  pop bc              ; restore loop counter.
haltr0 ld a,(23672)        ; current clock setting.
       ld (pretim),a       ; store it for next time.
       xor a               ; zero in accumulator.
       ld (sndtyp),a       ; reset sound type.
       ret

sndtyp defb 0              ; sound type.

; Main menu.
; This routine decides whether to display the menu, high scores
; or credits, depending on the the value in menuno.

menu   call cls            ; clear the screen.
       ld hl,menuno        ; menu number to display.
       inc (hl)            ; move it along.
       ld a,(hl)           ; get menu number.
       and 7               ; only 8 to display.
       rlca                ; multiply byte by 2.
       ld hl,menptr        ; pointer to menu routines.
       call addac          ; point to address of routine.
       ld e,(hl)           ; low byte of address in e.
       inc hl              ; next byte.
       ld d,(hl)           ; high byte of address in e.
       ex de,hl            ; put address in hl.
       jp (hl)             ; jump to routine.

menptr defw menua,dishs    ; menu display routine sequence.
       defw menua,infosc
       defw menua,dishs
       defw menua,creds
menuno defb 255



; Menu options.
; Detect keypresses.

menu1  ld b,0              ; delay of approx 10 seconds.
menu0  halt
       push bc             ; store count.
       halt
       call 654            ; keyboard routine.
       pop bc              ; restore count.
       ld a,e              ; test key returned.
       cp 36               ; keyboard selected?
       jp z,gamek
       cp 28               ; Kempston joystick option?
       jp z,gamejk
       cp 20               ; is it Sinclair?
       jp z,gamejs
       cp 12               ; is it redefine keys?
       jp z,redkey         ; redefine them then.
       djnz menu0
       jp menu


mseq   defb 0              ; menu sequence.

menua  call expand         ; decompress menu screen.
       ld de,mtxt0         ; menu text.
       ld bc,crdtxt-mtxt0  ; size of string.
       call 8252           ; display string.
       jp menu1            ; now wait for a keypress.
creds  ld de,crdtxt        ; credits text.
       ld bc,redkey-crdtxt ; size of string.
       call 8252           ; display string.
       jp menu1            ; now wait for a keypress.

mtxt0  defb 22,4,16,16,6
       defb '1. KEYBOARD'
       defb 22,5,16
       defb '2. KEMPSTON'
       defb 22,6,16
       defb '3. SINCLAIR'
       defb 22,7,16
       defb '4. NEW KEYS'

crdtxt defb 22,1,11,16,5
       defb 'INITIATOR'
       defb 22,7,5
       defb 'DESIGN AND PROGRAMMING'
       defb 22,13,12
       defb 'GRAPHICS'
       defb 22,19,10
       defb 'THE DRINKER'
       defb 22,3,9,16,6
       defb 'DAVID CROOKES'
       defb 22,9,7
       defb 'JONATHAN CAULDWELL'
       defb 22,15,9
       defb 'JARROD BENTLEY'
       defb 22,21,14
       defb 'GEO'

redkey call 654            ; ROM routine to read keypress.
       ld a,e              ; get value of key pressed.
       inc a               ; is something being pressed now?
       jr nz,redkey        ; yes - loop until key is released.
       call cls            ; ROM routine to clear screen.
       ld hl,keys          ; redefined keys.
       ld de,keys+1        ; next byte of buffer.
       ld bc,4             ; keys to set minus one.
       ld (hl),255         ; reset first key.
       ldir                ; clear remaining keys.
       ld de,ktxt0         ; prompt to display.
       ld bc,ktxt1-ktxt0   ; length.
       call 8252           ; display prompt for next key.
       call redky0         ; wait for keypress.
       ld a,e
       ld (keys),a         ; set this as up key.
       ld de,ktxt1         ; prompt to display.
       ld bc,ktxt2-ktxt1   ; length.
       call 8252           ; display prompt for next key.
       call redky0         ; wait for keypress.
       ld a,e
       ld (keys+2),a       ; set this as left key.
       ld de,ktxt2         ; prompt to display.
       ld bc,ktxt3-ktxt2   ; length.
       call 8252           ; display prompt for next key.
       call redky0         ; wait for keypress.
       ld a,e
       ld (keys+3),a       ; set this as right key.
       ld de,ktxt3         ; prompt to display.
       ld bc,ktxt4-ktxt3   ; length.
       call 8252           ; display prompt for next key.
       call redky0         ; wait for keypress.
       ld a,e
       ld (keys+4),a       ; set this as fire key.
       ld de,ktxt4         ; prompt to display.
       ld bc,ktxt5-ktxt4   ; length.
       call 8252           ; display prompt for next key.
       call redky0         ; wait for keypress.
       ld a,e
       ld (keys+5),a       ; set this as fire key.
       ld b,50             ; one second.
       call delay          ; pause to show last key definition.
       jp menu

ktxt0  defb 22,2,9,16,4
       defb 'PRESS KEY FOR:'
       defb 22,7,8,16,6
       defb 'FORWARD',22,7,20
ktxt1  defb 22,9,8
       defb 'TURN LEFT',22,9,20
ktxt2  defb 22,11,8
       defb 'TURN RIGHT',22,11,20
ktxt3  defb 22,13,8
       defb 'FIRE',22,13,20
ktxt4  defb 22,15,8
       defb 'PAUSE',22,15,20
ktxt5  defb '   B   H   Y   6   5   T   G   V'
       defb '   N   J   U   7   4   R   F   C'
       defb '   M   K   I   8   3   E   D   X'
       defb ' SYM   L   O   9   2   W   S   Z'
       defb ' SPA ENT   P   0   1   Q   ACAPS'

redky0 call 654            ; get keypress in e register.
       ld a,e              ; get value.
       inc a               ; is it 255?
       jr z,redky0         ; yes - loop until something pressed.
       ld a,(keys)         ; first key.
       cp e                ; is this the same?
       jr z,redky0         ; yes - try again.
       ld a,(keys+1)       ; left key.
       cp e                ; is this the same?
       jr z,redky0         ; yes - try again.
       ld a,(keys+2)       ; right key.
       cp e                ; is this the same?
       jr z,redky0         ; yes - try again.
       ld a,(keys+3)       ; fire key.
       cp e                ; is this the same?
       jr z,redky0         ; yes - try again.
       ld a,(keys+4)       ; pause key.
       cp e                ; is this the same?
       jr z,redky0         ; yes - try again.
       push de             ; store keypress on stack.
       ld hl,ktxt5         ; table of key names.
       sla e
       sla e               ; multiply key value by 4.
       ld d,0              ; no high byte.
       add hl,de           ; name of key in look-up table.
       ex de,hl            ; want result in DE.
       ld bc,4             ; length = 4.
       call 8252           ; ROM routine to display string.
       pop de              ; retrieve keypress.
       ret

infosc call cls            ; clear screen.
       ld de,instx0        ; text screen one.
       ld bc,dishs-instx0  ; length of string.
       call 8252           ; display text.
       jp menu1            ; menu options.

instx0 defb 22,1,10,16,6
       defb 'INSTRUCTIONS'
       defb 22,5,2,16,5
       defb 'SHOOT THE NASTIES TO COLLECT'
       defb 22,7,2
       defb 'FRUIT, THEN DROP IT INTO THE'
       defb 22,9,6
       defb 'DEMI-JOHN TO FERMENT'
       defb 22,12,2,16,7
       defb 'COLLECT CAKES AND BURGERS TO'
       defb 22,14,3
       defb 'REBUILD YOUR ENERGY LEVELS'
       defb 22,17,3,16,5
       defb 'DIAMONDS TURN ENEMIES INTO'
       defb 22,19,2
       defb 'STARS WHICH CAN BE PICKED UP'
       defb 22,21,8
       defb 'FOR BONUS POINTS'


; Display high-score table.

dishs  ld de,hstxt0        ; title at the top.
       ld bc,hstab-hstxt0  ; length of string.
       call 8252           ; show text.
       ld b,6              ; number of entries to display.
       ld hl,hstab         ; high score table.
dishs0 push bc
       push hl
       ld a,9              ; top of table.
       sub b               ; subtract position.
       rlca
       push af             ; store pos.
       ld a,22             ; AT code.
       rst 16              ; display this char.
       pop af              ; restore position.
       rst 16              ; display this char.
       ld a,8              ; column number.
       rst 16              ; display this char.
       pop de
       push de
       ld bc,8             ; length of name.
       call 8252           ; display name.
       ld a,32             ; space character.
       rst 16              ; display it.

       pop hl
       push hl
       ld de,8             ; length of each name.
       add hl,de           ; add to pointer to find score.
       ex de,hl            ; string address in de please.
       ld bc,6             ; number of digits in score.
       call 8252           ; show digits.
       pop hl
       pop bc
       ld de,14            ; distance to next name.
       add hl,de
       djnz dishs0         ; repeat for all entries.
       call random         ; random number.
       and 7               ; put it in range 0 - 7.
       add a,APPLE         ; displacement for first fruit sprite.
       ld (frtreq),a       ; set up display sprite.
       ld hl,12+24*256     ; coordinates for sprite.
       ld (dispx),hl       ; set the display coordinates.
       call dfrt           ; show the fruit.
       ld hl,frtreq        ; fruit we displayed.
       inc (hl)            ; show next one now.
       ld hl,12+216*256    ; coordinates for sprite.
       ld (dispx),hl       ; set the display coordinates.
       call dfrt           ; show the fruit.
       jp menu1            ; return to menu.

hstxt2 defb 22,5,7,16,6
       defb "THAT'S A NICE DROP"
       defb 22,8,5,16,5
       defb 'PLEASE LEAVE YOUR NAME'
       defb 22,10,4
       defb 'AND THE WINE CIRCLE WILL'
       defb 22,12,3
       defb 'BE IN TOUCH FOR THE RECIPE'
       defb 16,7

; High score table text.

hstxt0 defb 22,2,9,16,4
       defb 'MASTER BREWERS'
       defb 22,21,9
       defb 'COPYRIGHT 2009'
       defb 16,6
hstab  defb 'JONATHAN000500'
       defb 'JARROD  000400'
       defb 'DAVID   000300'
       defb 'BARNEY  000200'
       defb 'MOJO    000100'
       defb 'IAN     000005'



; Try high score table to see if score qualifies.

tryhs  ld hl,hstab+78      ; end of high score table.
       ld b,6              ; number of scores to check.
tryhs0 push bc             ; store list counter.
       ld de,score         ; player's score.
       push hl             ; store position in table.
       ld b,6              ; digits to compare.
tryhs3 ld a,(de)           ; player's score digit.
       cp (hl)             ; table digit.
       jr c,tryhs1         ; score is greater.
       jr nz,tryhs2        ; score is lower.
       inc hl              ; digit is the same - go to next column.
       inc de
       djnz tryhs3
       jr tryhs1           ; score is same, first one registered comes first.
tryhs2 pop hl
       ld de,65522         ; 65536 - 14.
       add hl,de           ; subtract 14 - next entry in table.
       pop bc
       djnz tryhs0
       ld hl,hstab+8

; top of table.

tryhs4 push hl
       ld de,65528
       add hl,de
       ld d,h
       ld e,l
       ld hl,hstab+84
       and a
       sbc hl,de
       ld b,h
       ld c,l
       ld hl,hstab+69
       ld de,hstab+83
       lddr
       pop hl
       ld de,score         ; player's score.
       ex de,hl
       ld bc,6             ; number of digits to transfer.
       ldir                ; copy to table.
       ld hl,65522         ; minus 14.
       add hl,de           ; go back to start of name.
       ex de,hl            ; address of name in de.
       push de
       call ent            ; enter name.
       pop de
       ld hl,buffer        ; place where name is stored.
       ld bc,8             ; length of name.
       ldir                ; copy to high score table.
       ret
tryhs1 pop hl
       pop bc
       ld a,b
       cp 6                ; failed at bottom of table?
       ret z               ; yes - no high score then.
       ld de,14            ; next element down.
       add hl,de           ; point to next in list.
       jr tryhs4           ; put in table at this point.

ent    call hsnam          ; high score name routine.
       jp ent13a           ; check for rude words etc.


; Standard high score name entry routine.

hsnam  call cls            ; clear screen.
       ld de,hstxt2        ; high score text.
       ld bc,hstxt0-hstxt2 ; length of string.
       call 8252           ; prompt for name.
       ld hl,buffer        ; input line in buffer.
       ld (txbuf),hl       ; set buffer address pointer.
       ld de,buffer+1      ; second char of buffer.
       ld bc,7             ; 8 characters maximum.
       ld (hl),32          ; make first character a space.
       ldir                ; copy to rest of buffer.
hsn00  call curon          ; switch on cursor.
hsn02  ld de,nament        ; string we're typing in.
       ld bc,11            ; 8 chars plus position string.
       call 8252           ; display on screen.
       ld a,(23556)        ; key pressed.
       inc a               ; anything pressed?
       jr nz,hsn02         ; yes, debounce it.
hsn01  ld a,(23556)        ; what was key pressed?
       ld c,a              ; store keypress.
       inc a               ; anything pressed?
       jr z,hsn01          ; no, wait until it is.
       call curoff         ; remove cursor.
       ld a,c              ; restore keypress.
       cp 13               ; enter.
       ret z
       cp '5'              ; 5/cursor left.
       jp z,hsn08
       cp '8'              ; 8/cursor right.
       jp z,hsn09
       cp '0'              ; 0/delete.
       jp z,hsn12
       cp 32               ; is it space?
       jr z,hsn32          ; frig a space character.
       cp 58               ; below letter 'A'?
       jr c,hsn02          ; yes, don't accept.
hsn32  ld c,a              ; put key in c.
hsn00a ld a,(buffer+7)     ; last char.
       cp ' '              ; is it space?
       jp nz,hsn00         ; no more room on line.
       ld hl,(txbuf)       ; pointer to place in string.
hsn00b ld a,(hl)           ; get character we'll overwrite.
       ld (hl),c           ; place character there.
       ld c,a              ; get character overwritten.
       inc hl              ; point to next in string.
       ld a,(hl)           ; get this character.
       cp ENDCHR           ; end of buffer?
       jr nz,hsn00b        ; no, keep going.
       ld hl,(txbuf)       ; text position.
       call curoff         ; cursor is already off, just want address.
       ld a,l              ; put into accumulator.
       cp 19               ; reached end of string?
       jp z,hsn00          ; yes, cursor is at end of string.
       ld hl,(txbuf)       ; get buffer position.
       inc hl              ; point to next one.
       ld (txbuf),hl       ; store pointer.
       jp hsn00            ; wait for next character.
hsn08  call curoff         ; get address in l.
       ld a,l              ; y position now in accumulator.
       cp 12               ; reached the minimum?
       jp z,hsn00          ; yes.
       ld hl,(txbuf)       ; buffer position.
       dec hl              ; back a character.
       ld (txbuf),hl       ; new position.
       jp hsn00            ; back to input routine.
hsn09  call curoff         ; get address in l.
       ld a,l              ; y position now in accumulator.
       cp 19               ; reached the end?
       jp z,hsn00          ; yes, wait for next character.
       ld hl,(txbuf)       ; text pointer.
       inc hl              ; next character.
       ld (txbuf),hl       ; point there.
       jp hsn00            ; next letter.
hsn12  call curoff         ; position on screen.
       ld a,l              ; y position now in accumulator.
       cp 12               ; at left edge?
       jp z,hsn00          ; yes, no more please.
       ld hl,(txbuf)       ; text position.
       ld d,h              ; copy to de.
       ld e,l
       dec hl              ; left a character.
       ld (txbuf),hl       ; new pointer.
hsn12b ld a,(de)           ; get buffer character.
       cp ENDCHR           ; reached the end?
       jr z,hsn12a         ; no, keep going.
       ld (hl),a           ; write new character.
       inc hl              ; next in buffer.
       inc de              ; ditto.
       jr hsn12b           ; repeat until end of string.
hsn12a ld (hl),' '         ; space out final character.
       jp hsn00

; Cursor on/off.

curoff ld b,0              ; cursor off.
       jr curset           ; draw cursor.
curon  ld b,124            ; cursor on.
curset ld hl,(txbuf)       ; buffer position.
       ld de,buffer-12     ; buffer minus 12 chars for centre of screen.
       and a               ; clear carry.
       sbc hl,de           ; find number of chars along.
       ld h,87             ; final segment, last line.
       ld (hl),b           ; set sursor.
       ret



; Check if a known word has been entered.
; On entry HL points to the buffer.

word   ld de,vocab             ; table of naughty words.
word4  ld hl,buffer            ; point to buffer.
word0  ld a,(de)               ; DE=char in vocabulary table.
       dec a
       inc de
       cp (hl)                 ; do they match?
       jr nz,word1             ; no - find next word.
       cp 254                  ; end of listed word?
       jr z,word2              ; yes - try next word in table.
       inc hl                  ; next buffer char.
       jr word0
word1  cp 254                  ; end of word?
       jr z,word5              ; yes - return verb number.
word2  ld a,(de)
       inc de
       cp 255                  ; have we reached end of word in table?
       jr nz,word2             ; no, loop round again.
       ld a,(de)               ; which word is this?
       inc de
       cp 255                  ; is it the last one?
       jr nz,word4             ; no.
       xor a                   ; return 0 (not found) in A.
       ret
word5  ld a,(de)               ; name number returned in A.
       ret

; This is the recognised naughty word list.

vocab  defb '!!!!!!!',255,1    ; empty.
       defb '.......',255,1
       defb '///////',255,1
       defb 'GVDL',255,2       ; swear words.
       defb 'DVOU',255,2
       defb 'CPMMPDLT',255,2
       defb 'BSTF',255,2
       defb 'TIJU',255,2
       defb 'XBOL',255,2
       defb 'QJTT',255,2
       defb 'UXBU',255,2
       defb 'UJUT',255,2
       defb 'QSJDL',255,2
       defb 'DPNNPEF',255,2
       defb 'GPHBSUZ',255,3    ; Fogarty.
       defb 'DBODFS',255,4
       defb 255,255

namep  defb 'NAMELESS'     ; empty name text.
nameq  defb 'NOT HIM!'     ; custom name.
name   defb 'CHARMING'     ; obscenity substition.
namec  defb 'BEAT IT '     ; cheat text.


; Oh dear, how boring.  Let's put something else in for him.

entemp ld hl,namep         ; blank name substitute.
entob0 ld de,buffer        ; text buffer.
       ld bc,8             ; number of chars to copy.
       ldir                ; copy name into buffer.
       ret

ent13a ld hl,buffer        ; point to buffer.
       call word           ; find word in list.
       and a               ; is it recognised?
       ret z               ; no, it's okay as it is then.
       dec a               ; is it empty?
       jr z,entemp         ; replace rudeness.
       dec a               ; is it naughty?
       jr z,entob          ; replace rudeness.
       dec a               ; is it the custom stuff?
       jp z,endcus         ; yes, let's be silly.
       dec a               ; is it the cheat?
       ret nz              ; no, it's okay as it is then.
       ld a,5              ; lives with which game starts.
       ld (cheatb+1),a     ; set cheat mode - start with 5 lives.
       ld hl,namec         ; cheat name.
       jr entob0

entob  ld hl,name          ; obscenity substitute.
       jr entob0           ; use this name instead.

endcus ld a,3              ; default is 3 lives.
       ld (cheatb+1),a     ; reset cheat mode - start with 3 lives.
       ld hl,nameq         ; custom name.
       jr entob0           ; copy name across.


gamejk ld a,1              ; Kempston setting.
       jr gamek0           ; start.
gamejs ld a,2              ; Sinclair setting.
       jr gamek0           ; start.
gamek  xor a               ; keyboard setting.
gamek0 ld (contrl),a       ; set control type.

; Game initialisation.

game   xor a               ; zero.
       ld (levnum),a       ; start at first level.
cheatb ld a,3              ; set up lives in accumulator.
       ld (numlif),a       ; number of lives.
       ld a,4              ; levels before life is awarded.
       ld (levlif),a       ; set up counter.

       ld a,(23672)        ; get current clock setting.
       ld (seed),a         ; seed our random numbers from this.

       ld hl,score         ; score string.
       ld b,6              ; number of digits.
init0  ld (hl),'0'         ; set to zero.
       inc hl              ; next digit.
       djnz init0

       call cls            ; clear the screen.

       call newlif         ; new life stuff.

rstart ld hl,oldtab        ; start of old table.
       ld de,oldtab+1      ; second byte.
       ld bc,LEN+LEN-1     ; total area occupied by both tables.
       ld (hl),255         ; blank first byte.
       ldir                ; blank entire table.

       ld de,newtab        ; new sprite table.
       ld hl,stpos         ; player starting position.
       ld bc,SPRSIZ        ; bytes to copy.
       ldir                ; set up first player position.

       ld hl,bultab            ; table of bullets.
       ld de,bultab+1          ; next byte.
       ld bc,NUMBUL*BULSIZ-1   ; area to initialise.
       ld (hl),255             ; set first character.
       ldir                    ; reset all bullets.

       xor a               ; zero.
       ld (juice),a        ; reset juice level.
       ld (nasty),a        ; reset nasty ingredients counter.
       ld (fruit),a        ; no juice to top-up.
       ld (sndtyp),a       ; reset beeper sound type.

       ld (dflag),a        ; reset player dead flag.
       ld (fnum),a         ; zeroise pieces collected.
       ld (deplet),a       ; reset energy depletion flag.
       ld (increm),a       ; reset energy increment flag.
       ld a,(levnum)       ; level number.
       add a,128           ; minimum duration before bonus appears.
       ld (pflag),a        ; set pick-up timer.

       call lvint          ; level intro.

; Start of main loop.

mloop  call dspr           ; display and delete sprites.
       ld hl,newtab        ; new sprite table.
       ld de,oldtab        ; old positions.
       ld bc,newtab-oldtab ; bytes to copy.
       ldir                ; copy to old positions.

; Now process sprite movement.

       ld a,(dflag)        ; player dead flag.
       and a               ; has he been killed?
       call z,player       ; no, do all player controls.
       call colour         ; do additional colours.
       call pspr           ; process all sprites.
       call spawn          ; spawn new sprites.
       call mvbul          ; move bullets.
       call drop           ; drop any held fruit.
       call pick           ; pick-ups and bonuses.
       call cnasty         ; check for nasties in the wine.
       call rshld          ; any shield reductions.
       call pause          ; pause key.
       call haltr          ; keep at a constant 25FPS.
       call nexlev         ; check for level completion.

       call demij          ; add any juice to demi-john.
       ld hl,gfram         ; frame number.
       inc (hl)            ; move it along.
       ld a,(dflag)        ; dead flag.
       and a               ; is it set?
       jp z,mloop          ; no, keep playing.

; End of main loop.  If we reach this point, the player is dead.
; Explode the sprite, then wait a second before restarting.

pdead  inc a               ; increment counter.
       ld (dflag),a        ; new value.
       cp 80               ; time to restart?
       jr nc,pdead1        ; yes, stop this nonsense.

       and 3               ; every 4th frame.
       dec a               ; time to update explosion sprite?
       jp nz,mloop         ; no, keep going.
       ld a,(newtab)       ; player sprite type.
       inc a               ; move it along.
       jp z,mloop          ; nothing to show.
       cp LASTEX           ; past final frame?
       jr c,pdead0         ; no, frame is okay.
       ld a,255            ; disable sprite completely.
       ld (newtab),a       ; new sprite frame.
       jp mloop            ; keep going for now.
pdead0 ld (newtab),a       ; new sprite frame.
       ld a,3              ; sound to play.
       ld (sndtyp),a       ; set the sound type.
       jp mloop            ; keep going for now.

pdead1 ld hl,numlif        ; number of lives.
       dec (hl)            ; one less.
       jr z,gameov         ; none left, game over.
       call cls            ; clear the screen.
       call newlif         ; new life stuff.
       jp rstart           ; not reached the end, restart.

gameov ld de,gotxt         ; game over text.
       ld bc,newlif-gotxt  ; length of string.
       call 8252           ; display text.
       ld b,150            ; three seconds.
       call delay          ; pause.
       call tryhs          ; check for a high-score.
       jp menu             ; restart menu.
gotxt  defb 22,10,11,16,7
       defb 'GAME'
       defb 22,10,17
       defb 'OVER'

; Draw the status panel.

newlif ld de,panel         ; status panel address.
       ld hl,20672         ; 16 pixels up from screen bottom.
       ld b,16             ; lines to write.
dpanl0 push bc             ; store line counter.
       push hl             ; store screen address.
       ex de,hl            ; swap source into hl, target into de.
       ld bc,32            ; bytes to transfer to screen.
       ldir                ; copy line of image to screen.
       ex de,hl            ; swap source and target back.
       pop hl              ; restore screen address.
       call nline          ; next line down.
       pop bc              ; restore loop counter.
       djnz dpanl0
       ex de,hl            ; flip source address into hl.
       ld bc,64            ; two rows to colour.
       ld de,23232         ; two attribute rows up.
       ldir                ; copy attributes to screen.

       ld hl,20708         ; right edge of energy bar.
       ld (enbar),hl       ; set up pointer.
       ld a,1              ; rightmost bit.
       ld (enbit),a        ; set bit as first to go.
       ld bc,1*256+32      ; 1 line up, 32 left.
       call 3545           ; set cursor position.
       ld de,pantxt        ; panel text.
       ld bc,dflag-pantxt  ; length of string.
       call 8252           ; set up panel.
       call dscor          ; display score.

dlives ld bc,1*256+22      ; 1 line up, 22 chars left.
       call 3545           ; set cursor position.
       ld a,(numlif)       ; number of lives remaining.
       add a,'0'           ; add ASCII code for zero.
       rst 16              ; display lives counter.
       ret

; Level intro stuff.


lvint  call getlev         ; current level.
       rlca                ; multiply by 2.
       ld hl,levreq        ; table of fruit required.
       call addac          ; point to entry for table.
       ld de,frtreq        ; fruit required.
       ldi                 ; transfer the fruit.
       ldi                 ; and the quantity.
       ld de,levtxt        ; next level text.
       ld bc,levqty-levtxt ; length of string we wish to display.
       call 8252           ; call ROM routine to display string.
       ld a,(frtqty)       ; quantity wanted.
       ld c,a              ; copy to bc pair.
       ld b,0
       call 6683           ; show as a number.
       ld hl,52+96*256     ; (52, 96).
       ld (dispx),hl       ; set the display coordinates.
       call dfrt           ; display fruit required.
       ld de,levqty        ; next level text.
       ld bc,levclr-levqty ; length of string we wish to display.
       call 8252           ; call ROM routine to display string.
       ld a,(levnum)       ; level number.
       inc a               ; start at 1 instead of 0.
       ld c,a              ; copy accumulator to bc.
       ld b,0              ; obviously, no high byte.
       call 6683           ; ROM routine to display bc as a number.

; Draw the demi-john.

       ld de,demigr        ; demi-john graphic.
       ld hl,18668         ; thirteen chars in to segment.
       ld b,72             ; pixel lines to draw.
ddemi0 push bc             ; store line counter.
       push hl             ; store screen address.
       ex de,hl            ; swap source into hl, target into de.
       ld bc,8             ; bytes to transfer to screen.
       ldir                ; copy line of image to screen.
       ex de,hl            ; swap source and target back.
       pop hl              ; restore screen address.
       call nline          ; next line down.
       pop bc              ; restore loop counter.
       djnz ddemi0

       call colour         ; do additional colours.

; Display the bonus fruit on the status panel.

       ld a,(frtreq)       ; fruit wanted.
       sub APPLE           ; start at zero.
       rlca
       rlca
       rlca                ; multiply by 8.
       ld hl,tinyf         ; tiny fruit graphics.
       call addac          ; set hl to address of fruit.
       ld (23675),hl       ; point UDGs there.
       ld bc,1*256+12      ; 1 line up, 12 chars left.
       call 3545           ; set the print position.
       ld a,144            ; UDG 'A'.
       rst 16              ; display UDG.

       ld a,(frtreq)       ; required fruit.
       ld hl,frtcol        ; fruit colour table.
       sub APPLE           ; adjust for start of table.
       call addac          ; point to colour.
       ld a,(hl)           ; get sprite colour.
       ld (23285),a        ; attribute address.

       call dbon           ; display bonus quantity.

       ld b,200            ; delay length.
       call delay          ; pause momentarily.
       ld de,levclr        ; next level text clearance string.
       ld bc,nexlev-levclr ; length of string we wish to display.
       jp 8252             ; show string.

; Display bonus status.

dbon   ld bc,1*256+13      ; 1 line up, 13 chars left.
       call 3545           ; set cursor position.
       ld a,(frtqty)       ; quantity wanted.
       add a,'0'           ; convert to ASCII.
       rst 16              ; display number required.
       ret

frtreq defb 0              ; fruit required for bonus.
frtqty defb 0              ; number still outstanding.

dfrt   ld a,(frtreq)       ; fruit wanted.
       call sspr           ; show single sprite.
       ld a,(frtreq)       ; fruit wanted.
       ld hl,frtcol        ; fruit colour table.
       sub APPLE           ; adjust for start of table.
       call addac          ; point to colour.
       ld a,(hl)           ; get sprite colour.
       ld ix,dispx-2       ; cspr uses (ix+2, ix+3) as coordinates.
       jp cspr             ; set the colours.

bontx1 defb 22,9,9
       defb 'BONUS A'
       defb 'CHIEVED'
bontx2 defb 22,9,10
       defb 'BONUS '
       defb 'MISSED'
bonclr defb 22,9,0,6,6
levtxt defb 22,4,12        ; 22 is the AT code, followed by X and Y.
       defb 'LEVEL'
       defb 22,7,4
       defb 'COLLECT'
       defb 22,7,15,'X '
levqty defb ' FOR BONUS'
       defb 22,4,18
levclr defb 22,4,10        ; 22 is the AT code, followed by X and Y.
       defb 6,6,6,6,6,6,6  ; 6 is code for TAB, which spaces out the text.
       defb 6,6,6

nexlev ld a,(juice)        ; need to check level here.
       cp 40               ; filled container?
       ret c               ; no, not yet.
       ld a,(dflag)        ; player still alive?
       and a               ; zero indicates he is.
       ret nz              ; no, can't proceed.

; Level completed.

       pop de              ; remove return address from stack.

       ld a,(levlif)       ; levels before another life appears.
       and a               ; is extra life overdue?
       jr z,nexlv0         ; yes, don't wrap to 255.
       dec a               ; one less to complete.
       ld (levlif),a       ; set new tally.
nexlv0 ld hl,newtab        ; start of old sprite table.
       ld de,newtab+1      ; second byte.
       ld bc,LEN-1         ; total area occupied by table.
       ld (hl),255         ; blank first byte.
       ldir                ; blank entire table.
       call dspr           ; delete the old sprites.
       call dbul           ; delete the bullets.
       call colour         ; colour the demi-john.
       ld hl,levnum        ; level number.
       inc (hl)            ; next one please.

       call chirp          ; short sound effect.
       ld b,100            ; delay length of 2 seconds.
       call delay          ; short pause.

; Did we achieve the bonus?

       ld a,(frtqty)       ; quantity of fruit required.
       and a               ; any still left to collect?
       call z,adbon        ; no, add bonus.
       ld a,(frtqty)       ; quantity of fruit required.
       and a               ; any still left to collect?
       call nz,nobon       ; no, skip bonus.
       ld de,bonclr        ; success text.
       ld bc,levtxt-bonclr ; length of string.
       call 8252           ; display text.
       jp rstart           ; restart level.

adbon  ld de,bontx1        ; success text.
       ld bc,bontx2-bontx1 ; length of string.
       call 8252           ; display text.
       ld b,50             ; one second.
       call delay          ; call delay routine.
       ld b,200            ; iterations required.
adbon0 push bc             ; outer loop.
       ld b,1              ; total to add.
       ld hl,score+5       ; units column.
       call uscor          ; up the score.
       call dscor          ; show score.
       call random         ; random number.
       and 248             ; mask away the border colour bits.
       out (254),a         ; send to speaker.
       pop bc              ; delay length.
       push bc             ; replace on stack.
adbon1 djnz adbon1         ; decrement until zero.
       pop bc              ; retrieve loop counter from stack.
       djnz adbon0         ; and repeat.
       ld b,50             ; one second.
       jp delay            ; call delay routine.
nobon  ld de,bontx2        ; failure text.
       ld bc,bonclr-bontx2 ; length of string.
       call 8252           ; display text.
       ld b,100            ; two seconds.
       jp delay            ; call delay routine.

; Process sprites.

pspr   ld ix,newtab+SPRSIZ ; sprite table.
       ld b,NUMSPR-1       ; table positions to search.
pspr0  push bc             ; store sprite counter.
       ld a,(ix)           ; switched on/off.
       inc a               ; is it on?
       call nz,pspr1       ; yes, slot is used.
       ld de,SPRSIZ        ; size of sprite table.
       add ix,de           ; point to next slot.
       pop bc              ; restore counter.
       djnz pspr0          ; repeat for all slots.
       ret
pspr1  dec a               ; back to previous value.
       cp SPAWN+4          ; spawn sprite?
       jr c,pspaw          ; process that.
       cp FRUIT            ; piece of fruit?
       jp c,pfrt           ; yes, fall.
       cp BAD              ; bad ingredient?
       jp c,pfish          ; yes, manipulate it.
       cp WASP+8           ; wasp sprite?
       jp c,pwasp          ; process that.
       cp BAT+8            ; other nasty?
       jp c,pbat           ; yes, manipulate it.
       cp BIRD+8           ; bird?
       jp c,pbird          ; yes, manipulate it.
       cp DRAG+8           ; homing sprite?
       jp c,home           ; home in on player.
       cp FUZZ+4           ; fuzzy sprite?
       jp c,pfuzz          ; bounce him around.
       cp ROBOT+4          ; robot sprite?
       jp c,probo          ; bounce that around.
       cp STAR+4           ; star?
       jp c,pstar          ; animate and wait to be collected.
       cp PICKUP+4         ; pick-ups?
       jp c,ppick          ; do that processing.

; If we arrived here it must be an explosion.

       ld a,3              ; sound to play.
       ld (sndtyp),a       ; set the sound type.
       call random         ; random number generator.
       and 5               ; green and blue bits random.
       or 66               ; set red and bright bits.
;       ld c,a              ; colour is red, magenta, yellow or white.
       call cspr           ; colour the sprite.
       inc (ix+1)          ; frame counter.
       ld a,(ix+1)         ; get value.
       and 1               ; only move frame along every 2nd frame.
       ret nz              ; not time to advance the frame yet.
       ld a,(ix)           ; sprite type.
       inc a               ; advance frame.
       cp LASTEX           ; beyond the last sprite frame?
       jp nc,kspr          ; yes, kill this sprite off now.
       ld (ix),a           ; next frame.
       ret


; Process spawning sprite.

pspaw  call random         ; random colour.
       and 67              ; 0 - 3.
       add a,3             ; magenta to yellow.
       call cspr           ; colour this sprite.
       inc (ix+1)          ; increment timer.
       ld a,(ix+1)         ; timer value.
       and 31              ; ignore highest bits.
       jr z,pspaw0         ; time is up, generate something.
       ld a,(ix)           ; sprite type.
       inc a               ; next one.
       and SPAWN+3         ; loop round at the end.
       ld (ix),a           ; new sprite.
       ret
pspaw0 call getlev         ; level number.
       rlca                ; multiply by 8.
       rlca
       rlca
       ld b,a              ; store in b register.
       call random         ; random number.
       and 7               ; range 0 - 7.
       add a,b             ; add level displacement.
       ld hl,ranspr        ; random sprite table.
       call addac          ; get position in table.
       ld a,(hl)           ; fetch sprite there.
       ld (ix),a           ; set sprite type.
       call random         ; random direction.
       and 3               ; 2 flags only.
       ld (ix+1),a         ; set direction.
       ret
pwasp  ld a,70             ; bright yellow.
       call cspr           ; colour the sprite.
       ld a,(ix+1)         ; direction.
       cp 4                ; stationary bits set?
       jp nc,pstil         ; yes, stay still for a bit.
       rra                 ; test left/right flag.
       jr c,pwaspl         ; go left.
pwaspr ld b,WASPR          ; image facing right.
       ld a,(ix+3)         ; y coordinate.
       cp 239              ; at right edge?
       jr nc,pwaspy        ; yes, flip y direction.
       add a,2             ; move right.
       ld (ix+3),a         ; new position.
       jp pwaspv           ; now do vertical.
pwaspl ld b,WASP           ; image facing left.
       ld a,(ix+3)         ; y coordinate.
       cp 2                ; at left edge?
       jr c,pwaspy         ; yes, flip y direction.
       sub 2               ; move left.
       ld (ix+3),a         ; new position.
pwaspv ld a,(ix)           ; sprite type.
       inc a               ; next one.
       and 3               ; loop round at the end.
       or b                ; base sprite.
       ld (ix),a           ; new sprite.
       ld a,(ix+1)         ; direction.
       and 2               ; test up/down flag.
       jr nz,pwaspu        ; go up.
pwaspd ld a,(ix+2)         ; x coordinate.
       cp 159              ; at bottom edge?
       jr nc,pwaspx        ; yes, flip x direction.
       add a,2             ; move down.
       ld (ix+2),a         ; new position.
       call stat           ; stationary decision.
       jp nchk             ; nasty collision check.
pwaspu ld a,(ix+2)         ; y coordinate.
       cp 2                ; at top edge?
       jr c,pwaspx         ; yes, flip x direction.
       sub 2               ; move left.
       ld (ix+2),a         ; new position.
       call stat           ; stationary decision.
       jp nchk             ; nasty collision check.
pwaspy ld a,(ix+1)         ; direction.
       xor 1               ; toggle.
       ld (ix+1),a         ; set new direction.
       jp pwaspv           ; now do the vertical.
pwaspx ld a,(ix+1)         ; direction.
       xor 2               ; toggle.
       ld (ix+1),a         ; set new direction.
       ret

; Process the fish.

pfish  sub APPLE           ; get fruit number 0 -7.
       ld e,a              ; displacement in e.
       ld d,0              ; zeroise high byte of de.
       ld hl,frtcol        ; fruit colours.
       add hl,de           ; point to colour.
       ld a,(hl)           ; get colour.
       call cspr           ; colour the fruit.
       ld a,(ix+1)         ; direction/counter byte.
       add a,4             ; use bits d2-d7 as a counter.
       ld b,a              ; result in b.
       jp c,pfishs         ; end of cycle, seek demi-john.
       ld a,(gfram)        ; game frame.
       and 3               ; 3/4 frames.
       jr nz,pfish0        ; only update on 4th frame.
       ld (ix+1),b         ; new value.
pfish0 ld a,(ix+1)         ; direction/counter byte.
       rra                 ; test left/right flag.
       jr c,pfishl         ; go left.
pfishr ld a,(ix+3)         ; y coordinate.
       cp 239              ; at right edge?
       jr nc,pfishy        ; yes, flip y direction.
       add a,2             ; move right.
       ld (ix+3),a         ; new position.
       jp pfishv           ; now do vertical.
pfishl ld a,(ix+3)         ; y coordinate.
       cp 2                ; at left edge?
       jr c,pfishy         ; yes, flip y direction.
       sub 2               ; move left.
       ld (ix+3),a         ; new position.
pfishv ld a,(ix+1)         ; direction.
       and 2               ; test up/down flag.
       jr nz,pfishu        ; go up.
pfishd ld a,(ix+1)         ; get counter again.
       cp 192              ; reaching time to drop into demi-john?
       jr nc,pfishx        ; yes, only go up.
       ld a,(ix+2)         ; x coordinate.
       cp 159              ; at bottom edge?
       jr nc,pfishx        ; yes, flip x direction.
       add a,2             ; move down.
       ld (ix+2),a         ; new position.
       call stat           ; stationary decision.
       jp nchk             ; nasty collision check.
pfishu ld a,(ix+2)         ; y coordinate.
       cp 2                ; at top edge?
       jr c,pfishx         ; yes, flip x direction.
       sub 2               ; move left.
       ld (ix+2),a         ; new position.
       call stat           ; stationary decision.
       jp nchk             ; nasty collision check.
pfishy ld a,(ix+1)         ; direction.
       xor 1               ; toggle.
       ld (ix+1),a         ; set new direction.
       jp pfishv           ; now do the vertical.
pfishx ld a,(ix+1)         ; direction.
       xor 2               ; toggle.
       ld (ix+1),a         ; set new direction.
       ret
pfishf ld a,120            ; exact coordinate.
       ld (ix+3),a         ; set y coordinate.
       ld a,(ix+2)         ; x coordinate.
       cp 159              ; at bottom edge?
       jr nc,pfishb        ; yes, remove juice.
       add a,2             ; move down.
       ld (ix+2),a         ; new position.
       jp nchk             ; nasty collision check.
pfishb ld a,(nasty)        ; nasties in the wine.
       add a,4             ; increase nasty level.
       ld (nasty),a        ; new amount.
       jp kspr             ; kill the sprite.
pfishs ld a,(ix+3)         ; y position.
       cp 120              ; in centre of screen?
       jr z,pfishf         ; yes, fall into centre.
       jr c,pfshsr         ; fish is to left, move right.
       dec (ix+3)          ; move left.
       jp nchk             ; nasty collision check.
pfshsr inc (ix+3)          ; move right.
       jp nchk             ; nasty collision check.


; Home in on player.

home   ld a,68             ; bright green.
       call cspr           ; colour the sprite.
       call random         ; random number.
       rra                 ; every other frame.
       jr c,home0          ; no vertical move this time.
       ld a,(newtab+2)     ; player x integer.
       cp (ix+2)           ; compare sprite position.
       jr z,home0          ; both coords the same.
       jr c,home1          ; sprite is lower.
       inc (ix+2)          ; sprite is too high, move down one pixel.
home0  ld a,(newtab+3)     ; player y integer.
       cp (ix+3)           ; compare sprite position.
       jp z,home3          ; both coords are the same.
       jr c,home2          ; sprite is to right.
       inc (ix+3)          ; sprite is to left, move right one pixel.
       ld b,DRAGR          ; basic sprite type for dragon right.
       jr home4
home2  dec (ix+3)          ; move left.
       ld b,DRAG           ; basic sprite type for dragon left.
       jr home4
home3  ld a,(ix)           ; sprite type.
       and 252             ; remove frame bits.
       ld b,a              ; basic sprite type in b register.

; Animate sprite.

home4  ld a,(gfram)        ; game frame.
       and 3               ; every 4th frame.
       jp nz,nchk          ; only update on 4th frame.
       ld a,(ix)           ; sprite type.
       inc a               ; next frame.
       and 3               ; keep to 4 frames.
       add a,b             ; add new frame bits.
       ld (ix),a           ; new sprite.
       jp nchk             ; nasty collision check.

home1  dec (ix+2)          ; move up.
       jr home0            ; now do horizontal.

; Process still wasp.

pstil  sub 4               ; decrement counter.
       ld (ix+1),a         ; new value.
       call animsp         ; animate wasp.
       jp nchk             ; nasty collision check.

stat   call random         ; random number generator.
       cp 129              ; one in 256 chance.
       ret nz              ; keep moving.
       ld a,(ix+1)         ; get direction and stationary counter.
       add a,128           ; stop still for a while.
       ld (ix+1),a         ; put value into table.
       ret

; Process bats code.

pbat   ld a,67             ; bright magenta.
       call cspr           ; colour the sprite.
       ld a,(ix)           ; sprite type.
       and 252             ; mask away frames to get basic type.
       ld b,a              ; store in b for now.
       ld a,(gfram)        ; current game frame.
       defb 221,133        ; undocumented instruction: add a,ixl.
       defb 221,133        ; combine table address with frame to
                           ; ensure each sprite movement pattern
                           ; is slightly different.
       ld (dragt),a        ; store it for use later.
       and 24              ; time to move vertically only?
       jr z,pbata          ; skip horizontal.
       ld a,(ix+1)         ; direction.
       rra                 ; test left/right flag.
       jr c,pbatl          ; go left.
pbatr  ld b,BATR           ; moving right.
       ld a,(ix+3)         ; y coordinate.
       cp 239              ; at right edge?
       jp nc,pbaty         ; yes, flip y direction.
       ld a,(ix+3)         ; y coordinate.
       add a,2             ; move right.
       ld (ix+3),a         ; new position.
       jp pbata            ; now do animation and vertical.
pbatl  ld b,BAT            ; images facing left.
       ld a,(ix+3)         ; y coordinate.
       cp 2                ; at left edge?
       jp c,pbaty          ; yes, flip y direction.
       ld a,(ix+3)         ; y coordinate.
       sub 2               ; move left.
       ld (ix+3),a         ; new position.
pbata  ld a,(dragt)        ; game frame.
       and 3               ; time to animate?
       jp nz,pbatv         ; no, just do the vertical.
       ld a,(ix)           ; sprite type.
       inc a               ; next one.
       and 3               ; only 4 frame in animation.
       add a,b             ; combine with type again.
       ld (ix),a           ; new sprite.
pbatv  ld a,(dragt)        ; current frame.
       and 24              ; time to move vertically only?
       cp 16               ; check phase.
       jp z,nchk           ; skip vertical.
       ld a,(ix+1)         ; direction.
       and 2               ; test up/down flag.
       jr nz,pbatu         ; go up.
pbatd  ld a,(ix+2)         ; x coordinate.
       cp 159              ; at bottom edge?
       jr nc,pbatx         ; yes, flip x direction.
       ld a,(ix+2)         ; x coordinate.
       add a,2             ; move down.
       ld (ix+2),a         ; new position.
       jp nchk             ; nasty collision check.
pbatu  ld a,(ix+2)         ; y coordinate.
       cp 35               ; at top edge?
       jr c,pbatx          ; yes, flip x direction.
       ld a,(ix+2)         ; x coordinate.
       sub 2               ; move up.
       ld (ix+2),a         ; new position.
       jp nchk             ; nasty collision check.
pbaty  ld a,(ix+1)         ; direction.
       xor 1               ; toggle.
       ld (ix+1),a         ; set new direction.
       jp pbata            ; now do animation then the vertical.
pbatx  ld a,(ix+1)         ; direction.
       xor 2               ; toggle.
       ld (ix+1),a         ; set new direction.
       jp nchk             ; nasty collisions.


; Process bird sprite.

pbird  ld a,71             ; bright white.
       call cspr           ; colour the sprite.
       ld a,(ix+1)         ; solely vertical movement flag.
       and 4               ; moving up?
       jp nz,pbirdu        ; yes, move up slowly.
       ld a,(ix+1)         ; other vertical flag.
       and 2               ; going down or flying straight?
       jr nz,pbirdd        ; going down slowly.

; Not gliding down, so animate slowly.

       call z,anims4       ; animate bird every 4 frames.
       call random         ; random number 0 - 255.
       cp 252              ; chance of staying on line.
       jr c,pbirdh         ; stay on this line.
       set 1,(ix+1)        ; set gliding down slowly flag.

; Horizontal movement for bird.

pbirdh ld a,(ix+1)         ; bird direction.
       rra                 ; left/right flag.
       ld a,(ix+3)         ; bird y coordinate.
       jr c,pbirdr         ; moving right.
pbirdl cp 2                ; as far left as possible?
       jr c,pbird1         ; yes, go right next.
       sub 2               ; left 2 pixels.
       ld (ix+3),a         ; new position.
       ld a,(ix)           ; sprite type.
       and 3               ; remember frame.
       add a,BIRD          ; bird left images.
       ld (ix),a           ; new frame.
       ret
pbirdr cp 239              ; as far right as possible?
       jr nc,pbird2        ; yes, go left next.
       add a,2             ; right 2 pixels.
       ld (ix+3),a         ; new position.
       ld a,(ix)           ; sprite type.
       and 3               ; remember frame.
       add a,BIRDR         ; bird right images.
       ld (ix),a           ; new frame.
       jp nchk             ; nasty collision check.
pbird1 ld (ix),BIRDR       ; new sprite type.
       set 0,(ix+1)        ; move right next.
       jp nchk             ; nasty collision check.
pbird2 ld (ix),BIRD        ; new sprite type.
       res 0,(ix+1)        ; move left next.
       jp nchk             ; nasty collision check.

pbirdd ld a,(ix+2)         ; bird's x position.
       cp 128              ; at bottom extent of area?
       jr nc,pbirdt        ; yes, back to top.
       inc (ix+2)          ; glide down.
       call random         ; random number generator.
       cp 248              ; chance of straightening up.
       jr c,pbirdh         ; stay gliding.
       res 1,(ix+1)        ; clear gliding down slowly flag.
       jr pbirdh           ; horizontal movement next.
pbirdt ld (ix+1),4         ; straight up now.
       jp nchk             ; nasty collision check.
pbirdu ld a,(ix+2)         ; x coordinate.
       cp 2                ; at top already?
       jr c,pbirdm         ; yes, need new movement.
       sub 2               ; move up.
       ld (ix+2),a         ; set new position.
       call random         ; random number.
       and 31              ; chance of new direction.
       jr z,pbirdm         ; choose new direction.
       call animsp         ; animate sprite.
       jp nchk             ; nasty collision check.
pbirdm call random         ; random number.
       and 3               ; just want left/right and slow vertical flags.
       ld (ix+1),a         ; new direction flag.
       jp nchk             ; nasty collision check.
anims4 ld a,(gfram)        ; game frame.
       and 3               ; every 4th frame.
       ret nz              ; don't animate 3 out of every 4 frames.
animsp ld a,(ix)           ; sprite type.
       ld b,a              ; store in b register.
       and 252             ; mask away frame bits.
       ld c,a              ; basic sprite type in c.
       ld a,b              ; restore type.
       inc a               ; next frame.
       and 3               ; only 4 frames.
       add a,c             ; add type.
       ld (ix),a           ; new sprite type and frame.
       ret


; Process pick-ups.

ppick  ld a,(gfram)        ; frame number.
       and 3               ; 4 frames of colour.
       add a,2             ; red to cyan.
       call cspr           ; colour this sprite.
       ld a,(ix+2)         ; x position.
       cp 160              ; at base of screen already?
       jr nc,ppick0        ; yes, no moves required.
       inc (ix+2)          ; down a pixel.
ppick0 ld c,(ix+2)         ; sprite x position.
       ld b,(ix+3)         ; sprite y coord.
       ld hl,(newtab+2)    ; player position.
       call colc16         ; clipped collision detection.
       ret nc              ; no collision, no problem.
       call random         ; random number.
       and 31              ; 0 - 31 for time.
       add a,64            ; minimum time before another item appears.
       ld (pflag),a        ; set the pick-up counter.
       ld a,(ix)           ; sprite type.
       cp GEM              ; star bonus?
       jp z,tstar          ; turn all nasties to stars.
       cp EXLIFE           ; extra life?
       jp z,exlif          ; award an extra life.
energy ld a,2              ; sound to play.
       ld (sndtyp),a       ; set the sound type.
       ld a,(increm)       ; shield increment.
       add a,6             ; add a few pixels to energy.
       ld (increm),a       ; store increment.
       jp kspr             ; kill off this sprite.

; Process stars.

pstar  call animsp         ; animate this sprite.
       ld c,(ix+2)         ; sprite x position.
       ld b,(ix+3)         ; sprite y coord.
       ld hl,(newtab+2)    ; player position.
       call colc16         ; clipped collision detection.
       jr nc,pstar0        ; no collision, no problem.
       ld b,5              ; bonus points to award.
       ld hl,score+4       ; tens column.
       call uscor          ; increase the score.
       call dscor          ; display new value.
       ld a,2              ; sound to play.
       ld (sndtyp),a       ; set the sound type.
       jp kspr             ; kill off this sprite.

; Stars use (ix+1) as a counter.
; When it reaches zero they explode.

pstar0 dec (ix+1)          ; decrement star counter.
       jp z,xspr           ; time up, combust.
       ld a,(ix+1)         ; current value.
       cp 55               ; about to turn nasty?
       jr nc,pstar1        ; no, plenty of time yet.

; Sprite starts to shake and glow red as timer nears zero.

       ld a,(gfram)        ; frame.
       and 4               ; bit determines colour.
       add a,66            ; yellow or red.
       call cspr           ; colour this sprite.
       call random         ; random number.
       rra                 ; rotate bit into carry.
       ld b,a              ; store rest of random number in b.
       jr c,pstaru         ; move up.
       ld a,(ix+2)         ; x coordinate.
       cp 160              ; too close to screen edge?
       jr nc,pstarh        ; yes, don't bother with vertical then.
       inc (ix+2)          ; move sprite down.
       jr pstarh           ; now do horizontal coordinate.
pstaru ld a,(ix+2)         ; x coordinate.
       and a               ; too close to screen edge?
       jr z,pstarh         ; yes, just do the horizontal shaking.
       dec (ix+2)          ; move sprite up.
pstarh rr b                ; rotate next bit out of random number.
       jr c,pstarl         ; move left.
       ld a,(ix+3)         ; y coordinate.
       cp 240              ; too close to screen edge?
       ret nc              ; yes, don't bother moving then.
       inc (ix+3)          ; move sprite right.
       ret
pstarl ld a,(ix+3)         ; y coordinate.
       and a               ; too close to screen edge?
       ret z               ; yes, can't move.
       dec (ix+3)          ; move sprite left.
       ret
pstar1 ld a,(gfram)        ; frame.
       and 1               ; lowest bit determines colour.
       add a,70            ; yellow or white.
       jp cspr             ; colour this sprite.

; Process the robot.

probo  ld a,69             ; bright cyan.
       call cspr           ; colour the sprite.
       ld a,(ix+1)         ; direction.
       cp 4                ; stationary bits set?
       jp nc,probst        ; yes, stay still for a bit.
       rra                 ; test left/right flag.
       jr c,probol         ; go left.
probor ld a,(ix+3)         ; y coordinate.
       cp 239              ; at right edge?
       jr nc,proboy        ; yes, flip y direction.
       add a,2             ; move right.
       ld (ix+3),a         ; new position.
       jp probov           ; now do vertical.
probol ld a,(ix+3)         ; y coordinate.
       cp 2                ; at left edge?
       jr c,proboy         ; yes, flip y direction.
       sub 2               ; move left.
       ld (ix+3),a         ; new position.
probov call anims4         ; animate sprite every 4th frame.
       ld a,(ix+1)         ; direction.
       and 2               ; test up/down flag.
       jr nz,probou        ; go up.
probod ld a,(ix+2)         ; x coordinate.
       cp 159              ; at bottom edge?
       jr nc,probox        ; yes, flip x direction.
       add a,2             ; move down.
       ld (ix+2),a         ; new position.
       call stat           ; stationary decision.
       jp nchk             ; nasty collision check.
probou ld a,(ix+2)         ; y coordinate.
       cp 2                ; at top edge?
       jr c,probox         ; yes, flip x direction.
       sub 2               ; move left.
       ld (ix+2),a         ; new position.
       call stat           ; stationary decision.
       jp nchk             ; nasty collision check.
proboy ld a,(ix+1)         ; direction.
       xor 1               ; toggle.
       ld (ix+1),a         ; set new direction.
       jp probov           ; now do the vertical.
probox ld a,(ix+1)         ; direction.
       xor 2               ; toggle.
       ld (ix+1),a         ; set new direction.
       ret
probst sub 4               ; decrement counter.
       ld (ix+1),a         ; new value.
       call anims4         ; animate robot.
       jp nchk             ; nasty collision check.


tstar  ld hl,newtab+SPRSIZ ; table of sprites.
       ld b,NUMSPR-1       ; table positions to search.
       ld de,SPRSIZ        ; size of sprite table.
tstar0 ld a,(hl)           ; switched on/off.
       cp SPAWN+4          ; is it spawning?
       jr c,tstar1         ; yes, turn to star.
       cp FRUIT            ; is it fruit or below?
       jr c,tstar2         ; yes, don't bother.
       cp STAR             ; is it a nasty?
       jr c,tstar1         ; yes, turn to star.
tstar2 add hl,de           ; point to next slot.
       djnz tstar0         ; repeat for all slots.
       ld a,2              ; sound to play.
       ld (sndtyp),a       ; set the sound type.
       jp kspr             ; kill off this sprite.
tstar1 ld (hl),STAR        ; make it a star.
       inc hl              ; counter byte.
       push hl             ; store pointer.
       call random         ; random number.
       pop hl              ; restore pointer position.
       and 31              ; random number in range 0 - 31.
       add a,220           ; minimum spin time for stars.
       ld (hl),a           ; start spin counter.
       dec hl              ; restore pointer.
       jp tstar2

; Fuzzies code.

pfuzz  call animsp         ; animate it.
       ld a,(ix+1)         ; direction and speed.
       ld b,a              ; store in b.
       rla                 ; x direction.
       jr c,pfuzzu         ; move up.
       rlca                ; rotate speed into lowest 3 bits.
       rlca
       rlca
       and 3               ; 0 - 3.
       add a,(ix+2)        ; add to x coordinate.
       cp 161              ; reached bottom of screen?
       jr c,pfuzzh         ; no, do horizontal.
       ld (ix+2),160       ; bottom of screen.
pfuznd call random         ; random number.
       ld (ix+1),a         ; set new direction.
       jp pfuzzc           ; now set colours.
pfuzzu rlca                ; rotate speed into lowest 3 bits.
       rlca
       rlca
       and 3               ; 0 to 3.
       ld c,a              ; put that in c register.
       ld a,(ix+2)         ; get x coordinate.
       sub c               ; subtract distance.
       jr nc,pfuzzh        ; not past top, do horizontal now.
       ld (ix+2),0         ; top of screen.
       jp pfuznd           ; choose new direction.
pfuzzh ld (ix+2),a         ; set new coordinate.
       ld a,b              ; get direction and speed.
       bit 4,b             ; test direction bit.
       jr nz,pfuzzl        ; move left.
       and 3               ; speed is in lowest 3 bits.
       inc a               ; minimum speed 1, maximum 4.
       add a,(ix+3)        ; add to y coordinate.
       cp 241              ; reached edge of screen?
       jr c,pfuzzo         ; no, done.
       ld (ix+3),240       ; edge of screen.
       jp pfuznd           ; random direction.
pfuzzl and 3               ; speed is in lowest 3 bits.
       inc a               ; minimum speed 1, maximum 4.
       ld c,a              ; put that in c register.
       ld a,(ix+3)         ; get y coordinate.
       sub c               ; subtract distance.
       jr nc,pfuzzo        ; not past top, done.
       ld (ix+3),0         ; edge of screen.
       jp pfuznd           ; choose new direction.
pfuzzo ld (ix+3),a         ; set new y coordinate.
pfuzzc ld a,66             ; bright green.
       call cspr           ; colour the sprite.
       jp nchk             ; nasty collision detection.

dragt  defb 0              ; bat timer.

pfrt   sub APPLE           ; get fruit number 0 - 7.
       ld e,a              ; displacement in e.
       ld d,0              ; zeroise high byte of de.
       ld hl,frtcol        ; fruit colours.
       add hl,de           ; point to colour.
       ld a,(hl)           ; get colour.
       call cspr           ; colour the fruit.
       inc (ix+2)          ; move down 1 pixel.
       ld a,(ix+1)         ; dropped-by-player flag.
       and a               ; has it been set?
       jp nz,cdemi         ; yes, check for demi-john.

; Uncollected fruit continues here.

       ld a,(ix+2)         ; fetch new coordinate.
       cp 160              ; past bottom of screen?
       jp nc,xspr          ; yes, kill sprite.
       ld a,(dflag)        ; player dead flag.
       and a               ; has he been killed?
       ret nz              ; yes, can't collect fruit.
       ld c,(ix+2)         ; nasty sprite x.
       ld b,(ix+3)         ; sprite y coord.
       ld hl,(newtab+2)    ; player position.
       call colc16         ; clipped collision detection.
       ret nc              ; no collision, no problem.
       ld hl,fbuff         ; fruit buffer.
       ld a,(fnum)         ; number collected so far.
       cp 8                ; reached maximum?
       ret z               ; can't pick this up then.
       ld e,a              ; displacement to address.
       ld d,0              ; no high byte needed.
       add hl,de           ; point to spot in inventory.
       ld a,(ix)           ; get type.
       ld (hl),a           ; store fruit.
       ld hl,fnum          ; items collected.
       inc (hl)            ; add to tally.
       ld a,2              ; sound to play.
       ld (sndtyp),a       ; set the sound type.
kspr   ld (ix),255         ; set all bits.
       ret
xspr   ld (ix),EXPLOD      ; turn sprite into an explosion.
       ld (ix+1),0         ; zeroise second byte.
       ret

; Dropped fruit.  Check if fruit has hit level of demi-john.

cdemi  ld hl,juice         ; juice level.
       ld a,169            ; bottom line minus sprite height.
       sub (hl)            ; get coordinate to check.
       cp (ix+2)           ; compare fruit x.
       ret nc              ; not hit the must yet.

; Hit the level, top up the juice.

       ld a,(fruit)        ; fruit to go in.
       add a,2             ; add to total.
       ld (fruit),a        ; store new total.

; Bonus fruit check.

       ld a,(frtreq)       ; bonus fruit type.
       cp (ix)             ; sprite type.
       call z,bonus        ; they match, do bonus stuff.
       ld b,5              ; amount to add.
       ld hl,score+5       ; units column.
       call uscor          ; up the score.
       call dscor          ; display player's score.
       jr kspr             ; don't forget to kill sprite.

; Bonus fruit added.

bonus  ld a,(frtqty)       ; fruit still required.
       and a               ; any remaining?
       ret z               ; no, we've already earned the bonus.
       dec a               ; one less.
       ld (frtqty),a       ; store new quantity.
       jp dbon             ; display bonus fruit remaining.

demij  ld hl,fruit         ; juice to go in.
       ld a,(hl)           ; amount by which to increase.
       and a               ; any to top up?
       ret z               ; no, nothing going in.
       dec (hl)            ; reduce amount to go in.
       ld hl,juice         ; juice level.
       inc (hl)            ; up the level.

; Drop through to juice display/deletion routine.

; Display juice level.

djuic  ld hl,juice         ; juice level.
       ld a,184            ; bottom coordinate.
       sub (hl)            ; number of pixels up the screen.
       ld (dispx),a        ; that's our vertical position.
       ld a,104            ; pixels across.
       ld (dispy),a        ; set display position.
       call scadd          ; find screen address.
       ld a,(hl)           ; get byte there.
       cpl                 ; combine with image we want.
       ld (hl),a           ; write to screen.
       inc l               ; next column.
       ld a,(hl)           ; get byte there.
       cpl                 ; reverse all bits.
       ld (hl),a           ; write image to screen.
       inc l               ; next column.
       ld a,(hl)           ; get byte there.
       cpl                 ; reverse all bits.
       ld (hl),a           ; write image to screen.
       inc l               ; next column.
       ld a,(hl)           ; get byte there.
       cpl                 ; reverse all bits.
       ld (hl),a           ; write image to screen.
       inc l               ; next column.
       ld a,(hl)           ; get byte there.
       cpl                 ; reverse all bits.
       ld (hl),a           ; write image to screen.
       inc l               ; next column.
       ld a,(hl)           ; get byte there.
       cpl                 ; merge with image.
       ld (hl),a           ; write image to screen.
       ret

; Check for nasty things in wine.

cnasty ld hl,nasty         ; nasty flag.
       ld a,(hl)           ; get value.
       and a               ; is it set?
       ret z               ; no, nothing nasty in wine.

; Something nasty fell in.

       dec (hl)            ; one fewer to drop.
       ld a,(juice)        ; juice level.
       and a               ; is there any in?
       ret z               ; no.
       call djuic          ; display juice level.
       ld hl,juice         ; juice level.
       dec (hl)            ; decrease juice level.
       ret

juice  defb 0              ; level of juice.
fruit  defb 0              ; fruit juice to go in.
nasty  defb 0              ; something nasty when non-zero.

; Fruit buffer.
; Store carried fruit.

fbuff  defw 0,0,0,0
fnum   defb 0              ; pieces collected.
dropc  defb 0              ; drop counter.

; Drop fruit.

drop   ld a,(dropc)        ; drop counter.
       and a               ; is it set?
       jr nz,countd        ; not time to drop, decremnt counter.
       ld a,(newtab+2)     ; player x integer.
       cp 112              ; above the demi-john?
       ret nc              ; no, don't drop anything.
       ld a,(newtab+3)     ; player's horizontal position.
       cp 112              ; to left of demi-john?
       ret c               ; yes, nothing to do.
       cp 136              ; to right of demi-john?
       ret nc              ; yes, nothing to do.

; Spawn fruit over demi-john.

nfrt   ld a,(fnum)         ; fruit collected.
       and a               ; do we have any?
       ret z               ; nope, nothing to do.
       dec a               ; last fruit is always one position back.
       ld (fnum),a         ; store new value.
       ld hl,fbuff         ; fruit buffer.
       ld e,a              ; displacement to fruit.
       ld d,0              ; high byte.
       add hl,de           ; point to this item.
       ld c,(hl)           ; get fruit.
       ld ix,newtab+SPRSIZ ; table of sprites.
       ld b,NUMSPR-1       ; table positions to search.
       ld de,SPRSIZ        ; size of sprite table.
nfrt0  ld a,(ix)           ; switched on/off.
       inc a               ; is it on?
       jp z,nfrt1          ; no, slot is free.
       add ix,de           ; point to next slot.
       djnz nfrt0          ; repeat for all slots.
       ret
nfrt1  ld (ix),c           ; spawn the fruit.
       ld a,(newtab+2)     ; player x integer.
       add a,16            ; down 16 pixels.
       ld (ix+2),a         ; set x position.
       ld (ix+3),120       ; middle of screen.
       ld (ix+1),a         ; set dropped-by-layer flag.
       ld a,16             ; time until next item is dropped.
       ld (dropc),a        ; set the drop counter.
       ret
countd dec a               ; decrement counter.
       ld (dropc),a        ; set new counter value.
       ret

; Spawn new sprites.

spawn  ld hl,spchan        ; spawn chances for each level.
       call getlev         ; number of present level.
       call addac          ; find element in table.
       ld b,(hl)           ; chance of spawning.
       call random         ; random number generator.
       cp b                ; time to spawn?
       ret nc              ; no, not yet.
       ld ix,newtab+SPRSIZ ; table of sprites.
       call getlev         ; current level number.
       ld hl,numspr        ; number of sprites per level.
       call addac          ; point to number we want.
       ld b,(hl)           ; get loop counter.
       ld de,SPRSIZ        ; size of sprite table.
spawn0 ld a,(ix)           ; switched on/off.
       inc a               ; is it on?
       jp z,spawn1         ; no, slot is free.
       add ix,de           ; point to next slot.
       djnz spawn0         ; repeat for all slots.
       ret

spawn1 call random         ; get random coordinate.
       and 127             ; in range for x.
       ld (ix+2),a         ; set x.
       call random         ; random, coordinate.
       cp 241              ; in range?
       ret nc              ; no, don't spawn then.
       ld (ix+3),a         ; set sprite y.
       ld (ix),SPAWN       ; set spawn sprite.
       ld (ix+1),0         ; reset timer.
       ret

nchk   ld a,(dflag)        ; player dead flag.
       and a               ; has he been killed?
       ret nz              ; yes, don't need a collision check.
       ld c,(ix+2)         ; nasty sprite x.
       ld b,(ix+3)         ; sprite y coord.
       ld hl,(newtab+2)    ; player position.
       call colc16         ; clipped collision detection.
       ret nc              ; no collision, no problem.
       call xspr           ; explode the sprite.
       ld a,(levnum)       ; current level.
       add a,4             ; minimum depletion of 4.
       cp 10               ; reached maximum?
       jr c,nchk0          ; no, that's the energy to remove.
       ld a,10             ; maximum energy loss per collision.
nchk0  ld b,a              ; energy to lose.
       ld a,(deplet)       ; shield depletion count.
       add a,b             ; add total pixels to remove from bar.
       ld (deplet),a       ; set new tally.
       ret

increm defb 0              ; energy goes up when non-zero.
deplet defb 0              ; energy goes down when non-zero.
enbar  defw 0              ; address of energy graphic.
enbit  defb 1              ; next bit to remove.
pantxt defb 16,5           ; energy bar colour.
       defb 143,143,143    ; solid blocks.
       defb 143,16,7
dflag  defb 0              ; flag is non-zero when player is dead.

; Pick-ups.

pick   ld a,(gfram)        ; game's main frame counter.
       and 7               ; every 8 frames.
       ret nz              ; only check for pick-ups on 8th frame.
       ld a,(pflag)        ; pick-up flag.
       and a               ; is pick-up on the screen?
       ret z               ; yes, nothing else to do.
       dec a               ; decrement timer.
       ld (pflag),a        ; new setting.
       ret nz              ; not yet time to spawn.

; Every 5 levels we drop an extra life on to the screen.

       ld a,(levlif)       ; levels before next life appears.
       and a               ; have we reached zero?
       jr z,pickex         ; generate extra life pick-up.
       call random         ; random number.
       and 3               ; range 0-4.
       ld hl,bonlst        ; random bonus list.
       call addac          ; point to random bonus.
       ld c,(hl)           ; type of pick-up to generate.
pick0  call npick          ; generate new pick-up.
       ld (pflag),a        ; set the pick-up counter.
       ret
exlif  ld a,(numlif)       ; number of lives.
       cp 5                ; reached maximum?
       jp nc,energy        ; yes, just give extra energy instead.
       inc a               ; one more life.
       ld (numlif),a       ; set new lives total.
       call dlives         ; display lives remaining.
       ld a,2              ; sound to play.
       ld (sndtyp),a       ; set the sound type.
       jp kspr             ; kill off the bonus sprite.
pickex ld a,5              ; levels before another life appears.
       ld (levlif),a       ; set new tally.
       ld c,EXLIFE         ; extra life bonus.
       jr pick0            ; generate bonus.


; Generate new pick-up.

npick  ld ix,newtab+SPRSIZ ; table of sprites.
       ld b,NUMSPR-1       ; table positions to search.
       ld de,SPRSIZ        ; size of sprite table.
npick0 ld a,(ix)           ; switched on/off.
       inc a               ; is it on?
       jp z,npick1         ; no, slot is free.
       add ix,de           ; point to next slot.
       djnz npick0         ; repeat for all slots.

; Failed to generate an item, try again later.

       ld a,30             ; loops before we try again.
       ret

npick1 ld (ix),c           ; set up the type.
       ld (ix+2),0         ; x coordinate - top of screen.
       call random         ; random number.
       and 63              ; random position.
       ld b,a              ; store position.
       ld a,(newtab+3)     ; player's y coordinate.
       add a,136           ; 120 is central, so flip that to left side.
       and 128             ; 128 if player left, 0 if player on right.
       add a,b             ; add random element.
       add a,24            ; final adjustment.
       ld (ix+3),a         ; set y position.
       xor a               ; success - reset pflag to zero.
       ret


pflag  defb 0              ; pick up flag.
score  defb 48,48,48,49,49,48


; Random numbers code.

; Pseudo-random number generator.
; Steps a pointer through the ROM (held in seed), returning the contents
; of the byte at that location.

random ld hl,(seed)        ; pointer to ROM.
       ld a,(seed2)        ; previous seed.
       add a,(hl)          ; combine with number from location.
       ld (seed2),a        ; second seed.
       res 5,h             ; stay within first 8K of ROM.
       xor l               ; more randomness.
       inc hl              ; increment pointer.
       ld (seed),hl        ; new position.
       ret
seed   defw 999            ; random number seed.
seed2  defb 0

; Colour a sprite.

cspr   ld c,a              ; set up colour in c register.
       call ataddp         ; get pixel attribute address.
cspr3  ld a,(ix+3)         ; y coordinate.
       and 7               ; is sprite straddling character cells?
       jp z,cspr2          ; no, just 2 cells wide.
       ld (hl),c           ; set the colour.
       inc l               ; next cell.
       ld (hl),c           ; set the colour.
       inc l               ; next cell.
       ld (hl),c           ; set the colour.
       ld de,30            ; distance to next line down.
       add hl,de           ; next row.
       ld (hl),c           ; set the colour.
       inc l               ; next cell.
       ld (hl),c           ; set the colour.
       inc l               ; next cell.
       ld (hl),c           ; set the colour.
       ld a,(ix+2)         ; x position.
       and 7               ; is sprite straddling character cells?
       ret z               ; no, job done.
       add hl,de           ; next row.
       ld (hl),c           ; set the colour.
       inc l               ; next cell.
       ld (hl),c           ; set the colour.
       inc l               ; next cell.
       ld (hl),c           ; set the colour.
       ret
cspr2  ld (hl),c           ; set the colour.
       inc l               ; next cell.
       ld (hl),c           ; set the colour.
       ld de,31            ; distance to next line down.
       add hl,de           ; next row.
       ld (hl),c           ; set the colour.
       inc l               ; next cell.
       ld (hl),c           ; set the colour.
       ld a,(ix+2)         ; x position.
       and 7               ; is sprite straddling character cells?
       ret z               ; no, job done.
       add hl,de           ; next row.
       ld (hl),c           ; set the colour.
       inc l               ; next cell.
       ld (hl),c           ; set the colour.
       ret

; Colours.

colour call getlev         ; level number.
       ld hl,wincol        ; wine colours.
       call addac          ; point to colours for this batch.
       ld c,(hl)           ; get colour from table.
       ld hl,23022         ; left handle of demi-john.
       ld a,4              ; glass colour.
       ld (hl),a           ; set colour.
       inc l               ; next cell.
       inc l               ; next cell.
       inc l               ; next cell.
       ld (hl),a           ; set colour.
       ld hl,23053         ; second row down.
       ld (hl),a           ; set colour.
       inc l               ; next cell.
       ld (hl),a           ; set colour.
       inc l               ; next cell.
       inc l               ; next cell.
       inc l               ; next cell.
       ld (hl),a           ; set colour.
       inc l               ; next cell.
       ld (hl),a           ; set colour.
       ld l,44             ; third row down.
       ld (hl),a           ; set colour.
       inc l               ; next cell.
       ld (hl),a           ; set colour.
       ld l,49             ; right half.
       ld (hl),a           ; set colour.
       inc l               ; next cell.
       ld (hl),a           ; set colour.
       inc l               ; next cell.
       ld (hl),a           ; set colour.
       ld l,76             ; third row down.
       ld (hl),a           ; set colour.
       ld l,83             ; right half.
       ld (hl),a           ; set colour.
       ld l,108            ; third row down.
       ld (hl),a           ; set colour.
       ld l,115            ; right half.
       ld (hl),a           ; set colour.
       ld l,140            ; third row down.
       ld (hl),a           ; set colour.
       ld l,147            ; right half.
       ld (hl),a           ; set colour.
       ld l,172            ; third row down.
       ld (hl),a           ; set colour.
       ld l,179            ; right half.
       ld (hl),a           ; set colour.
       ld l,204            ; third row down.
       ld (hl),a           ; set colour.
       ld l,211            ; right half.
       ld (hl),a           ; set colour.
       ld l,236            ; last row.
       ld b,8              ; width.
colou0 ld (hl),a           ; set colour.
       inc l               ; next cell.
       djnz colou0

; Now colour the wine from the bottom up.

       ld a,(juice)        ; juice level.
       and a               ; any in bottle?
       jp z,colou1         ; no, colour player sprite.

       ld l,205            ; position in container.
       call cline          ; colour the wine.
       ld a,(juice)        ; juice level.
       cp 8                ; reached 8 pixels?
       jp c,colou1         ; no, colour player sprite.
       call cline          ; colour the wine.
       ld a,(juice)        ; juice level.
       cp 16               ; reached this level?
       jp c,colou1         ; no, colour player sprite.
       call cline          ; colour the wine.
       ld a,(juice)        ; juice level.
       cp 24               ; at this level?
       jp c,colou1         ; no, colour player sprite.
       call cline          ; colour the wine.
       ld a,(juice)        ; juice level.
       cp 32               ; up to 32 yet?
       jp c,colou1         ; no, colour player sprite.
       call cline          ; colour the wine.

colou1 ld ix,newtab        ; sprite table.
       ld a,66             ; bright red colour.
       call cspr           ; colour the player sprite.
       ret

cline  ld b,6              ; width inside.
cline0 ld (hl),c           ; set colour.
       inc l               ; next cell.
       djnz cline0         ; repeat for whole line.
       ld de,65498         ; minus 38.
       add hl,de           ; point to next position.
       ret


; Up the score.

uscor  ld a,(hl)           ; current value of digit.
       add a,b             ; add points to this digit.
       ld (hl),a           ; place new digit back in string.
       cp '9'+1            ; more than value '9'?
       ret c               ; no - relax.
       sub 10              ; subtract 10.
       ld (hl),a           ; put new character back in string.
uscor0 dec hl              ; previous character in string.
       inc (hl)            ; up this by one.
       ld a,(hl)           ; what's the new value?
       cp '9'+1            ; gone past nine?
       ret c               ; no, scoring done.
       sub 10              ; down by ten.
       ld (hl),a           ; put it back.
       jp uscor0           ; go round again.

; Display player's score.

dscor  ld bc,1*256+8       ; 1 line up, 8 left.
       call 3545           ; set cursor position.
       ld de,score         ; score text.
       ld bc,6             ; length of string.
       jp 8252             ; set up panel.


; 16-bit mathematics routines.
; 8 bits=integer
; 7 bits=decimal places
; 1 bit sign.

sub    ld a,1              ; HL = HL - DE
       xor e
       ld e,a
add    ld a,l              ; HL = HL + DE
       and 1
       ld c,a
       ld a,e
       and 1
       xor c
       jr nz,add1
       res 0,l
       res 0,e
       add hl,de
       ld a,l
       or c
       ld l,a
       ret
add1   bit 0,c
       jr z,add3
       ex de,hl
add3   res 0,l
       res 0,e
       sbc hl,de
       jr c,add2
       ret
add2   ex de,hl
       ld hl,2
       sbc hl,de
       ret


;sin    rlca                ; double the angle.
sin    and 30              ; in range for table.
       ld e,a              ; displacement in e.
       ld d,0              ; no high byte.
       ld hl,sine          ; start of sine table.
       add hl,de           ; point to sine.
       ld e,(hl)           ; sine low.
       inc hl              ; next byte of entry.
       ld d,(hl)           ; sine high.
       ret


; Player movement.

player call joykey         ; joystick and keyboard stuff.
       rra                 ; test low bit.
       call c,rpr          ; rotate player right.
       ld a,(joyval)       ; joystick value.
       and 2               ; bit set?
       call nz,rpl         ; rotate player left.
       ld a,(joyval)       ; joystick value.
       and 8               ; bit set?
       call nz,for         ; go forward.
       ld a,(joyval)       ; joystick value.
       and 16              ; bit set?
       jp nz,fire          ; fire.
nofir0 xor a               ; fire not pressed, set flag accordingly.
       ld (firefl),a
nofire ld a,(newtab+2)     ; player x integer.
       ld h,a              ; set high byte.
       ld a,(playxf)       ; player x fraction/sign.
       ld l,a              ; set low byte.
       ld de,64            ; tiniest fraction.
       call add            ; move down slowly.
       ld a,l              ; fraction/sign result.
       ld (playxf),a       ; set fraction.
       ld a,h              ; integer result.
       call cbot           ; check bottom of screen.
       ld (newtab+2),a     ; set new player x.
       ret

; Move forwards.

for    ld a,(newtab+1)     ; angle.
       call sin            ; get sine.
       ld a,(playxf)       ; player x fraction.
       ld l,a              ; low byte.
       ld a,(newtab+2)     ; x integer.
       ld h,a              ; that's the high byte.
       push de             ; store value.
       call add            ; add the two together.
       pop de              ; restore value.
       call add            ; add the two together.
       ld a,h              ; high byte of result.
       call cbot           ; check bottom of screen.
       ld (newtab+2),a     ; set the integer.
       ld a,l              ; low byte.
       call cneg           ; check top of screen.
       ld (playxf),a       ; set the fraction and sign.
       ld a,(newtab+1)     ; angle.
       add a,8             ; quarter rotation.
       call sin            ; get sine.
       ld a,(playyf)       ; player y fraction.
       ld l,a              ; low byte.
       ld a,(newtab+3)     ; y integer.
       ld h,a              ; that's the high byte.
       push de             ; store value.
       call add            ; add the two together.
       pop de              ; restore value.
       call add            ; add the two together.
       ld a,h              ; high byte of result.
       call crig           ; check right edge of screen.
       ld (newtab+3),a     ; set the integer.
       ld a,l              ; low byte.
       call cneg           ; check left edge of screen.
       ld (playyf),a       ; set the fraction and sign.
       ret
crig   cp 241              ; reached edge?
       ret c               ; no.
       ld a,240            ; new coordinate.
       ret
cbot   cp 161              ; reached bottom?
       ret c               ; no.
       ld a,160            ; new coordinate.
       ret
cneg   rra                 ; check carry.
       ld a,l              ; restore result.
       ret nc              ; it's still positive.
       xor a               ; reset sign.
       ret
firefl defb 0              ; firing flag.

; Rotate player left/right.

rpl    ld a,(newtab+1)     ; direction.
       dec a               ; rotate left.
       and 31              ; keep within range.
       ld (newtab+1),a     ; new direction.
       rra                 ; divide by 2.
       and 15              ; keep within range.
       ld (newtab),a       ; new direction.
       ret
rpr    ld a,(newtab+1)     ; direction.
       inc a               ; rotate left.
       and 31              ; keep within range.
       ld (newtab+1),a     ; new direction.
       rra                 ; divide by 2.
       and 15              ; keep within range.
       ld (newtab),a       ; new direction.
       ret

fire   ld hl,firefl        ; Must release key between bullets.
       ld a,(hl)           ; fire already pressed flag.
       and a               ; been pressed?
       jp nz,nofire        ; yes, release before firing again.
       inc (hl)            ; set pressed flag.
       ld hl,bultab        ; bullet table.
       ld b,NUMBUL         ; number of bullets.
       ld de,BULSIZ        ; size of bullet table entry.
fire1  ld a,(hl)
       rla                 ; check leftmost bit.
       jr c,fire0          ; it's set, bullet is disabled.
       add hl,de           ; try next in table.
       djnz fire1          ; repeat until we find empty spot.
       jp nofire           ; back to main loop.
fire0  xor a               ; zero.
       ld c,a              ; zero c register as well.
       ld (hl),a           ; enable bullet.
       inc hl              ; point to angle.
       ld a,(newtab+1)     ; angle of player.
       ld (hl),a           ; fire in direction we're pointing.
       inc hl              ; point to x integer.
       push hl             ; store pointer.
       ld a,(newtab+1)     ; angle of the barrel.
       call sin            ; sine of angle.
       ld h,d              ; copy to hl.
       ld l,e
       call add            ; double hl, so sin*2.
       ld e,l              ; copy to de.
       ld d,h
       call add            ; double hl, so sin*4.
       ld e,l              ; copy to de.
       ld d,h
       ld e,l              ; copy to de.
       ld d,h
       call add            ; double hl, so sin*8.
       ld a,(newtab+2)     ; barrel's x coordinate.
       add a,7             ; middle of sprite.
       ld d,a              ; put in integer.
       ld e,0              ; blank fraction/sign.
       call add            ; combine them.
       ex de,hl            ; put into de pair.
       pop hl              ; restore pointer.
       ld a,d
       ld (hl),e           ; set up fraction/sign.
       ld (dispx),a        ; coords set up dispx.
       inc hl              ; point to x fraction.
       ld (hl),a           ; set coordinate.
       inc hl              ; point to y fraction.
       push hl             ; store pointer.
       ld a,(newtab+1)     ; angle of barrel.
       add a,8             ; quarter rotation.
       call sin            ; cosine of angle.
       ld h,d              ; copy to hl.
       ld l,e
       call add            ; double hl, so cos*2.
       ld e,l              ; copy to de.
       ld d,h
       call add            ; double hl, so cos*4.
       ld e,l              ; copy to de.
       ld d,h
       call add            ; double hl, so cos*8.
       ld a,(newtab+3)     ; barrel y.
       add a,7             ; middle of sprite.
       jr c,fire2          ; would wrap around screen.
       cp 248              ; too near right edge?
       jr nc,fire2         ; yes, stop bullet before we fire.
       ld d,a              ; put in integer.
       ld e,0              ; blank fraction/sign.
       call add            ; combine them.
       ex de,hl            ; put into de pair.
       pop hl              ; restore pointer.
       ld a,d
       ld (dispy),a        ; coords set up dispx.
       ld (hl),e           ; set fraction.
       inc hl              ; point to y integer.
       ld (hl),a           ; set coordinate.
       call plot           ; plot bullet.
       ld a,1              ; sound type 1 = fire.
       ld (sndtyp),a       ; beeper sound type.
       jp nofire           ; rejoin main loop.
fire2  pop hl              ; restore pointer to bullet.
       dec hl              ; bullet x.
       dec hl
       dec hl              ; bullet direction.
       dec hl              ; on/off flag.
       ld (hl),128         ; turn it off.
       jp nofire           ; rejoin main loop.


; Check (l, h) for collision with (c, b), cutting corners.

colc16 ld a,l              ; x coord.
       sub c               ; subtract x.
       jr nc,colc1a        ; result is positive.
       neg                 ; make negative positive.
colc1a cp 16               ; within x range?
       ret nc              ; no - they've missed.
       ld e,a              ; store difference.

       ld a,h              ; y coord.
       sub b               ; subtract y.
       jr nc,colc1b        ; result is positive.
       neg                 ; make negative positive.
colc1b cp 16               ; within y range?
       ret nc              ; no - they've missed.

       add a,e             ; add x difference.
       cp 26               ; only 5 corner pixels touching?
       ret                 ; carry set if there's a collision.



; Move player's bullets.

mvbul  ld b,NUMBUL         ; bullets to move.
       ld ix,bultab        ; bullet table address.
mvbul0 push bc             ; store counter.
       ld a,(ix)
       rla                 ; check leftmost bit.
       call nc,mvbul1      ; not set, so bullet is enabled.
       pop bc              ; retrieve counter.
       ld de,BULSIZ        ; bytes required per bullet.
       add ix,de           ; point ix at next one.
       djnz mvbul0         ; repeat for all bullets.
       ret
mvbul1 ld l,(ix+3)         ; x integer.
       ld h,(ix+5)         ; y integer.
       ld (dispx),hl       ; workspace coordinates.
       call plot           ; plot new position.
       ld a,(ix+1)         ; what is the angle?
       call sin            ; get sine of this angle.
       ld l,e              ; copy de to hl.
       ld h,d
       call add            ; double it up.
       ld d,h              ; copy hl to de.
       ld e,l
       push de             ; store de.
       call add            ; quadruple result.
       pop de              ; restore de.
       call add            ; hexuple(?) result.
       ld e,(ix+2)         ; fraction of x.
       ld d,(ix+3)         ; integer of x.
       call add            ; add the sine.
       ld (ix+2),l         ; new fraction.
       ld (ix+3),h         ; new integer.
       ld a,l              ; put x sign in accumulator for checks.
       rra                 ; has it gone negative?
       jr c,kilbul         ; yes, bullet off screen.
       ld a,h              ; put x integer in accumulator now.
       cp 175              ; more than 175?
       jr nc,kilbul        ; yes, so not on screen.
       ld a,(ix+1)         ; what is the angle?
       add a,8             ; quarter turn for cosine.
       call sin            ; get cosine of this angle.
       ld l,e              ; copy de to hl.
       ld h,d
       call add            ; double it up.
       ld d,h              ; copy hl to de.
       ld e,l
       push de             ; store de.
       call add            ; quadruple result.
       pop de              ; restore de.
       call add            ; hexuple(?) result.
       ld e,(ix+4)         ; fraction of y.
       ld d,(ix+5)         ; integer of y.
       call add            ; add the sine.
       ld (ix+4),l         ; new fraction.
       ld (ix+5),h         ; new integer.
       rr l                ; get sign.
       jr c,kilbul         ; it's gone negative, remove bullet.
       ld a,h              ; get integer bit in accumulator.
       cp 248              ; right side of screen check.
       jr nc,kilbul        ; going past right edge, remove it.
       ld l,(ix+3)         ; x integer.
       ld h,(ix+5)         ; y integer.
       ld (dispx),hl       ; workspace coordinates.

       call bcol           ; check for collisions with enemies.
       ld a,(ix)           ; bullet status.

; Bullet came through unscathed so plot it at its new position.

       jp plot             ; plot new position.

; Kill bullet.

kilbul ld (ix),128         ; set flag to switch it off.
       ret

; Bullet collision detection.

bcol   ld hl,newtab+SPRSIZ ; table of sprite positions.
       ld b,NUMSPR-1       ; table positions to search.
bcol0  push bc             ; store sprite counter.
       push hl             ; store sprite pointer.
       ld a,(hl)           ; switched on/off.
       inc a               ; is it on?
       call nz,bcol1       ; yes, slot is used.
       pop hl              ; retrieve sprite pointer from stack.
       ld de,SPRSIZ        ; size of sprite table.
       add hl,de           ; point to next slot.
       pop bc              ; restore counter.
       djnz bcol0          ; repeat for all slots.
       ret
bcol1  dec a               ; back to previous value.
       cp FRUIT            ; piece of good fruit or below?
       ret c               ; yes, ignore it.
       cp ROBOT+4          ; bird, bad fruit or general nasty?
       ret nc              ; no, nothing to check.
;       jr c,bcol2          ; bad fruit collision check.
;       ret
bcol2  inc hl              ; point to x coordinate.
       inc hl
       ld a,(ix+3)         ; bullet x.
       inc a               ; bullet can be one pixel above.
       sub (hl)            ; compare against sprite x.
       cp 16               ; do coordinates match?
       ret nc              ; no, bullet missed.
       inc hl              ; point to sprite y.
       ld a,(ix+5)         ; bullet y.
       inc a               ; bullet can be one pixel left.
       sub (hl)            ; compare against sprite x.
       cp 16               ; do coordinates match?
       ret nc              ; no, bullet missed.
       dec hl              ; back to sprite flag.
       dec hl
       dec hl
       ld a,(hl)           ; type of item shot.
;       cp FRUIT            ; is it good fruit?
;       jr c,bcolf          ; yes.
       cp BAD              ; is it a bad ingredient?
       jr nc,bcolf         ; no, it's a nasty.
       ld (hl),EXPLOD      ; destroy bad fruit/fish/can.
       jp bcol3            ; kill bullet and continue.

; Change the sprite type into an explosion.

bcolf  ld (hl),EXPLOD      ; explosion type.
       inc hl              ; counter.
       ld (hl),0           ; reset counter.
       inc hl              ; point to x.
       ld a,(hl)           ; get x coordinate.
       add a,8             ; 8 pixels below = piece of fruit.
       cp 159              ; reached bottom already?
       ld e,a              ; x coordinate in e.
       jp nc,bcol3         ; yes, too low to spawn.
       inc hl              ; point to y.
       ld d,(hl)           ; get y coordinate.
       exx                 ; store coordinates.

; We've changed the nasty into an explosion.
; Now we want to generate a piece of fruit a few pixels below.

       push ix             ; store pointer to bullet.
       ld ix,newtab+SPRSIZ ; table of sprites.
       ld b,NUMSPR-1       ; table positions to search.
       ld de,SPRSIZ        ; size of sprite table.
spfr0  ld a,(ix)           ; switched on/off.
       inc a               ; is it on?
       jp z,spfr1          ; no, slot is free.
       add ix,de           ; point to next slot.
       djnz spfr0          ; repeat for all slots.

; Ran out of slots to spawn fruit.

       pop ix              ; retrieve bullet pointer.
       exx                 ; restore all registers.

bcol3  pop de              ; bcol return address off the stack.
       pop de              ; bcol sprite pointer off stack.
       pop de              ; bcol counter off the stack.
       pop de              ; mvbul1 return address off stack.
       ld b,1              ; amount to add.
       ld hl,score+4       ; tens column.
       call uscor          ; up the score.
       call dscor          ; display player's score.
       jp kilbul           ; stop the bullet.

spfr1  call random         ; random number.
       and 3               ; number of different fruits.
       ld hl,dropfr        ; dropped fruit table.
       call addac          ; add fruit displacement.
       call getlev         ; current level.
       rlca                ; multiply by 4.
       rlca
       call addac          ; add displacement for level.
       ld a,(hl)           ; get chosed fruit type.
       ld (ix),a           ; make sprite a piece of fruit.
       exx                 ; restore hl, de, bc registers.
       ld (ix+1),0         ; set to zero so it can be collected.
       ld (ix+2),e         ; set x coordinate.
       ld (ix+3),d         ; set y coordinate.
       pop ix              ; restore bullet pointer.
       jp bcol3            ; kill off the bullet and continue.

; Display or delete all bullets.

dbul   ld b,NUMBUL         ; bullets to move.
       ld ix,bultab        ; bullet table address.
;dbul0  push bc             ; store counter.
dbul0  ld a,(ix)
       rla                 ; check leftmost bit.
       call nc,dbul1       ; not set, so bullet is enabled.
;       pop bc              ; retrieve counter.
       ld de,BULSIZ        ; bytes required per bullet.
       add ix,de           ; point ix at next one.
       djnz dbul0          ; repeat for all bullets.
       ret
dbul1  ld l,(ix+3)         ; x integer.
       ld h,(ix+5)         ; y integer.
       ld (dispx),hl       ; workspace coordinates.
       jp plot             ; plot new position.



; Reduce shield.

rshld  ld hl,deplet        ; energy depletion counter.
       ld a,(hl)           ; get amount by which we need to decrease.
       and a               ; is energy going down?
       jp z,ishld          ; no, it's okay.
       dec (hl)            ; one less pixel to decrement.
       ld hl,(enbar)       ; energy bar.
       ld a,l              ; low byte of cell.
       cp 225              ; moving past edge of bar?
       jr c,ishld          ; yes, can't go further.
       ld a,(enbit)        ; bit to remove.
       ld c,a              ; store for now.
       ld b,8              ; height of bar.
rshld0 xor (hl)            ; remove edge.
       ld (hl),a           ; write new value.
       inc h               ; point to next one.
       ld a,c              ; get next bit to remove.
       djnz rshld0         ; repeat for each row.
       ld a,(enbit)        ; get energy bit.
       rlca                ; shift it left, with carry.
       ld (enbit),a        ; that's the new position.
       jr nc,ishld         ; not moving to another cell so we're done.
       ld hl,(enbar)       ; get screen graphic address.
       dec hl              ; one cell to the left.
       ld (enbar),hl       ; put it back.
       ld a,l              ; low byte of cell.
       cp 225              ; moving past edge of bar?
       jr nc,ishld         ; no, still energy left then.
die    ld hl,dflag         ; dead flag.
       ld (hl),1           ; set to non-zero value.
       ld a,EXPLOD         ; explosion type.
       ld (newtab),a       ; set sprite type.
       ret

; Increase shield.

ishld  ld hl,increm        ; energy increment counter.
       ld a,(hl)           ; get amount by which we need to decrease.
       and a               ; is energy going up?
       ret z               ; no, it's okay.
       dec (hl)            ; one less pixel to increment.
       ld hl,(enbar)       ; energy bar.
       ld a,(enbit)        ; bit to remove.
       rra                 ; shift it right.
       jr nc,ishld1        ; no carry, still within byte.
       ld hl,(enbar)       ; get screen graphic address.
       inc hl              ; one cell to the right.
       ld a,l              ; low byte of cell.
       cp 229              ; moving past edge of bar?
       ret nc              ; yes, can go no further.
       ld (enbar),hl       ; set new bar address.
ishld1 ld a,(enbit)        ; bit to remove.
       rrca                ; shift it right, with carry.
       ld (enbit),a        ; that's the new position.
       ld c,a              ; store for now.
       ld b,8              ; height of bar.
ishld0 xor (hl)            ; remove edge.
       ld (hl),a           ; write new value.
       inc h               ; point to next one.
       ld a,c              ; get next bit to remove.
       djnz ishld0         ; repeat for each row.
       ret




bultab defb 'Elderberry wine rules!'

       org bultab + BULSIZ * NUMBUL


; ix + 0 = type.  255 = off.
; ix + 1 = direction/timer.
; ix + 2 = x.
; ix + 3 = y.

oldtab equ $
       org oldtab + LEN
newtab equ $
       org newtab + LEN

; Get address for sprite image.

image  rlca                ; multiply by 2.
       ld l,a              ; low byte in c.
       ld h,96             ; high byte (24576/256).
       ld e,(hl)           ; low byte of address.
       inc hl              ; point to next byte.
       ld a,(dispy)        ; y coordinate.
       and 7               ; position straddling cells.
       ld c,a              ; store in c for now.
       add a,(hl)          ; add high byte of sprite address.
       ld d,a              ; address now in de.
       ld b,0              ; no high byte for mask displacement.
       ld hl,spmask        ; sprite masks.
       add hl,bc           ; point to mask.
       ld a,(hl)           ; get mask.
       ld b,a              ; left side.
       cpl                 ; reverse all bits.
       ld c,a              ; right side.
       ret

spmask defb 255,127,63,31,15,7,3,1

dspr   ld ix,oldtab        ; old table.
       ld b,NUMSPR         ; table positions to search.
dspr0  push bc             ; store sprite counter.
       ld a,(ix)           ; switched on/off.
       inc a               ; is it on?
       jp nz,dspr1         ; yes, old is on.
       ld a,(ix+LEN)       ; new sprite.
       inc a               ; is it on?
       jp nz,dnew          ; yes, new is on.

; Neither sprite slot enabled, nothing to do.

dspr2  ld de,SPRSIZ        ; size of sprite table.
       add ix,de           ; point to next slot.
       pop bc              ; restore counter.
       djnz dspr0          ; repeat for all slots.
       ret

; Old sprite is on - what about corresponding position in new table?

dspr1  ld a,(ix+LEN)       ; new sprite.
       inc a               ; is it on?
       jp nz,dboth         ; yes, both are on.
dold   ld l,(ix+2)         ; old x.
       ld h,(ix+3)         ; old y.
       ld (dispx),hl       ; set general coordinates.
       ld a,(ix)           ; old image type.
       call image          ; get image address.
       call scadd          ; get screen address.
       ld a,16             ; sprite height.
dold0  ex af,af'           ; store away.
       call dline          ; draw line of sprite.
       ex af,af'           ; restore loop counter.
       dec a               ; one less iteration.
       jp nz,dold0
       jp dspr2
dnew   ld l,(ix+LEN+2)     ; new x.
       ld h,(ix+LEN+3)     ; new y.
       ld (dispx),hl       ; set general coordinates.
       ld a,(ix+LEN)       ; new image type.
       call image          ; get image address.
       call scadd          ; get screen address.
       ld a,16             ; sprite height.
dnew0  ex af,af'           ; store away.
       call dline          ; draw line of sprite.
       ex af,af'           ; restore loop counter.
       dec a               ; one less iteration.
       jp nz,dnew0
       jp dspr2
dboth  ld l,(ix+2)         ; old x.
       ld h,(ix+3)         ; old y.
       ld (dispx),hl       ; set general coordinates.
       ld a,(ix)           ; old image type.
       call image          ; get image address.
       call scadd          ; get screen address.
       exx                 ; old sprite to alternate registers.
       ld l,(ix+LEN+2)     ; new x.
       ld h,(ix+LEN+3)     ; new y.
       ld (dispx),hl       ; set general coordinates.
       ld a,(ix+LEN)       ; new image type.
       call image          ; get image address.
       call scadd          ; get screen address.
       ld a,16             ; sprite height.
dboth0 ex af,af'           ; store line counter.
       call dline          ; draw line of sprite.
       exx                 ; restore old sprite.
       call dline          ; draw line of sprite.
       exx                 ; back to new sprite.
       ex af,af'           ; restore loop counter.
       dec a               ; one less iteration.
       jp nz,dboth0        ; repeat for all lines.
       jp dspr2


; Single sprite routine, for sprites we don't update in the main loop.
; Not used for main game sprites, but things like status panel.

sspr   call image          ; get image address.
       call scadd          ; get screen address.
       ld a,16             ; sprite height.
sspr0  ex af,af'           ; store away.
       call dline          ; draw line of sprite.
       ex af,af'           ; restore loop counter.
       dec a               ; one less iteration.
       jp nz,sspr0         ; repeat for all 16 lines.
       ret



; Calculate screen address, returning result in hl.
; Only corrupts hl and accumulator.

scadd  ld a,(dispx)        ; vertical position.
       ld l,a              ; store in l register.
       and 7               ; get position within char cell.
       add a,64            ; add to screen start address.
       ld h,a              ; place that in high byte.
       ld a,l              ; restore the vertical.
       rrca                ; divide by 8 to find segment.
       rrca
       rrca
       and 24              ; 0, 8 and 16 for segment displacement.
       add a,h             ; add to high byte.
       ld h,a              ; store in high byte.
       ld a,l              ; restore vertical again.
       rlca                ; multiply by 4.
       rlca
       and 224             ; multiples of 32 wanted.
       ld l,a              ; put in low byte.
       ld a,(dispy)        ; fetch horizontal position.
       rrca
       rrca
       rrca
       and 31
       add a,l
       ld l,a
       ret                 ; hl = address of screen.

; Calculate address of attribute for pixel at (dispx, dispy).

ataddp ld a,(ix+2)         ; Look at the vertical first.
       rlca                ; divide by 64.
       rlca                ; quicker than 6 rrca operations.
       ld l,a              ; store in e register for now.
       and 3               ; mask to find segment.
       add a,88            ; attributes start at 88*256=22528.
       ld h,a              ; that's our high byte sorted.
       ld a,l              ; vertical/64 - same as vertical*4.
       and 224             ; want a multiple of 32.
       ld l,a              ; vertical element calculated.
       ld a,(ix+3)         ; get horizontal position.
       rra                 ; divide by 8.
       rra
       rra
       and 31              ; want result in range 0-31.
       add a,l             ; add to existing low byte.
       ld l,a              ; that's the low byte done.
       ret                 ; attribute address now in de.


dline  ld a,(de)           ; source byte.
       and b               ; apply left mask.
       xor (hl)            ; combine with screen.
       ld (hl),a           ; write result.
       inc l               ; next target address.
       inc de              ; next source byte.

       ld a,(de)           ; source byte.
       xor (hl)            ; combine with screen.
       ld (hl),a           ; write result.
       inc l               ; next target address.
       dec de              ; back to first source byte of line.
       ld a,(de)           ; source byte.
       and c               ; apply right mask.
       xor (hl)            ; combine with screen.
       ld (hl),a           ; write result.
       inc de              ; next source byte.
       inc de              ; next source byte.
       dec l               ; restore hl to start of line.
       dec l

; Line drawn, now work out next target address.

nline  inc h               ; increment pixel.
       ld a,h              ; get pixel address.
       and 7               ; straddling character position?
       ret nz              ; no, we're on next line already.
       ld a,h              ; get pixel address.
       sub 8               ; subtract 8 for start of segment.
       ld h,a              ; new high byte of address.
       ld a,l              ; get low byte of address.
       add a,32            ; one line down.
       ld l,a              ; new low byte.
       ret nc              ; not reached next segment yet.
       ld a,h              ; address high.
       add a,8             ; add 8 to next segment.
       ld h,a              ; new high byte.
       ret

dispx  defb 0
dispy  defb 0

playxf defb 0
playyf defb 0



; Plots a 4-pixel square on the screen.
; Calls scadd to calculate screen address and uses small table of
; pixel values to write to this address.

plot   ld a,(dispy)        ; Which pixel within byte do we
       and 7               ; want to set first?
       rlca                ; 2 bytes for each square in look-up table.
       ld e,a
       ld d,0
       ld hl,dots          ; Table of pixel positions.
       add hl,de           ; hl points to values we want to POKE to screen.
       ld d,(hl)
       inc hl
       ld e,(hl)           ; pixel values in de.
       call scadd          ; screen address.
       ld a,(hl)           ; see what's already there.
       xor d               ; merge with pixels.
       ld (hl),a           ; put back on screen.
       inc hl              ; could straddle two character squares.
       ld a,(hl)           ; get existing image.
       xor e               ; combine pixels.
       ld (hl),a           ; write new image.
       dec hl              ; back to first cell.
       call nline          ; next line down.
       ld a,(hl)           ; get screen image.
       xor d               ; combine with plot image.
       ld (hl),a           ; write to screen.
       inc hl              ; next cell.
       ld a,(hl)           ; get screen data.
       xor e               ; merge images.
       ld (hl),a           ; write to screen.
       ret

dots   defb 192,0,96,0
       defb 48,0,24,0
       defb 12,0,6,0
       defb 3,0,1,128





; Joystick and keyboard reading routines.

joykey ld a,(contrl)       ; control flag.
       and a               ; is it the keyboard?
       jr nz,joyjoy        ; no, it's joystick.

; Keyboard controls.

       ld e,0              ; control.
       ld a,(keys+4)       ; fire key.
       call ktest          ; being pressed?
       ccf                 ; complement the carry.
       rl e                ; rotate into e register.
       ld a,(keys)         ; up key.
       call ktest          ; being pressed?
       ccf                 ; complement the carry.
       rl e                ; rotate into e register.
       ld a,(keys+1)       ; down key.
       call ktest          ; being pressed?
       ccf                 ; complement the carry.
       rl e                ; rotate into e register.
       ld a,(keys+2)       ; left key.
       call ktest          ; being pressed?
       ccf                 ; complement the carry.
       rl e                ; rotate into bit d0 of e register.
       ld a,(keys+3)       ; left key.
       call ktest          ; being pressed?
       ccf                 ; complement the carry.
       rl e                ; destined for bit d1 of e register.
       ld a,e              ; place result in accumulator.
       ld (joyval),a       ; put that into joyval.
       ret

joyjoy dec a               ; is it Kempston?
       jr z,joykem         ; yes.

; Sinclair joystick controls.

joysin ld bc,61438         ; port for Sinclair 2.
       in a,(c)            ; read joystick.
       ld d,a              ; clear values.
       xor a               ; clear accumulator.
       ld e,16             ; Kempston fire bit value.
       bit 0,d             ; fire bit pressed?
       call z,joysi0       ; add bit.
       ld e,1              ; Kempston bit value.
       bit 3,d             ; fire bit pressed?
       call z,joysi0       ; add bit.
       ld e,2              ; Kempston bit value.
       bit 4,d             ; fire bit pressed?
       call z,joysi0       ; add bit.
       ld e,8              ; Kempston bit value.
       bit 1,d             ; fire bit pressed?
       call z,joysi0       ; add bit.
       ld e,4              ; Kempston bit value.
       bit 2,d             ; fire bit pressed?
       call z,joysi0       ; add bit.
       ld (joyval),a       ; remember value.
       ret

joysi0 add a,e             ; add bit value.
       ret


; Kempston joystick controls.

joykem ld bc,31            ; port for Kempston interface.
       in a,(c)            ; read it.
       ld (joyval),a       ; remember value.
       ret

; Key test routine.

ktest  ld c,a              ; key to test in c.
       and 7               ; mask bits d0-d2 for row.
       inc a               ; in range 1-8.
       ld b,a              ; place in b.
       srl c               ; divide c by 8
       srl c               ; to find position within row.
       srl c
       ld a,5              ; only 5 keys per row.
       sub c               ; subtract position.
       ld c,a              ; put in c.
       ld a,254            ; high byte of port to read.
ktest0 rrca                ; rotate into position.
       djnz ktest0         ; repeat until we've found relevant row.
       in a,(254)          ; read port (a=high, 254=low).
ktest1 rra                 ; rotate bit out of result.
       dec c               ; loop counter.
       jp nz,ktest1        ; repeat until bit for position in carry.
       ret

; User defined keys.

keys   defb 28,29,27,35,16,1

joyval defb 0              ; keyboard/joystick value.
contrl defb 0              ; control method chosen.
joydir defw 0              ; joystick horizontal and vertical flags.


; Pause mode.
; Test for pause key.

pause  ld a,(keys+5)       ; pause key.
       call ktest          ; has it been pressed?
       ret c               ; no.
pause0 ld a,(keys+5)       ; pause key.
       call ktest          ; has it been pressed?
       jr nc,pause0        ; wait until pause key is released.
pause1 ld a,9              ; key J.
       call ktest          ; pressed?
       jr c,pause3         ; no, skip cheat.
       ld a,15             ; key C.
       call ktest          ; pressed?
       jr c,pause3         ; no, skip cheat.
       ld hl,fruit         ; fruit to add.
       ld (hl),100         ; fill it up.
       ret
pause3 ld a,(keys+5)       ; pause button.
       call ktest          ; pressed it yet?
       jr c,pause1         ; pause game until pause key pressed.
pause2 ld a,(keys+5)       ; pause key.
       call ktest
       jr nc,pause2        ; wait until pause key is released.
       ret

addac  ld e,a              ; position to write.
       ld d,0              ; no displacement high byte.
       add hl,de           ; point to trail position.
       ret

getlev ld a,(levnum)       ; level number.
       cp 10               ; reached maximum?
       ret c               ; no, this is the level we're on.
       and 1               ; zero or 1.
       add a,8             ; toggle between last 2 levels.
       ret

; Routine to clear the screen.
; Slightly quicker than the one in the Sinclair ROM.

cls    ld hl,16384         ; pixels.
       ld de,16385         ; second byte of screen.
       ld bc,6144          ; size of screen.
       ld (hl),l           ; blank the first byte.
       ldir                ; clear rest of screen.
       ld a,(23693)        ; attributes.
       ld (hl),a           ; set attribute.
       ld bc,767           ; bytes to write.
       ldir                ; set attributes for rest of screen.
       ret


levnum defb 0              ; current level.
levlif defb 5              ; levels before another extra life spawned.
numlif defb 3              ; number of lives.


; Tables of level data.

; Spawn chance.
; To increase the frequency with which sprites are spawned as the game
; progresses, the chances of spawning a sprite increase with each level.

spchan defb 6,7,7,8,9,10,11,12,13,13

; Table of 8 sprites for each level.
; One is chosen at random every time a sprite spawns.

ranspr defb WASP,WASP,WASP,WASP,WASP,WASP,WASP,WASP
       defb WASP,WASP,WASP,WASP,FISH,WASP,ROBOT,ROBOT
       defb ROBOT,BAT,WASP,CORE,WASP,WASP,ROBOT,ROBOT
       defb ROBOT,BAT,ROBOT,CORE,ROBOT,ROBOT,BAT,TIN
       defb BAT,BAT,WASP,FISH,WASP,BAT,BIRD,NUT
       defb BAT,WASP,WASP,CORE,WASP,BAT,BIRD,TIN
       defb BAT,BIRD,ROBOT,FISH,ROBOT,BAT,FUZZ,CORE
       defb BAT,BIRD,WASP,TIN,WASP,BAT,FUZZ,FISH
       defb BAT,DRAG,ROBOT,NUT,BAT,FUZZ,BIRD,TIN
       defb BAT,FUZZ,BAT,FISH,WASP,BIRD,DRAG,CORE

; Table containing maximum number of nasty sprites per level.

numspr defb 5,5,6,6,7,7,8,8,9,9

; Table of fruits to appear on each level.

dropfr defb GRAPE,APPLE,GRAPE,CHERRY
       defb GRAPE,APPLE,APPLE,STRAW
       defb STRAW,GRAPE,CHERRY,APPLE
       defb NANA,APPLE,CHERRY,GRAPE
       defb PEAR,GRAPE,NANA,STRAW
       defb CHERRY,MELON,LEMON,GRAPE
       defb MELON,NANA,PEAR,STRAW
       defb LEMON,ORANGE,GRAPE,MELON
       defb GRAPE,NANA,ORANGE,PEAR
       defb STRAW,MELON,LEMON,PLUM

; Bonus fruits required for each level.

levreq defb GRAPE,6
       defb APPLE,6
       defb STRAW,7
       defb NANA,7
       defb PEAR,8
       defb CHERRY,8
       defb MELON,8
       defb LEMON,9
       defb ORANGE,9
       defb PLUM,9

; Colours for each level.

wincol defb 69,70,66,67,6,2,4,70,67,69


; List of bonuses to generate.

bonlst defb CAKE,BURG,GEM,GEM


; Sprite graphics.
; Don't try this at home, kids.
; These are arranged in groups of 8 sprites, with each pre-shifted
; position 256 bytes away from the last.  This makes the sprite code
; simpler, in that I can take the horizontal coordinate and AND it
; with 7 to find the displacement for the correct pre-shifted
; version.  It's a nightmare to maintain, however.  Not recommended.


barrel defb 0,0,43,168,87,212,0,0,0,0,183,218,183,218,183,218
       defb 183,218,183,218,183,218,0,0,0,0,87,212,43,168,0,0
barr01 defb 13,0,18,96,0,248,40,52,77,133,91,225,91,232,187,236
       defb 23,221,135,218,160,218,44,58,15,20,6,68,0,144,0,96
barr02 defb 3,64,8,224,22,80,47,56,94,156,61,202,187,231,151,242
       defb 79,233,231,221,83,188,57,122,28,244,10,104,7,16,2,192
barr03 defb 1,96,14,0,17,48,71,128,159,24,136,220,71,204,143,200
       defb 47,226,103,229,119,17,48,250,35,240,25,140,0,112,12,128
barr04 defb 0,0,7,224,32,4,71,226,39,228,64,2,39,228,103,230
       defb 103,230,103,230,39,228,64,2,39,228,71,226,32,4,7,224
barr05 defb 12,128,0,112,25,140,35,240,48,250,119,17,103,229,47,226
       defb 143,200,71,204,136,220,159,24,71,128,17,48,14,0,1,96
barr06 defb 2,192,7,16,10,104,28,244,57,122,83,188,231,221,79,233
       defb 151,242,187,231,61,202,94,156,47,56,22,80,8,224,3,64
barr07 defb 0,176,6,72,31,0,44,20,161,178,135,218,23,218,55,221
       defb 187,232,91,225,91,5,92,52,40,240,34,96,9,0,6,0

       defb 0,0,21,212,43,234,0,0,0,0,91,237,91,237,91,237
       defb 91,237,91,237,91,237,0,0,0,0,43,234,21,212,0,0
       defb 6,128,9,48,0,124,20,26,166,194,173,240,45,244,93,246
       defb 139,238,67,237,80,109,22,29,7,138,3,34,0,72,0,48
       defb 1,160,4,112,11,40,23,156,47,78,30,229,221,243,75,249
       defb 167,244,243,238,41,222,28,189,14,122,5,52,3,136,1,96
       defb 0,176,7,0,8,152,35,192,79,140,68,110,35,230,71,228
       defb 23,241,179,242,187,136,24,125,17,248,12,198,0,56,6,64
       defb 0,0,3,240,16,2,35,241,19,242,32,1,19,242,51,243
       defb 51,243,51,243,19,242,32,1,19,242,35,241,16,2,3,240
       defb 6,64,0,56,12,198,17,248,24,125,187,136,179,242,23,241
       defb 71,228,35,230,68,110,79,140,35,192,8,152,7,0,0,176
       defb 1,96,3,136,5,52,14,122,28,189,41,222,243,238,167,244
       defb 75,249,221,243,30,229,47,78,23,156,11,40,4,112,1,160
       defb 0,88,3,36,15,128,22,10,80,217,67,237,11,237,155,238
       defb 93,244,173,240,173,130,46,26,20,120,17,48,4,128,3,0

       defb 0,0,10,234,21,245,0,0,0,0,173,246,173,246,173,246
       defb 173,246,173,246,173,246,0,0,0,0,21,245,10,234,0,0
       defb 3,64,4,152,0,62,10,13,83,97,86,248,22,250,46,251
       defb 69,247,161,246,168,54,139,14,3,197,1,145,0,36,0,24
       defb 0,208,2,56,5,148,11,206,23,167,143,114,238,249,165,252
       defb 83,250,121,247,20,239,142,94,7,61,2,154,1,196,0,176
       defb 0,88,3,128,4,76,17,224,39,198,34,55,17,243,35,242
       defb 139,248,89,249,93,196,140,62,8,252,6,99,0,28,3,32
       defb 0,0,1,248,8,1,145,248,9,249,144,0,9,249,153,249
       defb 153,249,153,249,9,249,144,0,9,249,145,248,8,1,1,248
       defb 3,32,0,28,6,99,8,252,140,62,93,196,89,249,139,248
       defb 35,242,17,243,34,55,39,198,17,224,4,76,3,128,0,88
       defb 0,176,1,196,2,154,7,61,142,94,20,239,121,247,83,250
       defb 165,252,238,249,143,114,23,167,11,206,5,148,2,56,0,208
       defb 0,44,1,146,7,192,11,5,168,108,161,246,133,246,77,247
       defb 46,250,86,248,86,193,23,13,10,60,8,152,2,64,1,128

       defb 0,0,5,117,138,250,0,0,0,0,86,251,86,251,86,251
       defb 86,251,86,251,86,251,0,0,0,0,138,250,5,117,0,0
       defb 1,160,2,76,0,31,133,6,169,176,43,124,11,125,151,125
       defb 162,251,80,251,84,27,69,135,129,226,128,200,0,18,0,12
       defb 0,104,1,28,2,202,5,231,139,211,71,185,247,124,82,254
       defb 41,253,188,251,138,119,71,47,131,158,1,77,0,226,0,88
       defb 0,44,1,192,2,38,8,240,19,227,145,27,136,249,17,249
       defb 69,252,172,252,46,226,70,31,4,126,131,49,0,14,1,144
       defb 0,0,0,252,132,0,72,252,132,252,72,0,132,252,204,252
       defb 204,252,204,252,132,252,72,0,132,252,72,252,132,0,0,252
       defb 1,144,0,14,131,49,4,126,70,31,46,226,172,252,69,252
       defb 17,249,136,249,145,27,19,227,8,240,2,38,1,192,0,44
       defb 0,88,0,226,1,77,131,158,71,47,138,119,188,251,41,253
       defb 82,254,247,124,71,185,139,211,5,231,2,202,1,28,0,104
       defb 0,22,0,201,3,224,133,130,84,54,80,251,66,251,166,251
       defb 23,125,43,124,171,96,139,134,5,30,4,76,1,32,0,192

       defb 0,0,130,186,69,125,0,0,0,0,171,125,171,125,171,125
       defb 171,125,171,125,171,125,0,0,0,0,69,125,130,186,0,0
       defb 0,208,1,38,128,15,66,131,84,216,21,190,133,190,203,190
       defb 209,125,168,125,170,13,162,195,64,241,64,100,0,9,0,6
       defb 0,52,0,142,1,101,130,243,197,233,163,220,123,190,41,127
       defb 148,254,222,125,197,59,163,151,65,207,128,166,0,113,0,44
       defb 0,22,0,224,1,19,4,120,137,241,200,141,196,124,136,252
       defb 34,254,86,126,23,113,163,15,2,63,193,152,0,7,0,200
       defb 0,0,0,126,66,0,36,126,66,126,36,0,66,126,102,126
       defb 102,126,102,126,66,126,36,0,66,126,36,126,66,0,0,126
       defb 0,200,0,7,193,152,2,63,163,15,23,113,86,126,34,254
       defb 136,252,196,124,200,141,137,241,4,120,1,19,0,224,0,22
       defb 0,44,0,113,128,166,65,207,163,151,197,59,222,125,148,254
       defb 41,127,123,190,163,220,197,233,130,243,1,101,0,142,0,52
       defb 0,11,128,100,1,240,66,193,42,27,168,125,161,125,211,125
       defb 139,190,21,190,85,176,69,195,2,143,2,38,0,144,0,96

       defb 0,0,65,93,162,190,0,0,0,0,213,190,213,190,213,190
       defb 213,190,213,190,213,190,0,0,0,0,162,190,65,93,0,0
       defb 0,104,0,147,192,7,161,65,42,108,10,223,66,223,101,223
       defb 232,190,212,62,213,6,209,97,160,120,32,50,128,4,0,3
       defb 0,26,0,71,128,178,193,121,226,244,81,238,61,223,148,191
       defb 74,127,239,62,226,157,209,203,160,231,64,83,128,56,0,22
       defb 0,11,0,112,128,137,2,60,196,248,228,70,98,62,68,126
       defb 17,127,43,63,139,184,209,135,129,31,96,204,128,3,0,100
       defb 0,0,0,63,33,0,18,63,33,63,18,0,33,63,51,63
       defb 51,63,51,63,33,63,18,0,33,63,18,63,33,0,0,63
       defb 0,100,128,3,96,204,129,31,209,135,139,184,43,63,17,127
       defb 68,126,98,62,228,70,196,248,2,60,128,137,0,112,0,11
       defb 0,22,128,56,64,83,160,231,209,203,226,157,239,62,74,127
       defb 148,191,61,223,81,238,226,244,193,121,128,178,0,71,0,26
       defb 128,5,64,50,0,248,161,96,149,13,212,62,208,190,233,190
       defb 69,223,10,223,42,216,162,225,129,71,1,19,0,72,0,48

       defb 0,0,160,174,81,95,0,0,0,0,106,223,106,223,106,223
       defb 106,223,106,223,106,223,0,0,0,0,81,95,160,174,0,0
       defb 0,52,128,73,224,3,208,160,21,54,133,111,161,111,178,239
       defb 116,95,106,31,106,131,232,176,80,60,16,25,64,2,128,1
       defb 0,13,128,35,64,89,224,188,113,122,40,247,158,239,202,95
       defb 165,63,119,159,241,78,232,229,208,115,160,41,64,28,0,11
       defb 128,5,0,56,192,68,1,30,98,124,114,35,49,31,34,63
       defb 136,191,149,159,69,220,232,195,192,143,48,102,192,1,0,50
       defb 0,0,128,31,16,128,137,31,144,159,9,0,144,159,153,159
       defb 153,159,153,159,144,159,9,0,144,159,137,31,16,128,128,31
       defb 0,50,192,1,48,102,192,143,232,195,69,220,149,159,136,191
       defb 34,63,49,31,114,35,98,124,1,30,192,68,0,56,128,5
       defb 0,11,64,28,160,41,208,115,232,229,241,78,119,159,165,63
       defb 202,95,158,239,40,247,113,122,224,188,64,89,128,35,0,13
       defb 192,2,32,25,0,124,80,176,202,134,106,31,104,95,116,223
       defb 162,239,133,111,21,108,209,112,192,163,128,137,0,36,0,24

       defb 0,0,80,87,168,175,0,0,0,0,181,111,181,111,181,111
       defb 181,111,181,111,181,111,0,0,0,0,168,175,80,87,0,0
       defb 0,26,192,36,240,1,104,80,10,155,194,183,208,183,217,119
       defb 186,47,181,15,181,65,116,88,40,30,136,12,32,1,192,0
       defb 128,6,192,17,160,44,112,94,56,189,148,123,207,119,229,47
       defb 210,159,187,207,120,167,244,114,232,57,208,20,32,14,128,5
       defb 192,2,0,28,96,34,0,143,49,62,185,17,152,143,145,31
       defb 196,95,202,207,34,238,244,97,224,71,24,51,224,0,0,25
       defb 0,0,192,15,8,64,196,143,200,79,4,128,200,79,204,207
       defb 204,207,204,207,200,79,4,128,200,79,196,143,8,64,192,15
       defb 0,25,224,0,24,51,224,71,244,97,34,238,202,207,196,95
       defb 145,31,152,143,185,17,49,62,0,143,96,34,0,28,192,2
       defb 128,5,32,14,208,20,232,57,244,114,120,167,187,207,210,159
       defb 229,47,207,119,148,123,56,189,112,94,160,44,192,17,128,6
       defb 96,1,144,12,0,62,40,88,101,67,181,15,180,47,186,111
       defb 209,119,194,183,10,182,104,184,224,81,192,68,0,18,0,12


grape  defb 8,0,16,0,112,24,9,172,2,196,27,216,43,204,61,156
       defb 60,108,26,176,36,236,86,214,119,94,123,30,48,12,0,0
melon  defb 0,0,0,0,0,0,0,0,191,253,173,181,191,253,191,253
       defb 189,189,175,245,191,253,187,221,79,242,112,14,15,240,0,0
lemon  defb 0,0,112,0,251,224,255,120,95,252,63,252,123,238,127,254
       defb 95,254,61,254,47,216,39,246,17,255,14,127,1,238,0,0
cherry defb 1,0,0,128,0,64,1,64,2,32,4,32,4,0,0,56
       defb 15,124,31,190,63,222,47,222,47,220,23,184,15,0,0,0
core   defb 0,128,1,0,12,124,63,254,124,242,72,64,3,48,7,240
       defb 7,224,2,224,2,64,3,24,34,126,29,252,15,176,0,0
can    defb 48,0,124,0,90,0,110,0,24,240,7,12,24,240,1,248
       defb 9,248,9,248,1,248,8,248,0,248,24,48,7,12,0,240
fish   defb 0,6,0,71,2,27,1,96,8,144,5,64,18,32,10,128
       defb 4,64,53,0,120,128,124,0,222,0,252,0,252,0,112,0
barr15 defb 7,240,30,216,122,4,168,4,161,178,134,218,150,218,182,221
       defb 187,109,91,97,91,1,92,5,40,53,32,182,26,220,15,224

       defb 4,0,8,0,56,12,4,214,1,98,13,236,21,230,30,206
       defb 30,54,13,88,18,118,43,107,59,175,61,143,24,6,0,0
       defb 0,0,0,0,0,0,0,0,223,254,214,218,223,254,223,254
       defb 222,222,215,250,223,254,221,238,39,249,56,7,7,248,0,0
       defb 0,0,56,0,125,240,127,188,47,254,31,254,61,247,63,255
       defb 47,255,30,255,23,236,19,251,136,255,135,63,0,247,0,0
       defb 0,128,0,64,0,32,0,160,1,16,2,16,2,0,0,28
       defb 7,190,15,223,31,239,23,239,23,238,11,220,7,128,0,0
       defb 0,64,0,128,6,62,31,255,62,121,36,32,1,152,3,248
       defb 3,240,1,112,1,32,1,140,17,63,14,254,7,216,0,0
       defb 24,0,62,0,45,0,55,0,12,120,3,134,12,120,0,252
       defb 4,252,4,252,0,252,4,124,0,124,12,24,3,134,0,120
       defb 0,3,128,35,129,13,0,176,4,72,2,160,9,16,5,64
       defb 2,32,26,128,60,64,62,0,111,0,126,0,126,0,56,0
       defb 3,248,15,108,61,2,84,2,80,217,67,109,75,109,219,110
       defb 221,182,173,176,173,128,174,2,148,26,16,91,13,110,7,240

       defb 2,0,4,0,28,6,2,107,0,177,6,246,10,243,15,103
       defb 15,27,6,172,9,59,149,181,157,215,158,199,12,3,0,0
       defb 0,0,0,0,0,0,0,0,111,255,107,109,111,255,111,255
       defb 111,111,107,253,111,255,110,247,147,252,156,3,3,252,0,0
       defb 0,0,28,0,62,248,63,222,23,255,15,255,158,251,159,255
       defb 151,255,143,127,11,246,137,253,196,127,195,159,128,123,0,0
       defb 0,64,0,32,0,16,0,80,0,136,1,8,1,0,0,14
       defb 3,223,135,239,143,247,139,247,11,247,5,238,3,192,0,0
       defb 0,32,0,64,3,31,143,255,159,60,18,16,0,204,1,252
       defb 1,248,0,184,0,144,0,198,136,159,7,127,3,236,0,0
       defb 12,0,31,0,22,128,27,128,6,60,1,195,6,60,0,126
       defb 2,126,2,126,0,126,2,62,0,62,6,12,1,195,0,60
       defb 128,1,192,17,192,134,0,88,2,36,1,80,4,136,2,160
       defb 1,16,13,64,30,32,31,0,55,128,63,0,63,0,28,0
       defb 1,252,7,182,30,129,42,1,168,108,161,182,165,182,109,183
       defb 110,219,86,216,86,192,87,1,74,13,136,45,6,183,3,248

       defb 1,0,2,0,14,3,129,53,128,88,3,123,133,121,135,179
       defb 135,141,3,86,132,157,202,218,206,235,207,99,134,1,0,0
       defb 0,0,0,0,0,0,0,0,183,255,181,182,183,255,183,255
       defb 183,183,181,254,183,255,183,123,73,254,206,1,1,254,0,0
       defb 0,0,14,0,31,124,31,239,139,255,135,255,207,125,207,255
       defb 203,255,199,191,5,251,196,254,226,63,225,207,192,61,0,0
       defb 0,32,0,16,0,8,0,40,0,68,0,132,0,128,0,7
       defb 129,239,195,247,199,251,197,251,133,251,2,247,1,224,0,0
       defb 0,16,0,32,129,143,199,255,79,158,9,8,0,102,0,254
       defb 0,252,0,92,0,72,0,99,196,79,131,191,1,246,0,0
       defb 6,0,15,128,11,64,13,192,3,30,128,225,3,30,0,63
       defb 1,63,1,63,0,63,1,31,0,31,3,6,128,225,0,30
       defb 192,0,224,8,96,67,0,44,1,18,0,168,2,68,1,80
       defb 0,136,6,160,15,16,15,128,27,192,31,128,31,128,14,0
       defb 0,254,3,219,143,64,149,0,84,54,80,219,82,219,182,219
       defb 183,109,43,108,43,96,171,128,165,6,196,22,131,91,1,252

       defb 0,128,1,0,135,1,192,154,64,44,129,189,194,188,195,217
       defb 195,198,1,171,194,78,101,109,231,117,231,177,195,0,0,0
       defb 0,0,0,0,0,0,0,0,219,255,90,219,219,255,219,255
       defb 219,219,90,255,219,255,219,189,36,255,231,0,0,255,0,0
       defb 0,0,7,0,15,190,143,247,197,255,195,255,231,190,231,255
       defb 229,255,227,223,130,253,98,127,241,31,240,231,224,30,0,0
       defb 0,16,0,8,0,4,0,20,0,34,0,66,0,64,128,3
       defb 192,247,225,251,227,253,226,253,194,253,129,123,0,240,0,0
       defb 0,8,0,16,192,199,227,255,39,207,4,132,0,51,0,127
       defb 0,126,0,46,0,36,128,49,226,39,193,223,0,251,0,0
       defb 3,0,7,192,5,160,6,224,1,143,192,112,1,143,128,31
       defb 128,159,128,159,128,31,128,143,128,15,1,131,192,112,0,15
       defb 96,0,112,4,176,33,0,22,0,137,0,84,1,34,0,168
       defb 0,68,3,80,7,136,7,192,13,224,15,192,15,192,7,0
       defb 0,127,129,237,71,160,74,128,42,27,168,109,169,109,219,109
       defb 219,182,21,182,21,176,85,192,82,131,98,11,193,173,0,254

       defb 0,64,0,128,195,128,96,77,32,22,192,222,97,94,225,236
       defb 97,227,128,213,97,39,178,182,243,186,243,216,97,128,0,0
       defb 0,0,0,0,0,0,0,0,237,255,173,109,237,255,237,255
       defb 237,237,173,127,237,255,237,222,146,127,115,128,128,127,0,0
       defb 0,0,3,128,7,223,199,251,226,255,225,255,115,223,243,255
       defb 242,255,241,239,193,126,177,63,248,143,248,115,112,15,0,0
       defb 0,8,0,4,0,2,0,10,0,17,0,33,0,32,192,1
       defb 224,123,240,253,241,254,241,126,225,126,192,189,0,120,0,0
       defb 0,4,0,8,224,99,241,255,147,231,2,66,128,25,128,63
       defb 0,63,0,23,0,18,192,24,241,19,224,239,128,125,0,0
       defb 1,128,3,224,2,208,3,112,128,199,96,56,128,199,192,15
       defb 192,79,192,79,192,15,192,71,192,7,128,193,96,56,128,7
       defb 48,0,56,2,216,16,0,11,128,68,0,42,0,145,0,84
       defb 0,34,1,168,3,196,3,224,6,240,7,224,7,224,3,128
       defb 128,63,192,246,35,208,37,64,149,13,212,54,212,182,237,182
       defb 109,219,10,219,10,216,42,224,169,65,177,5,224,214,0,127

       defb 0,32,0,64,97,192,176,38,16,11,96,111,48,175,112,246
       defb 176,241,192,106,176,147,89,91,121,221,121,236,48,192,0,0
       defb 0,0,0,0,0,0,0,0,246,255,214,182,246,255,246,255
       defb 246,246,214,191,246,255,118,239,201,63,57,192,192,63,0,0
       defb 0,0,1,192,131,239,227,253,241,127,240,255,185,239,249,255
       defb 249,127,248,247,96,191,216,159,252,71,252,57,184,7,0,0
       defb 0,4,0,2,0,1,0,5,128,8,128,16,0,16,224,0
       defb 240,61,248,126,120,255,120,191,112,191,224,94,0,60,0,0
       defb 0,2,0,4,240,49,248,255,201,243,1,33,192,12,192,31
       defb 128,31,128,11,0,9,96,12,248,137,240,119,192,62,0,0
       defb 0,192,1,240,1,104,1,184,192,99,48,28,192,99,224,7
       defb 224,39,224,39,224,7,224,35,224,3,192,96,48,28,192,3
       defb 24,0,28,1,108,8,128,5,64,34,0,21,128,72,0,42
       defb 0,17,0,212,1,226,1,240,3,120,3,240,3,240,1,192
       defb 1,128,3,224,2,208,3,112,128,199,96,56,128,199,192,15
       defb 192,79,192,79,192,15,192,71,192,7,128,193,96,56,128,7

       defb 0,16,0,32,48,224,88,19,136,5,176,55,152,87,56,123
       defb 216,120,96,53,216,73,172,173,188,238,60,246,24,96,0,0
       defb 0,0,0,0,0,0,0,0,251,127,107,91,251,127,251,127
       defb 123,123,235,95,251,127,187,119,228,159,28,224,224,31,0,0
       defb 0,0,0,224,193,247,241,254,248,191,248,127,220,247,252,255
       defb 252,191,252,123,176,95,236,79,254,35,254,28,220,3,0,0
       defb 0,2,0,1,128,0,128,2,64,4,64,8,0,8,112,0
       defb 248,30,124,63,188,127,188,95,184,95,112,47,0,30,0,0
       defb 0,1,0,2,248,24,252,127,228,249,128,144,96,6,224,15
       defb 192,15,192,5,128,4,48,6,252,68,248,59,96,31,0,0
       defb 0,96,0,248,0,180,0,220,224,49,24,14,224,49,240,3
       defb 240,19,240,19,240,3,240,17,240,1,96,48,24,14,224,1
       defb 12,0,142,0,54,4,192,2,32,17,128,10,64,36,0,21
       defb 128,8,0,106,0,241,0,248,1,188,1,248,1,248,0,224
       defb 224,15,176,61,8,244,9,80,101,67,181,13,181,45,187,109
       defb 219,118,194,182,2,182,10,184,106,80,108,65,184,53,192,31

spaw00 defb 0,0,3,192,12,48,16,8,32,8,33,196,66,36,68,36
       defb 68,200,68,8,66,48,33,192,32,1,16,0,12,4,3,176
spaw01 defb 0,0,7,192,24,48,32,8,64,4,67,132,132,66,136,34
       defb 137,34,9,34,132,196,132,4,3,24,64,224,0,0,16,0
spaw02 defb 13,192,32,48,0,8,128,4,3,132,12,66,16,34,19,34
       defb 36,34,36,66,35,132,16,4,16,8,12,48,3,192,0,0
spaw03 defb 0,8,0,0,7,2,24,192,32,33,35,33,68,144,68,145
       defb 68,17,66,33,33,194,32,2,16,4,12,24,3,224,0,0
bee00  defb 40,0,0,0,44,0,86,0,86,0,126,128,49,192,15,0
       defb 80,0,160,32,160,32,160,64,88,128,15,0,0,0,8,0
bee01  defb 20,0,0,0,44,0,86,0,86,0,126,128,60,192,1,128
       defb 2,0,20,0,20,16,20,16,20,16,2,32,5,192,8,0
bee02  defb 0,0,160,0,0,0,88,0,172,0,173,0,253,128,121,0
       defb 2,8,10,4,1,4,1,4,3,136,0,240,2,128,4,0
bee03  defb 0,28,160,162,1,65,89,65,172,65,173,1,253,130,123,132
       defb 7,24,12,0,1,192,3,224,3,224,0,192,2,128,4,0

       defb 0,0,1,224,6,24,8,4,16,4,16,226,33,18,34,18
       defb 34,100,34,4,33,24,16,224,144,0,8,0,6,2,1,216
       defb 0,0,3,224,12,24,16,4,32,2,33,194,66,33,68,17
       defb 68,145,4,145,66,98,66,2,1,140,32,112,0,0,8,0
       defb 6,224,16,24,0,4,64,2,1,194,6,33,8,17,9,145
       defb 18,17,18,33,17,194,8,2,8,4,6,24,1,224,0,0
       defb 0,4,0,0,3,129,12,96,144,16,145,144,34,72,162,72
       defb 162,8,161,16,16,225,16,1,8,2,6,12,1,240,0,0
       defb 20,0,0,0,22,0,43,0,43,0,63,64,24,224,7,128
       defb 40,0,80,16,80,16,80,32,44,64,7,128,0,0,4,0
       defb 10,0,0,0,22,0,43,0,43,0,63,64,30,96,0,192
       defb 1,0,10,0,10,8,10,8,10,8,1,16,2,224,4,0
       defb 0,0,80,0,0,0,44,0,86,0,86,128,126,192,60,128
       defb 1,4,5,2,0,130,0,130,1,196,0,120,1,64,2,0
       defb 0,14,80,81,128,160,172,160,214,32,214,128,126,193,61,194
       defb 3,140,6,0,0,224,1,240,1,240,0,96,1,64,2,0

       defb 0,0,0,240,3,12,4,2,8,2,8,113,16,137,17,9
       defb 17,50,17,2,16,140,8,112,72,0,4,0,3,1,0,236
       defb 0,0,1,240,6,12,8,2,16,1,16,225,161,16,162,8
       defb 162,72,130,72,33,49,33,1,0,198,16,56,0,0,4,0
       defb 3,112,8,12,0,2,32,1,0,225,131,16,132,8,132,200
       defb 137,8,137,16,8,225,4,1,4,2,3,12,0,240,0,0
       defb 0,2,0,0,129,192,6,48,72,8,72,200,17,36,81,36
       defb 81,4,80,136,136,112,136,0,4,1,3,6,0,248,0,0
       defb 10,0,0,0,11,0,21,128,21,128,31,160,12,112,3,192
       defb 20,0,40,8,40,8,40,16,22,32,3,192,0,0,2,0
       defb 5,0,0,0,11,0,21,128,21,128,31,160,15,48,0,96
       defb 0,128,5,0,5,4,5,4,5,4,0,136,1,112,2,0
       defb 0,0,40,0,0,0,22,0,43,0,43,64,63,96,30,64
       defb 0,130,2,129,0,65,0,65,0,226,0,60,0,160,1,0
       defb 0,7,168,40,64,80,86,80,107,16,107,64,191,96,30,225
       defb 1,198,3,0,0,112,0,248,0,248,0,48,0,160,1,0

       defb 0,0,0,120,1,134,2,1,4,1,132,56,136,68,136,132
       defb 8,153,8,129,8,70,4,56,36,0,2,0,129,128,0,118
       defb 0,0,0,248,3,6,4,1,136,0,136,112,80,136,81,4
       defb 81,36,65,36,144,152,144,128,0,99,8,28,0,0,2,0
       defb 1,184,4,6,0,1,144,0,128,112,65,136,66,4,66,100
       defb 68,132,68,136,132,112,130,0,2,1,1,134,0,120,0,0
       defb 0,1,0,0,64,224,3,24,36,4,36,100,8,146,40,146
       defb 40,130,40,68,68,56,68,0,130,0,1,131,0,124,0,0
       defb 5,0,0,0,5,128,10,192,10,192,15,208,6,56,1,224
       defb 10,0,20,4,20,4,20,8,11,16,1,224,0,0,1,0
       defb 2,128,0,0,5,128,10,192,10,192,15,208,7,152,0,48
       defb 0,64,2,128,2,130,2,130,2,130,0,68,0,184,1,0
       defb 0,0,20,0,0,0,11,0,21,128,21,160,31,176,15,32
       defb 0,65,129,64,128,32,128,32,0,113,0,30,0,80,0,128
       defb 128,3,84,20,32,40,43,40,53,136,53,160,95,176,143,112
       defb 0,227,1,128,0,56,0,124,0,124,0,24,0,80,0,128

       defb 0,0,0,60,0,195,129,0,130,0,66,28,68,34,68,66
       defb 132,76,132,64,4,35,2,28,18,0,1,0,64,192,0,59
       defb 0,0,0,124,1,131,130,0,68,0,68,56,40,68,40,130
       defb 40,146,32,146,72,76,72,64,128,49,4,14,0,0,1,0
       defb 0,220,2,3,128,0,72,0,64,56,32,196,33,2,33,50
       defb 34,66,34,68,66,56,65,0,129,0,0,195,0,60,0,0
       defb 128,0,0,0,32,112,1,140,18,2,18,50,4,73,20,73
       defb 20,65,20,34,34,28,34,0,65,0,128,193,0,62,0,0
       defb 2,128,0,0,2,192,5,96,5,96,7,232,3,28,0,240
       defb 5,0,10,2,10,2,10,4,5,136,0,240,0,0,0,128
       defb 1,64,0,0,2,192,5,96,5,96,7,232,3,204,0,24
       defb 0,32,1,64,1,65,1,65,1,65,0,34,0,92,0,128
       defb 0,0,10,0,0,0,5,128,10,192,10,208,15,216,7,144
       defb 128,32,64,160,64,16,64,16,128,56,0,15,0,40,0,64
       defb 192,1,42,10,16,20,21,148,26,196,26,208,47,216,71,184
       defb 128,113,0,192,0,28,0,62,0,62,0,12,0,40,0,64

       defb 0,0,0,30,128,97,64,128,65,0,33,14,34,17,34,33
       defb 66,38,66,32,130,17,1,14,9,0,0,128,32,96,128,29
       defb 0,0,0,62,128,193,65,0,34,0,34,28,20,34,20,65
       defb 20,73,16,73,36,38,36,32,192,24,2,7,0,0,0,128
       defb 0,110,129,1,64,0,36,0,32,28,16,98,16,129,16,153
       defb 17,33,17,34,33,28,32,128,64,128,128,97,0,30,0,0
       defb 64,0,0,0,16,56,0,198,9,1,9,25,130,36,138,36
       defb 138,32,10,17,17,14,17,0,32,128,192,96,0,31,0,0
       defb 1,64,0,0,1,96,2,176,2,176,3,244,1,142,0,120
       defb 2,128,5,1,5,1,5,2,2,196,0,120,0,0,0,64
       defb 0,160,0,0,1,96,2,176,2,176,3,244,1,230,0,12
       defb 0,16,0,160,128,160,128,160,128,160,0,17,0,46,0,64
       defb 0,0,5,0,0,0,2,192,5,96,5,104,7,236,3,200
       defb 64,16,32,80,32,8,32,8,64,28,128,7,0,20,0,32
       defb 224,0,21,5,8,10,10,202,13,98,13,104,23,236,35,220
       defb 192,56,0,96,0,14,0,31,0,31,0,6,0,20,0,32

       defb 0,0,0,15,192,48,32,64,32,128,16,135,145,8,145,16
       defb 33,19,33,16,193,8,0,135,4,128,0,64,16,48,192,14
       defb 0,0,0,31,192,96,32,128,17,0,17,14,10,17,138,32
       defb 138,36,136,36,18,19,18,16,96,12,129,3,0,0,0,64
       defb 0,55,192,128,32,0,18,0,16,14,8,49,136,64,136,76
       defb 136,144,8,145,16,142,16,64,32,64,192,48,0,15,0,0
       defb 32,0,0,0,8,28,0,99,132,128,132,140,65,18,69,18
       defb 69,16,133,8,8,135,8,128,16,64,96,48,128,15,0,0
       defb 0,160,0,0,0,176,1,88,1,88,1,250,0,199,0,60
       defb 1,64,130,128,130,128,2,129,1,98,0,60,0,0,0,32
       defb 0,80,0,0,0,176,1,88,1,88,1,250,0,243,0,6
       defb 0,8,0,80,64,80,64,80,64,80,128,8,0,23,0,32
       defb 0,0,2,128,0,0,1,96,2,176,2,180,3,246,1,228
       defb 32,8,16,40,16,4,16,4,32,14,192,3,0,10,0,16
       defb 112,0,138,130,4,5,5,101,6,177,6,180,11,246,17,238
       defb 96,28,0,48,0,7,128,15,128,15,0,3,0,10,0,16

       defb 0,0,128,7,96,24,16,32,16,64,136,67,72,132,72,136
       defb 144,137,16,136,96,132,128,67,2,64,0,32,8,24,96,7
       defb 0,0,128,15,96,48,16,64,8,128,8,135,133,8,69,16
       defb 69,18,68,18,137,9,9,8,48,6,192,129,0,0,0,32
       defb 128,27,96,64,16,0,9,0,8,7,132,24,68,32,68,38
       defb 68,72,132,72,8,71,8,32,16,32,96,24,128,7,0,0
       defb 16,0,0,0,4,14,128,49,66,64,66,70,32,137,34,137
       defb 34,136,66,132,132,67,4,64,8,32,48,24,192,7,0,0
       defb 0,80,0,0,0,88,0,172,0,172,0,253,128,99,0,30
       defb 0,160,65,64,65,64,129,64,0,177,0,30,0,0,0,16
       defb 0,40,0,0,0,88,0,172,0,172,0,253,128,121,0,3
       defb 0,4,0,40,32,40,32,40,32,40,64,4,128,11,0,16
       defb 0,0,1,64,0,0,0,176,1,88,1,90,1,251,0,242
       defb 16,4,8,20,8,2,8,2,16,7,224,1,0,5,0,8
       defb 56,0,69,65,130,2,130,178,131,88,3,90,5,251,8,247
       defb 48,14,0,24,128,3,192,7,192,7,128,1,0,5,0,8

bee04  defb 56,0,69,5,130,128,130,154,130,53,128,181,65,191,33,222
       defb 24,224,0,48,3,128,7,192,7,192,3,0,1,64,0,32
bee05  defb 0,0,0,5,0,0,0,26,0,53,0,181,1,191,0,158
       defb 16,64,32,80,32,128,32,128,17,192,15,0,1,64,0,32
bee06  defb 0,40,0,0,0,52,0,106,0,106,1,126,3,60,1,128
       defb 0,64,0,40,8,40,8,40,8,40,4,64,3,160,0,16
bee07  defb 0,20,0,0,0,52,0,106,0,106,1,126,3,140,0,240
       defb 0,10,4,5,4,5,2,5,1,26,0,240,0,0,0,16
appl00 defb 0,128,1,0,12,68,63,50,127,138,127,193,255,225,255,227
       defb 191,199,191,159,95,255,95,254,47,254,31,188,7,112,0,0
pear00 defb 3,0,54,128,16,0,103,0,111,192,207,240,205,252,79,190
       defb 7,254,7,253,5,191,5,247,2,226,2,118,1,156,0,112
strw00 defb 6,0,100,0,113,80,34,168,5,244,202,232,133,116,42,168
       defb 5,84,34,170,1,84,32,170,16,20,8,10,1,84,0,0
nana00 defb 0,0,28,0,18,0,34,0,34,0,115,0,127,0,111,128
       defb 111,224,127,254,119,250,59,246,63,60,31,248,7,224,0,0

       defb 28,0,162,130,65,64,65,77,193,26,192,90,160,223,16,239
       defb 12,112,0,24,1,192,3,224,3,224,1,128,0,160,0,16
       defb 0,0,128,2,0,0,0,13,128,26,128,90,128,223,0,79
       defb 8,32,16,40,16,64,16,64,8,224,7,128,0,160,0,16
       defb 0,20,0,0,0,26,0,53,0,53,0,191,1,158,0,192
       defb 0,32,0,20,4,20,4,20,4,20,2,32,1,208,0,8
       defb 0,10,0,0,0,26,0,53,0,53,0,191,1,198,0,120
       defb 0,5,130,2,130,2,129,2,0,141,0,120,0,0,0,8
       defb 0,64,0,128,6,34,31,153,63,197,191,224,255,240,255,241
       defb 223,227,223,207,175,255,47,255,23,255,15,222,3,184,0,0
       defb 1,128,27,64,8,0,51,128,55,224,103,248,102,254,39,223
       defb 3,255,131,254,130,223,130,251,1,113,1,59,0,206,0,56
       defb 3,0,50,0,56,168,17,84,2,250,101,116,66,186,21,84
       defb 2,170,17,85,0,170,16,85,8,10,4,5,0,170,0,0
       defb 0,0,14,0,9,0,17,0,17,0,57,128,63,128,55,192
       defb 55,240,63,255,59,253,29,251,31,158,15,252,3,240,0,0

       defb 14,0,81,65,32,160,160,166,96,141,96,45,208,111,136,119
       defb 6,56,0,12,0,224,1,240,1,240,0,192,0,80,0,8
       defb 0,0,64,1,0,0,128,6,64,13,64,45,192,111,128,39
       defb 4,16,8,20,8,32,8,32,4,112,3,192,0,80,0,8
       defb 0,10,0,0,0,13,128,26,128,26,128,95,0,207,0,96
       defb 0,16,0,10,2,10,2,10,2,10,1,16,0,232,0,4
       defb 0,5,0,0,0,13,128,26,128,26,128,95,0,227,0,60
       defb 128,2,65,1,65,1,64,129,128,70,0,60,0,0,0,4
       defb 0,32,0,64,3,17,143,204,159,226,95,240,127,248,255,248
       defb 239,241,239,231,215,255,151,255,139,255,7,239,1,220,0,0
       defb 0,192,13,160,4,0,25,192,27,240,51,252,51,127,147,239
       defb 129,255,65,255,193,111,193,125,128,184,128,157,0,103,0,28
       defb 1,128,25,0,28,84,8,170,1,125,50,186,33,93,10,170
       defb 1,85,136,170,0,85,136,42,4,5,130,2,0,85,0,0
       defb 0,0,7,0,4,128,8,128,8,128,28,192,31,192,27,224
       defb 27,248,159,255,157,254,142,253,15,207,7,254,1,248,0,0

       defb 7,0,168,160,16,80,80,83,176,70,176,22,232,55,196,59
       defb 3,28,0,6,0,112,0,248,0,248,0,96,0,40,0,4
       defb 0,0,160,0,0,0,64,3,160,6,160,22,224,55,192,19
       defb 2,8,4,10,4,16,4,16,2,56,1,224,0,40,0,4
       defb 0,5,0,0,128,6,64,13,64,13,192,47,128,103,0,48
       defb 0,8,0,5,1,5,1,5,1,5,0,136,0,116,0,2
       defb 128,2,0,0,128,6,64,13,64,13,192,47,128,113,0,30
       defb 64,1,160,128,160,128,160,64,64,35,0,30,0,0,0,2
       defb 0,16,0,32,129,136,71,230,79,241,47,248,63,252,127,252
       defb 247,248,247,243,235,255,203,255,197,255,131,247,0,238,0,0
       defb 0,96,6,208,2,0,12,224,13,248,25,254,153,191,201,247
       defb 192,255,160,255,224,183,224,190,64,92,192,78,128,51,0,14
       defb 0,192,12,128,14,42,4,85,128,190,25,93,144,174,5,85
       defb 128,170,68,85,128,42,68,21,130,2,65,1,128,42,0,0
       defb 0,0,3,128,2,64,4,64,4,64,14,96,15,224,13,240
       defb 13,252,207,255,78,255,199,126,135,231,3,255,0,252,0,0

       defb 3,128,84,80,8,40,168,41,88,35,88,11,244,27,226,29
       defb 1,142,0,3,0,56,0,124,0,124,0,48,0,20,0,2
       defb 0,0,80,0,0,0,160,1,80,3,80,11,240,27,224,9
       defb 1,4,2,5,2,8,2,8,1,28,0,240,0,20,0,2
       defb 128,2,0,0,64,3,160,6,160,6,224,23,192,51,0,24
       defb 0,4,128,2,128,130,128,130,128,130,0,68,0,58,0,1
       defb 64,1,0,0,64,3,160,6,160,6,224,23,192,56,0,15
       defb 160,0,80,64,80,64,80,32,160,17,0,15,0,0,0,1
       defb 0,8,0,16,64,196,35,243,167,248,23,252,31,254,63,254
       defb 123,252,251,249,245,255,229,255,226,255,193,251,0,119,0,0
       defb 0,48,3,104,1,0,6,112,6,252,12,255,204,223,228,251
       defb 224,127,208,127,240,91,112,95,32,46,96,39,192,25,0,7
       defb 0,96,6,64,7,21,130,42,64,95,140,174,72,87,130,170
       defb 64,85,162,42,64,21,162,10,65,1,160,128,64,21,0,0
       defb 0,0,1,192,1,32,2,32,2,32,7,48,7,240,6,248
       defb 6,254,231,255,167,127,99,191,195,243,129,255,0,126,0,0

       defb 1,192,42,40,4,20,212,20,172,17,172,5,250,13,241,14
       defb 0,199,128,1,0,28,0,62,0,62,0,24,0,10,0,1
       defb 0,0,40,0,0,0,208,0,168,1,168,5,248,13,240,4
       defb 0,130,129,2,1,4,1,4,0,142,0,120,0,10,0,1
       defb 64,1,0,0,160,1,80,3,80,3,240,11,224,25,0,12
       defb 0,2,64,1,64,65,64,65,64,65,0,34,0,29,128,0
       defb 160,0,0,0,160,1,80,3,80,3,240,11,96,28,128,7
       defb 80,0,40,32,40,32,40,16,208,8,128,7,0,0,128,0
       defb 0,4,0,8,32,98,145,249,83,252,11,254,15,255,31,255
       defb 61,254,253,252,250,255,242,255,241,127,224,253,128,59,0,0
       defb 0,24,1,180,0,128,3,56,3,126,134,127,230,111,242,125
       defb 240,63,232,63,248,45,184,47,16,23,176,19,224,12,128,3
       defb 0,48,3,32,131,138,65,21,160,47,70,87,164,43,65,85
       defb 160,42,81,21,160,10,81,5,160,128,80,64,160,10,0,0
       defb 0,0,0,224,0,144,1,16,1,16,3,152,3,248,3,124
       defb 3,127,243,255,211,191,177,223,225,249,192,255,0,63,0,0

       defb 0,224,21,20,2,10,106,10,214,8,214,2,253,6,120,135
       defb 128,99,192,0,0,14,0,31,0,31,0,12,0,5,128,0
       defb 0,0,20,0,0,0,104,0,212,0,212,2,252,6,120,2
       defb 0,65,64,129,0,130,0,130,0,71,0,60,0,5,128,0
       defb 160,0,0,0,208,0,168,1,168,1,248,5,240,12,0,6
       defb 0,1,160,0,160,32,160,32,160,32,0,17,128,14,64,0
       defb 80,0,0,0,208,0,168,1,168,1,248,5,48,14,192,3
       defb 40,0,20,16,20,16,20,8,104,4,192,3,0,0,64,0
       defb 0,2,0,4,16,49,200,252,41,254,5,255,135,255,143,255
       defb 30,255,126,254,253,127,249,127,248,191,240,126,192,29,0,0
       defb 0,12,0,218,0,64,1,156,1,191,195,63,243,55,249,62
       defb 248,31,244,31,252,22,220,23,136,11,216,9,112,6,192,1
       defb 0,24,1,144,65,197,160,138,208,23,163,43,210,21,160,170
       defb 80,21,168,138,80,5,168,130,80,64,40,32,80,5,0,0
       defb 0,0,0,112,0,72,0,136,0,136,1,204,1,252,1,190
       defb 129,191,249,255,233,223,216,239,240,252,224,127,128,31,0,0

       defb 0,112,10,138,1,5,53,5,107,4,107,1,126,131,188,67
       defb 192,49,96,0,0,7,128,15,128,15,0,6,128,2,64,0
       defb 0,0,10,0,0,0,52,0,106,0,106,1,126,3,60,1
       defb 128,32,160,64,0,65,0,65,128,35,0,30,128,2,64,0
       defb 80,0,0,0,104,0,212,0,212,0,252,2,120,6,0,3
       defb 128,0,80,0,80,16,80,16,80,16,128,8,64,7,32,0
       defb 40,0,0,0,104,0,212,0,212,0,252,2,24,7,224,1
       defb 20,0,10,8,10,8,10,4,52,2,224,1,0,0,32,0
       defb 0,1,0,2,136,24,100,126,20,255,130,255,195,255,199,255
       defb 143,127,63,127,254,191,252,191,252,95,120,63,224,14,0,0
       defb 0,6,0,109,0,32,0,206,128,223,225,159,249,155,124,159
       defb 252,15,250,15,126,11,238,11,196,5,236,4,56,3,224,0
       defb 0,12,0,200,160,226,80,69,232,11,209,149,233,10,80,85
       defb 168,10,84,69,168,2,84,65,40,32,20,16,168,2,0,0
       defb 0,0,0,56,0,36,0,68,0,68,0,230,0,254,0,223
       defb 192,223,252,255,244,239,236,119,120,126,240,63,192,15,0,0

bat4   defb 0,88,22,166,46,65,42,66,120,100,45,100,253,194,121,193
       defb 3,130,31,12,31,16,14,32,6,64,0,0,2,128,0,0
bat5   defb 0,0,22,0,46,0,42,0,120,0,45,0,253,176,121,136
       defb 3,4,27,8,29,4,13,2,6,140,0,112,2,128,0,0
bat6   defb 11,0,23,0,21,0,60,0,22,0,126,128,60,192,1,160
       defb 11,0,22,8,20,8,20,4,2,20,1,40,5,192,0,0
bat7   defb 11,0,23,0,21,0,60,0,22,0,126,128,49,192,15,128
       defb 88,0,160,16,160,16,163,32,86,160,40,64,5,0,0,0
bat0   defb 26,0,101,104,130,116,66,84,38,30,38,180,67,191,131,158
       defb 65,192,48,248,8,248,4,112,2,96,0,0,1,64,0,0
bat1   defb 0,0,0,104,0,116,0,84,0,30,0,180,13,191,17,158
       defb 32,192,16,216,32,184,64,176,49,96,14,0,1,64,0,0
bat2   defb 0,208,0,232,0,168,0,60,0,104,1,126,3,60,5,128
       defb 0,208,16,104,16,40,32,40,40,64,20,128,3,160,0,0
bat3   defb 0,208,0,232,0,168,0,60,0,104,1,126,3,140,1,240
       defb 0,26,8,5,8,5,4,197,5,106,2,20,0,160,0,0

       defb 0,44,11,83,151,32,21,33,60,50,22,178,126,225,188,224
       defb 1,193,15,134,15,136,7,16,3,32,0,0,1,64,0,0
       defb 0,0,11,0,23,0,21,0,60,0,22,128,126,216,60,196
       defb 1,130,13,132,14,130,6,129,3,70,0,56,1,64,0,0
       defb 5,128,11,128,10,128,30,0,11,0,63,64,30,96,0,208
       defb 5,128,11,4,10,4,10,2,1,10,0,148,2,224,0,0
       defb 5,128,11,128,10,128,30,0,11,0,63,64,24,224,7,192
       defb 44,0,80,8,80,8,81,144,43,80,20,32,2,128,0,0
       defb 13,0,50,180,65,58,33,42,19,15,19,90,161,223,65,207
       defb 32,224,24,124,4,124,2,56,1,48,0,0,0,160,0,0
       defb 0,0,0,52,0,58,0,42,0,15,0,90,134,223,8,207
       defb 16,96,8,108,16,92,32,88,24,176,7,0,0,160,0,0
       defb 0,104,0,116,0,84,0,30,0,52,0,191,1,158,2,192
       defb 0,104,8,52,8,20,16,20,20,32,10,64,1,208,0,0
       defb 0,104,0,116,0,84,0,30,0,52,0,191,1,198,0,248
       defb 0,13,132,2,132,2,130,98,2,181,1,10,0,80,0,0

       defb 0,22,133,169,75,144,138,144,30,25,11,89,191,112,94,112
       defb 128,224,7,195,7,196,3,136,1,144,0,0,0,160,0,0
       defb 0,0,5,128,11,128,10,128,30,0,11,64,63,108,30,98
       defb 0,193,6,194,7,65,131,64,1,163,0,28,0,160,0,0
       defb 2,192,5,192,5,64,15,0,5,128,31,160,15,48,0,104
       defb 2,192,5,130,5,2,5,1,0,133,0,74,1,112,0,0
       defb 2,192,5,192,5,64,15,0,5,128,31,160,12,112,3,224
       defb 22,0,40,4,40,4,40,200,21,168,10,16,1,64,0,0
       defb 6,128,25,90,32,157,16,149,137,135,9,173,208,239,160,231
       defb 16,112,12,62,2,62,1,28,0,152,0,0,0,80,0,0
       defb 0,0,0,26,0,29,0,21,128,7,0,45,195,111,132,103
       defb 8,48,4,54,8,46,16,44,12,88,3,128,0,80,0,0
       defb 0,52,0,58,0,42,0,15,0,26,128,95,0,207,1,96
       defb 0,52,4,26,4,10,8,10,10,16,5,32,0,232,0,0
       defb 0,52,0,58,0,42,0,15,0,26,128,95,0,227,0,124
       defb 128,6,66,1,66,1,65,49,129,90,0,133,0,40,0,0

       defb 0,11,194,212,37,200,69,72,143,12,133,172,95,184,47,56
       defb 64,112,131,225,3,226,1,196,0,200,0,0,0,80,0,0
       defb 0,0,2,192,5,192,5,64,15,0,5,160,31,182,15,49
       defb 128,96,3,97,131,160,65,160,128,209,0,14,0,80,0,0
       defb 1,96,2,224,2,160,7,128,2,192,15,208,7,152,0,52
       defb 1,96,2,193,2,129,130,128,128,66,0,37,0,184,0,0
       defb 1,96,2,224,2,160,7,128,2,192,15,208,6,56,1,240
       defb 11,0,20,2,20,2,20,100,10,212,5,8,0,160,0,0
       defb 3,64,12,173,144,78,136,74,196,195,132,214,232,119,208,115
       defb 8,56,6,31,1,31,0,142,0,76,0,0,0,40,0,0
       defb 0,0,0,13,128,14,128,10,192,3,128,22,225,183,194,51
       defb 4,24,2,27,4,23,8,22,6,44,1,192,0,40,0,0
       defb 0,26,0,29,0,21,128,7,0,13,192,47,128,103,0,176
       defb 0,26,2,13,2,5,4,5,5,8,2,144,0,116,0,0
       defb 0,26,0,29,0,21,128,7,0,13,192,47,128,113,0,62
       defb 64,3,161,0,161,0,160,152,64,173,128,66,0,20,0,0

       defb 128,5,97,106,18,228,34,164,71,134,66,214,47,220,23,156
       defb 32,56,193,240,1,241,0,226,0,100,0,0,0,40,0,0
       defb 0,0,1,96,2,224,2,160,7,128,2,208,15,219,135,152
       defb 64,48,129,176,65,208,32,208,192,104,0,7,0,40,0,0
       defb 0,176,1,112,1,80,3,192,1,96,7,232,3,204,0,26
       defb 0,176,129,96,129,64,65,64,64,33,128,18,0,92,0,0
       defb 0,176,1,112,1,80,3,192,1,96,7,232,3,28,0,248
       defb 5,128,10,1,10,1,10,50,5,106,2,132,0,80,0,0
       defb 1,160,134,86,72,39,68,37,226,97,66,107,244,59,232,57
       defb 4,28,131,15,128,143,0,71,0,38,0,0,0,20,0,0
       defb 0,0,128,6,64,7,64,5,224,1,64,11,240,219,225,25
       defb 2,12,129,13,130,11,4,11,3,22,0,224,0,20,0,0
       defb 0,13,128,14,128,10,192,3,128,6,224,23,192,51,0,88
       defb 0,13,129,6,129,2,130,2,2,132,1,72,0,58,0,0
       defb 0,13,128,14,128,10,192,3,128,6,224,23,192,56,0,31
       defb 160,1,80,128,80,128,80,76,160,86,64,33,0,10,0,0

       defb 192,2,48,181,9,114,17,82,35,195,33,107,23,238,11,206
       defb 16,28,96,248,128,248,0,113,0,50,0,0,0,20,0,0
       defb 0,0,0,176,1,112,1,80,3,192,1,104,135,237,67,204
       defb 32,24,64,216,32,232,16,104,96,52,128,3,0,20,0,0
       defb 0,88,0,184,0,168,1,224,0,176,3,244,1,230,0,13
       defb 0,88,64,176,64,160,32,160,160,16,64,9,0,46,0,0
       defb 0,88,0,184,0,168,1,224,0,176,3,244,1,142,0,124
       defb 2,192,133,0,133,0,5,25,2,181,1,66,0,40,0,0
       defb 0,208,67,43,164,19,162,18,241,48,161,53,250,29,244,28
       defb 2,14,193,135,192,71,128,35,0,19,0,0,0,10,0,0
       defb 0,0,64,3,160,3,160,2,240,0,160,5,248,109,240,140
       defb 1,6,192,134,193,5,130,5,1,139,0,112,0,10,0,0
       defb 128,6,64,7,64,5,224,1,64,3,240,11,224,25,0,44
       defb 128,6,64,131,64,129,65,1,1,66,0,164,0,29,0,0
       defb 128,6,64,7,64,5,224,1,64,3,240,11,96,28,128,15
       defb 208,0,40,64,40,64,40,38,80,43,160,16,0,5,0,0

       defb 96,1,152,90,4,185,8,169,145,225,144,181,11,247,5,231
       defb 8,14,48,124,64,124,128,56,0,25,0,0,0,10,0,0
       defb 0,0,0,88,0,184,0,168,1,224,0,180,195,246,33,230
       defb 16,12,32,108,16,116,8,52,48,26,192,1,0,10,0,0
       defb 0,44,0,92,0,84,0,240,0,88,1,250,0,243,128,6
       defb 0,44,32,88,32,80,16,80,80,8,160,4,0,23,0,0
       defb 0,44,0,92,0,84,0,240,0,88,1,250,0,199,0,62
       defb 1,96,66,128,66,128,130,140,129,90,0,161,0,20,0,0
       defb 0,104,161,149,210,9,81,9,120,152,208,154,253,14,122,14
       defb 1,7,224,195,224,35,192,17,128,9,0,0,0,5,0,0
       defb 0,0,160,1,208,1,80,1,120,0,208,2,252,54,120,70
       defb 0,131,96,67,224,130,193,2,128,197,0,56,0,5,0,0
       defb 64,3,160,3,160,2,240,0,160,1,248,5,240,12,0,22
       defb 64,3,160,65,160,64,160,128,0,161,0,82,128,14,0,0
       defb 64,3,160,3,160,2,240,0,160,1,248,5,48,14,192,7
       defb 104,0,20,32,20,32,20,19,168,21,80,8,128,2,0,0

       defb 176,0,76,45,130,92,132,84,200,240,200,90,133,251,130,243
       defb 4,7,24,62,32,62,64,28,128,12,0,0,0,5,0,0
       defb 0,0,0,44,0,92,0,84,0,240,0,90,97,251,16,243
       defb 8,6,16,54,8,58,4,26,24,13,224,0,0,5,0,0
       defb 0,22,0,46,0,42,0,120,0,44,0,253,128,121,64,3
       defb 0,22,16,44,16,40,8,40,40,4,80,2,128,11,0,0
       defb 0,22,0,46,0,42,0,120,0,44,0,253,128,99,0,31
       defb 0,176,33,64,33,64,65,70,64,173,128,80,0,10,0,0
       defb 0,52,208,202,233,4,168,132,60,76,104,77,126,135,61,7
       defb 128,131,240,97,240,17,224,8,192,4,0,0,128,2,0,0
       defb 0,0,208,0,232,0,168,0,60,0,104,1,126,27,60,35
       defb 128,65,176,33,112,65,96,129,192,98,0,28,128,2,0,0
       defb 160,1,208,1,80,1,120,0,208,0,252,2,120,6,0,11
       defb 160,1,208,32,80,32,80,64,128,80,0,41,64,7,0,0
       defb 160,1,208,1,80,1,120,0,208,0,252,2,24,7,224,3
       defb 52,0,10,16,10,16,138,9,212,10,40,4,64,1,0,0

bird0  defb 0,0,73,0,200,248,205,212,229,208,121,246,189,239,30,236
       defb 47,240,63,192,29,0,0,0,9,0,27,0,9,0,0,0
bird1  defb 0,0,1,0,0,248,1,212,1,208,193,246,141,239,62,236
       defb 63,240,123,192,117,0,96,0,201,0,27,0,9,0,0,0
bird2  defb 0,0,1,0,0,248,1,212,1,208,193,246,156,239,63,108
       defb 63,144,55,228,25,250,0,60,9,0,27,0,9,0,0,0
gem    defb 0,0,0,0,0,0,15,240,19,232,33,212,64,170,255,127
       defb 138,213,69,234,34,244,17,232,8,208,5,160,2,64,1,128
bird4  defb 0,0,0,146,31,19,43,179,11,167,111,158,247,189,55,120
       defb 15,244,3,252,0,184,0,0,0,144,0,216,0,144,0,0
bird5  defb 0,0,0,128,31,0,43,128,11,128,111,131,247,177,55,124
       defb 15,252,3,222,0,174,0,6,0,147,0,216,0,144,0,0
bird6  defb 0,0,0,128,31,0,43,128,11,128,111,131,247,57,54,252
       defb 9,252,39,236,95,152,60,0,0,144,0,216,0,144,0,0
       defw 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0    ; filler.

       defb 0,0,36,128,100,124,102,234,114,232,60,251,222,247,15,118
       defb 23,248,31,224,14,128,0,0,4,128,13,128,4,128,0,0
       defb 0,0,0,128,0,124,0,234,0,232,96,251,198,247,31,118
       defb 31,248,61,224,58,128,48,0,100,128,13,128,4,128,0,0
       defb 0,0,0,128,0,124,0,234,0,232,96,251,206,119,31,182
       defb 31,200,27,242,12,253,0,30,4,128,13,128,4,128,0,0
       defb 0,0,0,0,0,0,7,248,9,244,16,234,32,85,255,191
       defb 197,106,34,245,17,122,8,244,4,104,2,208,1,32,0,192
       defb 0,0,0,73,143,137,149,217,133,211,55,207,251,222,27,188
       defb 7,250,1,254,0,92,0,0,0,72,0,108,0,72,0,0
       defb 0,0,0,64,15,128,21,192,5,192,183,193,251,216,27,190
       defb 7,254,1,239,0,87,0,3,128,73,0,108,0,72,0,0
       defb 0,0,0,64,15,128,21,192,5,192,183,193,251,156,27,126
       defb 4,254,19,246,47,204,30,0,0,72,0,108,0,72,0,0
       defw 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0    ; filler.

       defb 0,0,18,64,50,62,51,117,57,116,158,125,239,123,7,187
       defb 11,252,15,240,7,64,0,0,2,64,6,192,2,64,0,0
       defb 0,0,0,64,0,62,0,117,0,116,176,125,227,123,15,187
       defb 15,252,30,240,29,64,24,0,50,64,6,192,2,64,0,0
       defb 0,0,0,64,0,62,0,117,0,116,176,125,231,59,15,219
       defb 15,228,13,249,134,126,0,15,2,64,6,192,2,64,0,0
       defb 0,0,0,0,0,0,3,252,4,250,8,117,144,42,255,223
       defb 98,181,145,122,8,189,4,122,2,52,1,104,0,144,0,96
       defb 0,0,128,36,199,196,202,236,194,233,155,231,125,239,13,222
       defb 3,253,0,255,0,46,0,0,0,36,0,54,0,36,0,0
       defb 0,0,0,32,7,192,10,224,2,224,219,224,125,236,13,223
       defb 3,255,128,247,128,43,128,1,192,36,0,54,0,36,0,0
       defb 0,0,0,32,7,192,10,224,2,224,219,224,125,206,13,191
       defb 2,127,9,251,23,230,15,0,0,36,0,54,0,36,0,0
       defw 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0    ; filler.

       defb 0,0,9,32,25,31,153,186,28,186,207,62,247,189,131,221
       defb 5,254,7,248,3,160,0,0,1,32,3,96,1,32,0,0
       defb 0,0,0,32,0,31,128,58,0,58,216,62,241,189,135,221
       defb 7,254,15,120,14,160,12,0,25,32,3,96,1,32,0,0
       defb 0,0,0,32,0,31,128,58,0,58,216,62,243,157,135,237
       defb 7,242,134,252,67,63,128,7,1,32,3,96,1,32,0,0
       defb 0,0,0,0,0,0,1,254,2,125,132,58,72,21,255,239
       defb 177,90,72,189,132,94,2,61,1,26,0,180,0,72,0,48
       defb 0,0,64,18,99,226,101,118,225,116,205,243,190,247,6,239
       defb 129,254,128,127,0,23,0,0,0,18,0,27,0,18,0,0
       defb 0,0,0,16,3,224,5,112,1,112,109,240,62,246,134,239
       defb 129,255,192,123,192,21,192,0,96,18,0,27,0,18,0,0
       defb 0,0,0,16,3,224,5,112,1,112,109,240,62,231,134,223
       defb 129,63,132,253,11,243,7,128,0,18,0,27,0,18,0,0
       defw 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0    ; filler.

       defb 0,0,4,144,140,143,76,221,14,93,103,159,251,222,193,238
       defb 2,255,3,252,1,208,0,0,0,144,1,176,0,144,0,0
       defb 0,0,0,16,128,15,64,29,0,29,108,31,248,222,195,238
       defb 3,255,7,188,7,80,6,0,12,144,1,176,0,144,0,0
       defb 0,0,0,16,128,15,64,29,0,29,108,31,249,206,195,246
       defb 3,249,67,126,161,159,192,3,0,144,1,176,0,144,0,0
       defb 0,0,0,0,0,0,0,255,129,62,66,29,164,10,255,247
       defb 88,173,164,94,66,47,129,30,0,141,0,90,0,36,0,24
       defb 0,0,32,9,49,241,50,187,112,186,230,249,223,123,131,119
       defb 64,255,192,63,128,11,0,0,0,9,128,13,0,9,0,0
       defb 0,0,0,8,1,240,2,184,0,184,54,248,31,123,195,119
       defb 192,255,224,61,224,10,96,0,48,9,128,13,0,9,0,0
       defb 0,0,0,8,1,240,2,184,0,184,54,248,159,115,195,111
       defb 192,159,194,126,133,249,3,192,0,9,128,13,0,9,0,0
       defw 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0    ; filler.

       defb 0,0,2,72,198,71,166,110,135,46,179,207,125,239,96,247
       defb 129,127,1,254,0,232,0,0,0,72,0,216,0,72,0,0
       defb 0,0,0,8,192,7,160,14,128,14,182,15,124,111,97,247
       defb 129,255,3,222,3,168,3,0,6,72,0,216,0,72,0,0
       defb 0,0,0,8,192,7,160,14,128,14,182,15,124,231,97,251
       defb 129,252,33,191,208,207,224,1,0,72,0,216,0,72,0,0
       defb 0,0,0,0,0,0,128,127,64,159,161,14,82,5,255,251
       defb 172,86,82,47,161,23,64,143,128,70,0,45,0,18,0,12
       defb 0,0,144,4,152,248,153,93,56,93,243,124,239,189,193,187
       defb 160,127,224,31,192,5,0,0,128,4,192,6,128,4,0,0
       defb 0,0,0,4,0,248,1,92,0,92,27,124,143,189,225,187
       defb 224,127,240,30,112,5,48,0,152,4,192,6,128,4,0,0
       defb 0,0,0,4,0,248,1,92,0,92,27,124,207,185,225,183
       defb 224,79,97,63,194,252,1,224,128,4,192,6,128,4,0,0
       defw 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0    ; filler.

       defb 0,0,1,36,227,35,83,55,67,151,217,231,190,247,176,123
       defb 192,191,0,255,0,116,0,0,0,36,0,108,0,36,0,0
       defb 0,0,0,4,224,3,80,7,64,7,219,7,190,55,176,251
       defb 192,255,1,239,1,212,1,128,3,36,0,108,0,36,0,0
       defb 0,0,0,4,224,3,80,7,64,7,219,7,190,115,176,253
       defb 64,254,144,223,232,103,240,0,0,36,0,108,0,36,0,0
       defb 0,0,0,0,0,0,192,63,160,79,80,135,169,2,255,253
       defb 86,43,169,23,208,139,160,71,64,35,128,22,0,9,0,6
       defb 0,0,72,2,76,124,204,174,156,46,121,190,247,222,224,221
       defb 208,63,240,15,224,2,0,0,64,2,96,3,64,2,0,0
       defb 0,0,0,2,0,124,0,174,0,46,13,190,199,222,240,221
       defb 240,63,120,15,184,2,24,0,76,2,96,3,64,2,0,0
       defb 0,0,0,2,0,124,0,174,0,46,13,190,231,220,240,219
       defb 240,39,176,159,97,126,0,240,64,2,96,3,64,2,0,0
       defw 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0    ; filler.

       defb 0,0,0,146,241,145,169,155,161,203,236,243,223,123,216,61
       defb 224,95,128,127,0,58,0,0,0,18,0,54,0,18,0,0
       defb 0,0,0,2,240,1,168,3,160,3,237,131,223,27,216,125
       defb 224,127,128,247,0,234,0,192,1,146,0,54,0,18,0,0
       defb 0,0,0,2,240,1,168,3,160,3,237,131,223,57,216,126
       defb 32,127,200,111,244,51,120,0,0,18,0,54,0,18,0,0
       defb 0,0,0,0,0,0,224,31,208,39,168,67,84,129,255,254
       defb 171,21,212,139,232,69,208,35,160,17,64,11,128,4,0,3
       defb 0,0,36,1,38,62,102,87,78,23,60,223,123,239,240,110
       defb 232,31,248,7,112,1,0,0,32,1,176,1,32,1,0,0
       defb 0,0,0,1,0,62,0,87,0,23,6,223,99,239,248,110
       defb 248,31,188,7,92,1,12,0,38,1,176,1,32,1,0,0
       defb 0,0,0,1,0,62,0,87,0,23,6,223,115,238,248,109
       defb 248,19,216,79,48,191,0,120,32,1,176,1,32,1,0,0
       defw 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0    ; filler.



drag0  defb 27,184,101,192,130,244,66,234,34,232,38,179,70,79,131,122
       defb 65,188,49,0,11,176,7,0,204,120,143,0,88,48,54,132
drag1  defb 3,184,1,192,0,244,1,234,7,232,3,179,1,79,12,122
       defb 16,188,32,192,16,208,32,128,193,184,179,0,78,48,54,132
drag2  defb 3,184,1,192,0,244,1,234,7,232,3,179,6,79,11,122
       defb 1,188,0,192,16,112,16,32,224,56,168,64,84,176,55,132
drag3  defb 3,184,1,192,0,244,1,234,7,232,2,179,1,79,3,138
       defb 1,244,0,24,8,5,8,5,196,197,141,106,90,48,54,132
drag4  defb 29,216,3,166,47,65,87,66,23,68,205,100,242,98,94,193
       defb 61,130,0,140,13,208,0,224,30,51,0,241,12,26,33,108
drag5  defb 29,192,3,128,47,0,87,128,23,224,205,192,242,128,94,48
       defb 61,8,3,4,11,8,1,4,29,131,0,205,12,114,33,108
drag6  defb 29,192,3,128,47,0,87,128,23,224,205,192,242,96,94,208
       defb 61,128,3,0,14,8,4,8,28,7,2,21,13,42,33,236
drag7  defb 29,192,3,128,47,0,87,128,23,224,205,64,242,128,81,192
       defb 47,128,24,0,160,16,160,16,163,35,86,177,12,90,33,108

       defb 13,220,50,224,65,122,33,117,17,116,147,89,163,39,65,189
       defb 32,222,24,128,5,216,3,128,102,60,71,128,44,24,27,66
       defb 1,220,0,224,0,122,0,245,3,244,129,217,128,167,6,61
       defb 8,94,16,96,8,104,16,64,96,220,89,128,39,24,27,66
       defb 1,220,0,224,0,122,0,245,3,244,129,217,131,39,5,189
       defb 0,222,0,96,8,56,8,16,112,28,84,32,42,88,27,194
       defb 1,220,0,224,0,122,0,245,3,244,129,89,128,167,1,197
       defb 0,250,0,12,132,2,132,2,226,98,70,181,45,24,27,66
       defb 14,236,1,211,151,160,43,161,11,162,102,178,121,49,175,96
       defb 30,193,0,70,6,232,0,112,143,25,128,120,6,13,16,182
       defb 14,224,1,192,23,128,43,192,11,240,102,224,121,64,47,24
       defb 30,132,1,130,5,132,0,130,142,193,128,102,6,57,16,182
       defb 14,224,1,192,23,128,43,192,11,240,102,224,121,48,47,104
       defb 30,192,1,128,7,4,2,4,142,3,129,10,6,149,16,246
       defb 14,224,1,192,23,128,43,192,11,240,102,160,121,64,40,224
       defb 23,192,12,0,80,8,80,8,209,145,171,88,6,45,16,182

       defb 6,238,25,112,32,189,144,186,8,186,201,172,209,147,160,222
       defb 16,111,12,64,2,236,1,192,51,30,35,192,22,12,13,161
       defb 0,238,0,112,0,61,128,122,1,250,192,236,192,83,131,30
       defb 4,47,8,48,4,52,8,32,48,110,44,192,19,140,13,161
       defb 0,238,0,112,0,61,128,122,1,250,192,236,193,147,130,222
       defb 0,111,0,48,4,28,4,8,56,14,42,16,21,44,13,225
       defb 0,238,0,112,0,61,128,122,1,250,192,172,192,83,128,226
       defb 0,125,0,6,66,1,66,1,113,49,163,90,22,140,13,161
       defb 7,118,128,233,75,208,149,208,5,209,51,89,188,152,87,176
       defb 143,96,0,35,3,116,0,56,199,140,64,60,131,6,8,91
       defb 7,112,0,224,11,192,21,224,5,248,51,112,60,160,23,140
       defb 15,66,0,193,2,194,0,65,199,96,64,51,131,28,8,91
       defb 7,112,0,224,11,192,21,224,5,248,51,112,60,152,23,180
       defb 15,96,0,192,3,130,1,2,199,1,64,133,131,74,8,123
       defb 7,112,0,224,11,192,21,224,5,248,51,80,60,160,20,112
       defb 11,224,6,0,40,4,40,4,232,200,85,172,131,22,8,91

       defb 3,119,12,184,144,94,72,93,4,93,100,214,232,201,80,111
       defb 136,55,6,32,1,118,0,224,25,143,17,224,11,6,134,208
       defb 0,119,0,56,128,30,64,61,0,253,96,118,224,41,65,143
       defb 130,23,4,24,2,26,4,16,24,55,22,96,9,198,134,208
       defb 0,119,0,56,128,30,64,61,0,253,96,118,224,201,65,111
       defb 128,55,0,24,2,14,2,4,28,7,21,8,10,150,134,240
       defb 0,119,0,56,128,30,64,61,0,253,96,86,224,41,64,113
       defb 128,62,0,3,161,0,161,0,184,152,81,173,11,70,134,208
       defb 3,187,192,116,37,232,74,232,130,232,153,172,94,76,43,216
       defb 71,176,128,17,1,186,0,28,99,198,32,30,65,131,132,45
       defb 3,184,0,112,5,224,10,240,2,252,25,184,30,80,11,198
       defb 7,161,128,96,1,97,128,32,99,176,160,25,65,142,132,45
       defb 3,184,0,112,5,224,10,240,2,252,25,184,30,76,11,218
       defb 7,176,0,96,1,193,0,129,227,128,160,66,65,165,132,61
       defb 3,184,0,112,5,224,10,240,2,252,25,168,30,80,10,56
       defb 5,240,3,0,20,2,20,2,116,100,42,214,65,139,132,45

       defb 129,187,6,92,72,47,164,46,130,46,50,107,244,100,168,55
       defb 196,27,3,16,0,187,0,112,140,199,8,240,5,131,67,104
       defb 128,59,0,28,64,15,160,30,128,126,48,59,240,20,160,199
       defb 193,11,2,12,1,13,2,8,140,27,11,48,4,227,67,104
       defb 128,59,0,28,64,15,160,30,128,126,48,59,240,100,160,183
       defb 192,27,0,12,1,7,1,2,142,3,10,132,5,75,67,120
       defb 128,59,0,28,64,15,160,30,128,126,48,43,240,20,160,56
       defb 64,31,128,1,80,128,80,128,92,76,168,214,5,163,67,104
       defb 129,221,96,58,18,244,37,116,65,116,76,214,47,38,21,236
       defb 35,216,192,8,0,221,0,14,49,227,16,15,160,193,194,22
       defb 1,220,0,56,2,240,5,120,1,126,12,220,15,40,5,227
       defb 131,208,64,48,128,176,64,16,49,216,208,12,32,199,194,22
       defb 1,220,0,56,2,240,5,120,1,126,12,220,15,38,5,237
       defb 3,216,0,48,128,224,128,64,113,192,80,33,160,210,194,30
       defb 1,220,0,56,2,240,5,120,1,126,12,212,15,40,5,28
       defb 2,248,1,128,10,1,10,1,58,50,21,107,160,197,194,22

       defb 192,221,3,46,164,23,82,23,65,23,153,53,122,50,212,27
       defb 226,13,1,136,128,93,0,56,198,99,4,120,130,193,33,180
       defb 192,29,0,14,160,7,80,15,64,63,152,29,120,10,208,99
       defb 224,133,1,6,128,134,1,4,198,13,5,152,130,113,33,180
       defb 192,29,0,14,160,7,80,15,64,63,152,29,120,50,208,91
       defb 224,13,0,6,128,131,0,129,199,1,5,66,130,165,33,188
       defb 192,29,0,14,160,7,80,15,64,63,152,21,120,10,80,28
       defb 160,15,192,0,40,64,40,64,46,38,84,107,130,209,33,180
       defb 192,238,48,29,9,122,18,186,32,186,38,107,23,147,10,246
       defb 17,236,96,4,128,110,0,7,152,241,136,7,208,96,97,11
       defb 0,238,0,28,1,120,2,188,0,191,6,110,7,148,130,241
       defb 65,232,32,24,64,88,32,8,24,236,104,6,144,99,97,11
       defb 0,238,0,28,1,120,2,188,0,191,6,110,7,147,130,246
       defb 1,236,0,24,64,112,64,32,56,224,168,16,80,105,97,15
       defb 0,238,0,28,1,120,2,188,0,191,6,106,7,148,2,142
       defb 1,124,0,192,133,0,133,0,29,25,138,181,208,98,97,11

       defb 224,110,1,151,210,11,169,11,160,139,204,154,61,25,234,13
       defb 241,6,0,196,192,46,0,28,227,49,2,60,193,96,16,218
       defb 224,14,0,7,208,3,168,7,160,31,204,14,60,5,232,49
       defb 240,66,0,131,64,67,0,130,227,6,2,204,193,56,16,218
       defb 224,14,0,7,208,3,168,7,160,31,204,14,60,25,232,45
       defb 240,6,0,3,192,65,128,64,227,128,2,161,193,82,16,222
       defb 224,14,0,7,208,3,168,7,160,31,204,10,60,5,40,14
       defb 208,7,96,0,20,32,20,32,23,19,170,53,193,104,16,218
       defb 96,119,152,14,4,189,9,93,16,93,147,53,139,201,5,123
       defb 8,246,48,2,64,55,128,3,204,120,196,3,104,48,176,133
       defb 0,119,0,14,0,188,1,94,128,95,3,55,3,202,193,120
       defb 32,244,16,12,32,44,16,4,12,118,52,3,200,49,176,133
       defb 0,119,0,14,0,188,1,94,128,95,3,55,131,201,65,123
       defb 0,246,0,12,32,56,32,16,28,112,84,8,168,52,176,135
       defb 0,119,0,14,0,188,1,94,128,95,3,53,3,202,1,71
       defb 0,190,0,96,66,128,66,128,142,140,197,90,104,49,176,133

       defb 112,55,128,203,233,5,212,133,208,69,102,77,158,140,245,6
       defb 120,131,0,98,96,23,0,14,241,152,1,30,96,176,8,109
       defb 112,7,128,3,232,1,212,3,208,15,102,7,158,2,244,24
       defb 120,33,128,65,160,33,0,65,113,131,1,102,96,156,8,109
       defb 112,7,128,3,232,1,212,3,208,15,102,7,158,12,244,22
       defb 120,3,128,1,224,32,64,32,113,192,129,80,96,169,8,111
       defb 112,7,128,3,232,1,212,3,208,15,102,5,158,2,20,7
       defb 232,3,48,0,10,16,10,16,139,137,213,26,96,180,8,109
       defb 176,59,76,7,130,94,132,174,136,46,201,154,197,228,130,189
       defb 4,123,24,1,160,27,192,1,102,60,226,1,52,24,216,66
       defb 128,59,0,7,0,94,0,175,192,47,129,155,1,229,96,188
       defb 16,122,8,6,16,22,8,2,6,59,154,1,228,24,216,66
       defb 128,59,0,7,0,94,0,175,192,47,129,155,193,228,160,189
       defb 0,123,0,6,16,28,16,8,14,56,42,4,84,26,216,67
       defb 128,59,0,7,0,94,0,175,192,47,129,154,1,229,128,163
       defb 0,95,0,48,33,64,33,64,71,70,98,173,180,24,216,66

; Fuzzball and star sprites.

fuzz0  defb 4,160,43,116,47,248,159,254,121,61,246,222,111,239,237,110
       defb 109,111,118,222,185,60,63,249,95,212,42,168,5,32,0,0
fuzz1  defb 0,0,34,80,23,104,47,240,191,242,121,60,246,222,111,239
       defb 45,108,237,111,118,220,249,58,87,214,41,88,2,72,0,0
fuzz2  defb 0,0,2,72,41,88,87,214,255,250,121,60,246,223,47,236
       defb 109,111,237,110,118,220,185,50,47,240,23,104,34,80,0,0
fuzz3  defb 5,32,46,212,31,244,127,249,185,62,118,223,239,238,109,111
       defb 237,110,118,222,57,61,159,252,43,250,21,84,4,160,0,0
star0  defb 0,0,1,128,1,128,1,128,3,192,3,192,127,254,63,252
       defb 15,240,7,224,7,224,15,240,14,112,28,56,16,8,0,0
star1  defb 0,0,0,64,0,192,96,192,57,192,63,192,31,240,15,254
       defb 15,254,31,240,63,192,57,192,96,192,0,192,0,64,0,0
star2  defb 0,0,16,8,28,56,14,112,15,240,7,224,7,224,15,240
       defb 63,252,127,254,3,192,3,192,1,128,1,128,1,128,0,0
star3  defb 0,0,2,0,3,0,3,6,3,156,3,252,15,248,127,240
       defb 127,240,15,248,3,252,3,156,3,6,3,0,2,0,0,0

       defb 2,80,21,186,23,252,79,255,188,158,123,111,183,247,118,183
       defb 182,183,59,111,92,158,159,252,47,234,21,84,2,144,0,0
       defb 0,0,17,40,11,180,23,248,95,249,60,158,123,111,183,247
       defb 22,182,246,183,59,110,124,157,43,235,20,172,1,36,0,0
       defb 0,0,1,36,20,172,43,235,127,253,60,158,251,111,23,246
       defb 182,183,118,183,59,110,92,153,23,248,11,180,17,40,0,0
       defb 2,144,23,106,15,250,191,252,92,159,187,111,119,247,182,183
       defb 118,183,59,111,156,158,79,254,21,253,10,170,2,80,0,0
       defb 0,0,0,192,0,192,0,192,1,224,1,224,63,255,31,254
       defb 7,248,3,240,3,240,7,248,7,56,14,28,8,4,0,0
       defb 0,0,0,32,0,96,48,96,28,224,31,224,15,248,7,255
       defb 7,255,15,248,31,224,28,224,48,96,0,96,0,32,0,0
       defb 0,0,8,4,14,28,7,56,7,248,3,240,3,240,7,248
       defb 31,254,63,255,1,224,1,224,0,192,0,192,0,192,0,0
       defb 0,0,1,0,1,128,1,131,1,206,1,254,7,252,63,248
       defb 63,248,7,252,1,254,1,206,1,131,1,128,1,0,0,0

       defb 1,40,10,221,11,254,167,255,94,79,189,183,219,251,187,91
       defb 219,91,157,183,46,79,79,254,23,245,10,170,1,72,0,0
       defb 0,0,8,148,5,218,11,252,175,252,30,79,189,183,219,251
       defb 11,91,251,91,29,183,190,78,149,245,10,86,0,146,0,0
       defb 0,0,0,146,10,86,149,245,191,254,30,79,253,183,11,251
       defb 219,91,187,91,29,183,174,76,11,252,5,218,8,148,0,0
       defb 1,72,11,181,7,253,95,254,174,79,221,183,187,251,219,91
       defb 187,91,157,183,78,79,39,255,138,254,5,85,1,40,0,0
       defb 0,0,0,96,0,96,0,96,0,240,0,240,159,255,15,255
       defb 3,252,1,248,1,248,3,252,3,156,7,14,4,2,0,0
       defb 0,0,0,16,0,48,24,48,14,112,15,240,7,252,131,255
       defb 131,255,7,252,15,240,14,112,24,48,0,48,0,16,0,0
       defb 0,0,4,2,7,14,3,156,3,252,1,248,1,248,3,252
       defb 15,255,159,255,0,240,0,240,0,96,0,96,0,96,0,0
       defb 0,0,0,128,0,192,128,193,0,231,0,255,3,254,31,252
       defb 31,252,3,254,0,255,0,231,128,193,0,192,0,128,0,0

       defb 0,148,133,110,5,255,211,255,175,39,222,219,237,253,221,173
       defb 237,173,206,219,151,39,39,255,139,250,5,85,0,164,0,0
       defb 0,0,4,74,2,237,5,254,87,254,143,39,222,219,237,253
       defb 133,173,253,173,142,219,95,39,202,250,5,43,0,73,0,0
       defb 0,0,0,73,5,43,202,250,95,255,143,39,254,219,133,253
       defb 237,173,221,173,142,219,87,38,5,254,2,237,4,74,0,0
       defb 0,164,133,218,131,254,47,255,215,39,238,219,221,253,237,173
       defb 221,173,206,219,167,39,147,255,69,127,130,170,0,148,0,0
       defb 0,0,0,48,0,48,0,48,0,120,0,120,207,255,135,255
       defb 1,254,0,252,0,252,1,254,1,206,3,135,2,1,0,0
       defb 0,0,0,8,0,24,12,24,7,56,7,248,3,254,193,255
       defb 193,255,3,254,7,248,7,56,12,24,0,24,0,8,0,0
       defb 0,0,2,1,3,135,1,206,1,254,0,252,0,252,1,254
       defb 135,255,207,255,0,120,0,120,0,48,0,48,0,48,0,0
       defb 0,0,0,64,0,96,192,96,128,115,128,127,1,255,15,254
       defb 15,254,1,255,128,127,128,115,192,96,0,96,0,64,0,0

       defb 0,74,66,183,130,255,233,255,215,147,239,109,246,254,238,214
       defb 246,214,231,109,203,147,147,255,69,253,130,170,0,82,0,0
       defb 0,0,2,37,129,118,2,255,43,255,199,147,239,109,246,254
       defb 194,214,254,214,199,109,175,147,101,125,130,149,128,36,0,0
       defb 0,0,128,36,130,149,101,125,175,255,199,147,255,109,194,254
       defb 246,214,238,214,199,109,43,147,2,255,129,118,2,37,0,0
       defb 0,82,66,237,65,255,151,255,235,147,247,109,238,254,246,214
       defb 238,214,231,109,211,147,201,255,162,191,65,85,0,74,0,0
       defb 0,0,0,24,0,24,0,24,0,60,0,60,231,255,195,255
       defb 0,255,0,126,0,126,0,255,0,231,129,195,129,0,0,0
       defb 0,0,0,4,0,12,6,12,3,156,3,252,1,255,224,255
       defb 224,255,1,255,3,252,3,156,6,12,0,12,0,4,0,0
       defb 0,0,129,0,129,195,0,231,0,255,0,126,0,126,0,255
       defb 195,255,231,255,0,60,0,60,0,24,0,24,0,24,0,0
       defb 0,0,0,32,0,48,96,48,192,57,192,63,128,255,7,255
       defb 7,255,128,255,192,63,192,57,96,48,0,48,0,32,0,0

       defb 0,37,161,91,193,127,244,255,235,201,247,182,123,127,119,107
       defb 123,107,243,182,229,201,201,255,162,254,65,85,0,41,0,0
       defb 0,0,129,18,64,187,129,127,149,255,227,201,247,182,123,127
       defb 97,107,127,107,227,182,215,201,178,190,193,74,64,18,0,0
       defb 0,0,64,18,193,74,178,190,215,255,227,201,255,182,97,127
       defb 123,107,119,107,227,182,149,201,129,127,64,187,129,18,0,0
       defb 0,41,161,118,160,255,203,255,245,201,251,182,119,127,123,107
       defb 119,107,243,182,233,201,228,255,209,95,160,170,0,37,0,0
       defb 0,0,0,12,0,12,0,12,0,30,0,30,243,255,225,255
       defb 128,127,0,63,0,63,128,127,128,115,192,225,64,128,0,0
       defb 0,0,0,2,0,6,3,6,1,206,1,254,128,255,240,127
       defb 240,127,128,255,1,254,1,206,3,6,0,6,0,2,0,0
       defb 0,0,64,128,192,225,128,115,128,127,0,63,0,63,128,127
       defb 225,255,243,255,0,30,0,30,0,12,0,12,0,12,0,0
       defb 0,0,0,16,0,24,48,24,224,28,224,31,192,127,131,255
       defb 131,255,192,127,224,31,224,28,48,24,0,24,0,16,0,0

       defb 128,18,208,173,224,191,250,127,245,228,123,219,189,191,187,181
       defb 189,181,121,219,242,228,228,255,81,127,160,170,128,20,0,0
       defb 0,0,64,137,160,93,192,191,202,255,241,228,123,219,189,191
       defb 176,181,191,181,113,219,235,228,89,95,96,165,32,9,0,0
       defb 0,0,32,9,96,165,89,95,235,255,241,228,127,219,176,191
       defb 189,181,187,181,113,219,202,228,192,191,160,93,64,137,0,0
       defb 128,20,80,187,208,127,229,255,250,228,125,219,187,191,189,181
       defb 187,181,121,219,244,228,242,127,232,175,80,85,128,18,0,0
       defb 0,0,0,6,0,6,0,6,0,15,0,15,249,255,240,255
       defb 192,63,128,31,128,31,192,63,192,57,224,112,32,64,0,0
       defb 0,0,0,1,0,3,1,131,0,231,0,255,192,127,248,63
       defb 248,63,192,127,0,255,0,231,1,131,0,3,0,1,0,0
       defb 0,0,32,64,224,112,192,57,192,63,128,31,128,31,192,63
       defb 240,255,249,255,0,15,0,15,0,6,0,6,0,6,0,0
       defb 0,0,0,8,0,12,24,12,112,14,240,15,224,63,193,255
       defb 193,255,224,63,240,15,112,14,24,12,0,12,0,8,0,0

       defb 64,9,232,86,240,95,253,63,122,242,189,237,222,223,221,218
       defb 222,218,188,237,121,114,242,127,168,191,80,85,64,10,0,0
       defb 0,0,160,68,208,46,224,95,229,127,120,242,189,237,222,223
       defb 216,90,223,218,184,237,117,242,172,175,176,82,144,4,0,0
       defb 0,0,144,4,176,82,172,175,245,255,120,242,191,237,216,95
       defb 222,218,221,218,184,237,101,114,224,95,208,46,160,68,0,0
       defb 64,10,168,93,232,63,242,255,125,114,190,237,221,223,222,218
       defb 221,218,188,237,122,114,249,63,244,87,168,42,64,9,0,0
       defb 0,0,0,3,0,3,0,3,128,7,128,7,252,255,248,127
       defb 224,31,192,15,192,15,224,31,224,28,112,56,16,32,0,0
       defb 0,0,128,0,128,1,128,193,128,115,128,127,224,63,252,31
       defb 252,31,224,63,128,127,128,115,128,193,128,1,128,0,0,0
       defb 0,0,16,32,112,56,224,28,224,31,192,15,192,15,224,31
       defb 248,127,252,255,128,7,128,7,0,3,0,3,0,3,0,0
       defb 0,0,0,4,0,6,12,6,56,7,248,7,240,31,224,255
       defb 224,255,240,31,248,7,56,7,12,6,0,6,0,4,0,0



; Explosion sprites.

expl0  defb 0,0,0,0,0,0,0,0,0,0,0,192,3,240,7,240
       defb 15,224,3,248,7,224,1,128,0,0,0,0,0,0,0,0
expl1  defb 0,0,0,0,0,0,0,0,0,224,15,248,31,248,31,240
       defb 15,224,7,248,31,252,31,184,7,0,0,0,0,0,0,0
expl2  defb 0,0,0,0,24,56,60,124,62,254,63,252,31,248,7,224
       defb 7,224,7,252,31,254,63,62,31,28,6,0,0,0,0,0
expl3  defb 0,0,0,0,24,56,60,124,60,56,24,192,3,224,7,240
       defb 7,240,3,224,1,198,24,14,28,12,12,0,0,0,0,0
expl4  defb 0,0,0,0,16,16,8,40,24,48,0,0,1,128,3,192
       defb 3,192,1,128,0,6,16,10,12,0,8,0,0,0,0,0
expl5  defb 0,0,0,0,16,16,0,0,0,0,0,0,0,128,1,192
       defb 0,128,0,0,0,2,0,0,8,0,0,0,0,0,0,0
cake   defb 1,128,2,192,25,88,114,222,241,191,240,127,248,255,127,254
       defb 55,220,3,130,0,38,76,126,123,218,111,126,61,220,15,240
burg   defb 0,0,15,240,50,220,39,244,77,190,115,254,12,0,94,190
       defb 169,247,235,251,107,158,8,96,106,246,107,46,55,172,0,32

       defb 0,0,0,0,0,0,0,0,0,0,0,96,1,248,3,248
       defb 7,240,1,252,3,240,0,192,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,112,7,252,15,252,15,248
       defb 7,240,3,252,15,254,15,220,3,128,0,0,0,0,0,0
       defb 0,0,0,0,12,28,30,62,31,127,31,254,15,252,3,240
       defb 3,240,3,254,15,255,31,159,15,142,3,0,0,0,0,0
       defb 0,0,0,0,12,28,30,62,30,28,12,96,1,240,3,248
       defb 3,248,1,240,0,227,12,7,14,6,6,0,0,0,0,0
       defb 0,0,0,0,8,8,4,20,12,24,0,0,0,192,1,224
       defb 1,224,0,192,0,3,8,5,6,0,4,0,0,0,0,0
       defb 0,0,0,0,8,8,0,0,0,0,0,0,0,64,0,224
       defb 0,64,0,0,0,1,0,0,4,0,0,0,0,0,0,0
       defb 0,192,1,96,12,172,57,111,248,223,248,63,252,127,63,255
       defb 27,238,1,193,0,19,38,63,61,237,55,191,30,238,7,248
       defb 0,0,7,248,25,110,19,250,38,223,57,255,6,0,47,95
       defb 212,251,245,253,53,207,4,48,53,123,53,151,27,214,0,16

       defb 0,0,0,0,0,0,0,0,0,0,0,48,0,252,1,252
       defb 3,248,0,254,1,248,0,96,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,56,3,254,7,254,7,252
       defb 3,248,1,254,7,255,7,238,1,192,0,0,0,0,0,0
       defb 0,0,0,0,6,14,15,31,143,191,15,255,7,254,1,248
       defb 1,248,1,255,135,255,143,207,7,199,1,128,0,0,0,0
       defb 0,0,0,0,6,14,15,31,15,14,6,48,0,248,1,252
       defb 1,252,0,248,128,113,134,3,7,3,3,0,0,0,0,0
       defb 0,0,0,0,4,4,2,10,6,12,0,0,0,96,0,240
       defb 0,240,0,96,128,1,132,2,3,0,2,0,0,0,0,0
       defb 0,0,0,0,4,4,0,0,0,0,0,0,0,32,0,112
       defb 0,32,0,0,128,0,0,0,2,0,0,0,0,0,0,0
       defb 0,96,0,176,6,86,156,183,252,111,252,31,254,63,159,255
       defb 13,247,128,224,128,9,147,31,158,246,155,223,15,119,3,252
       defb 0,0,3,252,12,183,9,253,147,111,156,255,3,0,151,175
       defb 234,125,250,254,154,231,2,24,154,189,154,203,13,235,0,8

       defb 0,0,0,0,0,0,0,0,0,0,0,24,0,126,0,254
       defb 1,252,0,127,0,252,0,48,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,28,1,255,3,255,3,254
       defb 1,252,0,255,131,255,3,247,0,224,0,0,0,0,0,0
       defb 0,0,0,0,3,7,135,143,199,223,135,255,3,255,0,252
       defb 0,252,128,255,195,255,199,231,131,227,0,192,0,0,0,0
       defb 0,0,0,0,3,7,135,143,7,135,3,24,0,124,0,254
       defb 0,254,0,124,192,56,195,1,131,129,1,128,0,0,0,0
       defb 0,0,0,0,2,2,1,5,3,6,0,0,0,48,0,120
       defb 0,120,0,48,192,0,66,1,1,128,1,0,0,0,0,0
       defb 0,0,0,0,2,2,0,0,0,0,0,0,0,16,0,56
       defb 0,16,0,0,64,0,0,0,1,0,0,0,0,0,0,0
       defb 0,48,0,88,3,43,206,91,254,55,254,15,255,31,207,255
       defb 134,251,64,112,192,4,201,143,79,123,205,239,135,187,1,254
       defb 0,0,1,254,134,91,132,254,201,183,206,127,1,128,203,215
       defb 245,62,125,127,205,115,1,12,205,94,205,101,134,245,0,4

       defb 0,0,0,0,0,0,0,0,0,0,0,12,0,63,0,127
       defb 0,254,128,63,0,126,0,24,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,14,128,255,129,255,1,255
       defb 0,254,128,127,193,255,129,251,0,112,0,0,0,0,0,0
       defb 0,0,0,0,129,131,195,199,227,239,195,255,129,255,0,126
       defb 0,126,192,127,225,255,227,243,193,241,0,96,0,0,0,0
       defb 0,0,0,0,129,131,195,199,131,195,1,140,0,62,0,127
       defb 0,127,0,62,96,28,225,128,193,192,0,192,0,0,0,0
       defb 0,0,0,0,1,1,128,130,1,131,0,0,0,24,0,60
       defb 0,60,0,24,96,0,161,0,0,192,0,128,0,0,0,0
       defb 0,0,0,0,1,1,0,0,0,0,0,0,0,8,0,28
       defb 0,8,0,0,32,0,0,0,0,128,0,0,0,0,0,0
       defb 0,24,0,44,129,149,231,45,255,27,255,7,255,143,231,255
       defb 195,125,32,56,96,2,228,199,167,189,230,247,195,221,0,255
       defb 0,0,0,255,195,45,66,127,228,219,231,63,0,192,229,235
       defb 122,159,190,191,230,185,0,134,102,175,230,178,195,122,0,2

       defb 0,0,0,0,0,0,0,0,0,0,0,6,128,31,128,63
       defb 0,127,192,31,0,63,0,12,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,7,192,127,192,255,128,255
       defb 0,127,192,63,224,255,192,253,0,56,0,0,0,0,0,0
       defb 0,0,0,0,192,193,225,227,241,247,225,255,192,255,0,63
       defb 0,63,224,63,240,255,241,249,224,248,0,48,0,0,0,0
       defb 0,0,0,0,192,193,225,227,193,225,0,198,0,31,128,63
       defb 128,63,0,31,48,14,112,192,96,224,0,96,0,0,0,0
       defb 0,0,0,0,128,128,64,65,128,193,0,0,0,12,0,30
       defb 0,30,0,12,48,0,80,128,0,96,0,64,0,0,0,0
       defb 0,0,0,0,128,128,0,0,0,0,0,0,0,4,0,14
       defb 0,4,0,0,16,0,0,0,0,64,0,0,0,0,0,0
       defb 0,12,0,22,192,202,243,150,255,141,255,131,255,199,243,255
       defb 225,190,16,28,48,1,242,99,211,222,243,123,225,238,128,127
       defb 0,0,128,127,225,150,161,63,242,109,243,159,0,96,242,245
       defb 189,79,223,95,243,92,0,67,179,87,115,89,97,189,0,1

       defb 0,0,0,0,0,0,0,0,0,0,0,3,192,15,192,31
       defb 128,63,224,15,128,31,0,6,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,128,3,224,63,224,127,192,127
       defb 128,63,224,31,240,127,224,126,0,28,0,0,0,0,0,0
       defb 0,0,0,0,224,96,240,241,248,251,240,255,224,127,128,31
       defb 128,31,240,31,248,127,248,252,112,124,0,24,0,0,0,0
       defb 0,0,0,0,224,96,240,241,224,240,0,99,128,15,192,31
       defb 192,31,128,15,24,7,56,96,48,112,0,48,0,0,0,0
       defb 0,0,0,0,64,64,160,32,192,96,0,0,0,6,0,15
       defb 0,15,0,6,24,0,40,64,0,48,0,32,0,0,0,0
       defb 0,0,0,0,64,64,0,0,0,0,0,0,0,2,0,7
       defb 0,2,0,0,8,0,0,0,0,32,0,0,0,0,0,0
       defb 0,6,0,11,96,101,121,203,255,198,255,193,255,227,249,255
       defb 112,223,8,14,152,0,249,49,105,239,249,189,112,247,192,63
       defb 0,0,192,63,112,203,208,159,249,54,249,207,0,48,249,122
       defb 222,167,239,175,121,174,128,33,217,171,185,172,176,222,128,0

       defb 0,0,0,0,0,0,0,0,0,0,128,1,224,7,224,15
       defb 192,31,240,7,192,15,0,3,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,192,1,240,31,240,63,224,63
       defb 192,31,240,15,248,63,112,63,0,14,0,0,0,0,0,0
       defb 0,0,0,0,112,48,248,120,252,125,248,127,240,63,192,15
       defb 192,15,248,15,252,63,124,126,56,62,0,12,0,0,0,0
       defb 0,0,0,0,112,48,248,120,112,120,128,49,192,7,224,15
       defb 224,15,192,7,140,3,28,48,24,56,0,24,0,0,0,0
       defb 0,0,0,0,32,32,80,16,96,48,0,0,0,3,128,7
       defb 128,7,0,3,12,0,20,32,0,24,0,16,0,0,0,0
       defb 0,0,0,0,32,32,0,0,0,0,0,0,0,1,128,3
       defb 0,1,0,0,4,0,0,0,0,16,0,0,0,0,0,0
       defb 0,3,128,5,176,50,188,229,127,227,255,224,255,241,252,255
       defb 184,111,4,7,76,0,252,152,180,247,252,222,184,123,224,31
       defb 0,0,224,31,184,101,232,79,124,155,252,231,0,24,124,189
       defb 239,83,247,215,60,215,192,16,236,213,92,214,88,111,64,0


orange defb 7,0,24,192,51,48,117,136,118,180,251,116,216,130,253,154
       defb 253,173,174,181,191,57,95,194,93,254,39,252,24,248,7,224
plum   defb 0,0,6,192,31,112,63,248,47,252,95,252,111,254,87,254
       defb 79,254,71,254,43,254,37,124,16,188,12,24,3,224,0,0
z80    defb 31,240,95,244,95,244,24,48,91,180,88,52,31,240,88,52
       defb 90,180,24,48,95,244,89,180,26,176,91,52,95,244,31,240
nut    defb 0,0,7,224,31,248,63,252,120,30,240,47,120,94,191,253
       defb 95,251,167,231,80,15,162,175,85,78,34,172,21,72,0,0
robot0 defb 0,0,6,192,8,32,0,0,15,224,56,56,118,220,14,224
       defb 238,238,0,0,118,220,56,56,15,228,64,14,0,6,0,0
robot1 defb 0,0,6,192,8,32,0,0,15,224,56,56,118,220,15,224
       defb 237,110,13,96,118,220,56,56,79,228,96,12,0,0,0,0
robot2 defb 0,0,6,192,8,32,0,0,15,224,56,56,118,220,15,224
       defb 237,110,13,96,118,220,56,56,79,224,224,4,192,0,0,0
robot3 defb 0,0,6,192,8,32,0,0,15,224,56,56,118,220,15,224
       defb 237,110,13,96,118,220,56,56,79,228,96,12,0,0,0,0

       defb 3,128,12,96,25,152,58,196,59,90,125,186,108,65,126,205
       defb 254,214,215,90,223,156,47,225,46,255,19,254,12,124,3,240
       defb 0,0,3,96,15,184,31,252,23,254,47,254,55,255,43,255
       defb 39,255,35,255,21,255,18,190,8,94,6,12,1,240,0,0
       defb 15,248,47,250,47,250,12,24,45,218,44,26,15,248,44,26
       defb 45,90,12,24,47,250,44,218,13,88,45,154,47,250,15,248
       defb 0,0,3,240,15,252,31,254,60,15,248,23,60,47,223,254
       defb 175,253,211,243,168,7,209,87,42,167,17,86,10,164,0,0
       defb 0,0,3,96,4,16,0,0,7,240,28,28,59,110,7,112
       defb 119,119,0,0,59,110,28,28,7,242,32,7,0,3,0,0
       defb 0,0,3,96,4,16,0,0,7,240,28,28,59,110,7,240
       defb 118,183,6,176,59,110,28,28,39,242,48,6,0,0,0,0
       defb 0,0,3,96,4,16,0,0,7,240,28,28,59,110,7,240
       defb 118,183,6,176,59,110,28,28,39,240,112,2,96,0,0,0
       defb 0,0,3,96,4,16,0,0,7,240,28,28,59,110,7,240
       defb 118,183,6,176,59,110,28,28,39,242,48,6,0,0,0,0

       defb 1,192,6,48,12,204,29,98,29,173,62,221,182,32,191,102
       defb 127,107,107,173,111,206,151,240,151,127,9,255,6,62,1,248
       defb 0,0,1,176,7,220,15,254,11,255,23,255,155,255,149,255
       defb 147,255,145,255,138,255,9,95,4,47,3,6,0,248,0,0
       defb 7,252,23,253,23,253,6,12,22,237,22,13,7,252,22,13
       defb 22,173,6,12,23,253,22,109,6,172,22,205,23,253,7,252
       defb 0,0,1,248,7,254,15,255,158,7,252,11,158,23,111,255
       defb 215,254,233,249,212,3,232,171,149,83,8,171,5,82,0,0
       defb 0,0,1,176,2,8,0,0,3,248,14,14,29,183,3,184
       defb 187,187,0,0,29,183,14,14,3,249,144,3,128,1,0,0
       defb 0,0,1,176,2,8,0,0,3,248,14,14,29,183,3,248
       defb 187,91,3,88,29,183,14,14,19,249,24,3,0,0,0,0
       defb 0,0,1,176,2,8,0,0,3,248,14,14,29,183,3,248
       defb 187,91,3,88,29,183,14,14,19,248,56,1,48,0,0,0
       defb 0,0,1,176,2,8,0,0,3,248,14,14,29,183,3,248
       defb 187,91,3,88,29,183,14,14,19,249,24,3,0,0,0,0

       defb 0,224,3,24,6,102,14,177,142,214,159,110,91,16,95,179
       defb 191,181,181,214,55,231,75,248,203,191,132,255,3,31,0,252
       defb 0,0,0,216,3,238,7,255,133,255,139,255,205,255,202,255
       defb 201,255,200,255,197,127,132,175,130,23,1,131,0,124,0,0
       defb 3,254,139,254,139,254,3,6,139,118,139,6,3,254,139,6
       defb 139,86,3,6,139,254,139,54,3,86,139,102,139,254,3,254
       defb 0,0,0,252,3,255,135,255,207,3,254,5,207,11,183,255
       defb 107,255,244,252,234,1,244,85,202,169,132,85,2,169,0,0
       defb 0,0,0,216,1,4,0,0,1,252,7,7,142,219,1,220
       defb 221,221,0,0,142,219,7,7,129,252,200,1,192,0,0,0
       defb 0,0,0,216,1,4,0,0,1,252,7,7,142,219,1,252
       defb 221,173,1,172,142,219,7,7,137,252,140,1,0,0,0,0
       defb 0,0,0,216,1,4,0,0,1,252,7,7,142,219,1,252
       defb 221,173,1,172,142,219,7,7,9,252,156,0,24,0,0,0
       defb 0,0,0,216,1,4,0,0,1,252,7,7,142,219,1,252
       defb 221,173,1,172,142,219,7,7,137,252,140,1,0,0,0,0

       defb 0,112,1,140,3,51,135,88,71,107,79,183,45,136,175,217
       defb 223,218,90,235,155,243,37,252,229,223,194,127,129,143,0,126
       defb 0,0,0,108,1,247,131,255,194,255,197,255,230,255,229,127
       defb 228,255,228,127,226,191,194,87,193,11,128,193,0,62,0,0
       defb 1,255,69,255,69,255,1,131,69,187,69,131,1,255,69,131
       defb 69,171,1,131,69,255,69,155,1,171,69,179,69,255,1,255
       defb 0,0,0,126,129,255,195,255,231,129,255,2,231,133,219,255
       defb 181,255,122,126,245,0,250,42,229,84,194,42,129,84,0,0
       defb 0,0,0,108,0,130,0,0,0,254,131,131,199,109,0,238
       defb 238,238,0,0,199,109,131,131,64,254,228,0,96,0,0,0
       defb 0,0,0,108,0,130,0,0,0,254,131,131,199,109,0,254
       defb 238,214,0,214,199,109,131,131,68,254,198,0,0,0,0,0
       defb 0,0,0,108,0,130,0,0,0,254,131,131,199,109,0,254
       defb 238,214,0,214,199,109,131,131,4,254,78,0,12,0,0,0
       defb 0,0,0,108,0,130,0,0,0,254,131,131,199,109,0,254
       defb 238,214,0,214,199,109,131,131,68,254,198,0,0,0,0,0

       defb 0,56,0,198,129,153,67,172,163,181,167,219,22,196,215,236
       defb 111,237,173,117,205,249,18,254,242,239,225,63,192,199,0,63
       defb 0,0,0,54,128,251,193,255,225,127,226,255,243,127,242,191
       defb 242,127,242,63,241,95,225,43,224,133,192,96,0,31,0,0
       defb 128,255,162,255,162,255,128,193,162,221,162,193,128,255,162,193
       defb 162,213,128,193,162,255,162,205,128,213,162,217,162,255,128,255
       defb 0,0,0,63,192,255,225,255,243,192,127,129,243,194,237,255
       defb 218,255,61,63,122,128,125,21,114,170,97,21,64,170,0,0
       defb 0,0,0,54,0,65,0,0,0,127,193,193,227,182,0,119
       defb 119,119,0,0,227,182,193,193,32,127,114,0,48,0,0,0
       defb 0,0,0,54,0,65,0,0,0,127,193,193,227,182,0,127
       defb 119,107,0,107,227,182,193,193,34,127,99,0,0,0,0,0
       defb 0,0,0,54,0,65,0,0,0,127,193,193,227,182,0,127
       defb 119,107,0,107,227,182,193,193,2,127,39,0,6,0,0,0
       defb 0,0,0,54,0,65,0,0,0,127,193,193,227,182,0,127
       defb 119,107,0,107,227,182,193,193,34,127,99,0,0,0,0,0

       defb 0,28,0,99,192,204,33,214,209,218,211,237,11,98,107,246
       defb 183,246,214,186,230,252,9,127,249,119,240,159,224,99,128,31
       defb 0,0,0,27,192,125,224,255,240,191,241,127,249,191,249,95
       defb 249,63,249,31,248,175,240,149,240,66,96,48,128,15,0,0
       defb 192,127,209,127,209,127,192,96,209,110,209,96,192,127,209,96
       defb 209,106,192,96,209,127,209,102,192,106,209,108,209,127,192,127
       defb 0,0,128,31,224,127,240,255,121,224,191,192,121,225,246,255
       defb 237,127,158,159,61,64,190,138,57,85,176,138,32,85,0,0
       defb 0,0,0,27,128,32,0,0,128,63,224,224,113,219,128,59
       defb 187,187,0,0,113,219,224,224,144,63,57,0,24,0,0,0
       defb 0,0,0,27,128,32,0,0,128,63,224,224,113,219,128,63
       defb 187,181,128,53,113,219,224,224,145,63,49,128,0,0,0,0
       defb 0,0,0,27,128,32,0,0,128,63,224,224,113,219,128,63
       defb 187,181,128,53,113,219,224,224,129,63,19,128,3,0,0,0
       defb 0,0,0,27,128,32,0,0,128,63,224,224,113,219,128,63
       defb 187,181,128,53,113,219,224,224,145,63,49,128,0,0,0,0

       defb 0,14,128,49,96,102,16,235,104,237,233,246,5,177,53,251
       defb 91,251,107,93,115,126,132,191,252,187,248,79,240,49,192,15
       defb 0,0,128,13,224,62,240,127,248,95,248,191,252,223,252,175
       defb 252,159,252,143,252,87,248,74,120,33,48,24,192,7,0,0
       defb 224,63,232,191,232,191,96,48,104,183,104,176,224,63,104,176
       defb 104,181,96,48,232,191,104,179,96,53,104,182,232,191,224,63
       defb 0,0,192,15,240,63,248,127,60,240,95,224,188,240,251,127
       defb 246,191,207,79,30,160,95,69,156,170,88,69,144,42,0,0
       defb 0,0,128,13,64,16,0,0,192,31,112,112,184,237,192,29
       defb 221,221,0,0,184,237,112,112,200,31,28,128,12,0,0,0
       defb 0,0,128,13,64,16,0,0,192,31,112,112,184,237,192,31
       defb 221,218,192,26,184,237,112,112,200,159,24,192,0,0,0,0
       defb 0,0,128,13,64,16,0,0,192,31,112,112,184,237,192,31
       defb 221,218,192,26,184,237,112,112,192,159,9,192,1,128,0,0
       defb 0,0,128,13,64,16,0,0,192,31,112,112,184,237,192,31
       defb 221,218,192,26,184,237,112,112,200,159,24,192,0,0,0,0


; Demi-john image.

demigr defb 0,0,0,0,0,0,0,0,0,0,3,0,0,192,0,0
       defb 0,0,63,0,0,252,0,0,0,0,127,0,0,254,0,0
       defb 0,0,99,0,0,198,0,0,0,0,99,0,0,198,0,0
       defb 0,0,127,0,0,254,0,0,0,0,63,0,0,252,0,0
       defb 0,0,3,0,0,192,0,0,0,0,3,0,0,192,0,0
       defb 0,0,3,0,0,192,0,0,0,0,3,0,0,192,0,0
       defb 0,0,127,0,0,254,0,0,0,1,254,0,0,127,128,0
       defb 0,7,192,0,0,3,224,0,0,31,0,0,0,0,248,0
       defb 0,60,0,0,0,54,60,0,0,112,0,0,0,27,14,0
       defb 0,224,0,0,0,13,135,0,1,192,0,0,0,0,3,128
       defb 3,128,0,0,0,6,193,192,3,128,0,0,0,3,97,192
       defb 7,0,0,0,0,3,96,224,7,0,0,0,0,0,0,224
       defb 6,0,0,0,0,0,0,96,6,0,0,0,0,0,0,96
       defb 6,0,0,0,0,0,0,96,6,0,0,0,0,0,0,96
       defb 6,0,0,0,0,0,0,96,6,0,0,0,0,0,0,96
       defb 6,0,0,0,0,0,0,96,6,0,0,0,0,0,0,96
       defb 6,0,0,0,0,0,0,96,6,0,0,0,0,0,0,96
       defb 6,0,0,0,0,0,0,96,6,0,0,0,0,0,0,96
       defb 6,0,0,0,0,0,0,96,6,0,0,0,0,0,0,96
       defb 6,0,0,0,0,0,0,96,6,0,0,0,0,0,0,96
       defb 6,0,0,0,0,0,0,96,6,0,0,0,0,0,0,96
       defb 6,0,0,0,0,0,0,96,6,0,0,0,0,0,0,96
       defb 6,0,0,0,0,0,0,96,6,0,0,0,0,0,0,96
       defb 6,0,0,0,0,0,0,96,6,0,0,0,0,0,0,96
       defb 6,0,0,0,0,0,0,96,6,0,0,0,0,0,0,96
       defb 6,0,0,0,0,0,0,96,6,0,0,0,0,0,0,96
       defb 6,0,0,0,0,0,0,96,6,0,0,0,0,0,0,96
       defb 6,0,0,0,0,0,0,96,6,0,0,0,0,0,0,96
       defb 6,0,0,0,0,0,0,96,6,0,0,0,0,0,0,96
       defb 70,0,0,0,0,0,0,98,214,0,0,0,0,0,0,107
       defb 246,0,0,0,0,0,0,111,246,0,0,0,0,0,0,111
       defb 246,0,0,0,0,0,0,111,214,0,0,0,0,0,0,107
       defb 230,0,0,0,0,0,0,103,247,255,255,255,255,255,255,239
       defb 247,255,255,255,255,255,255,239,112,0,0,0,0,0,0,14
       defb 111,255,255,255,255,255,255,246,111,191,235,191,235,191,235,246
       defb 69,23,65,23,65,23,65,162,0,0,0,0,0,0,0,0



panel  defb 96,161,96,161,96,161,96,161,96,161,96,32,6,0,0,0
       defb 0,0,0,96,4,161,96,161,96,161,96,161,96,161,96,161
       defb 53,21,53,21,53,21,53,21,53,21,53,73,6,0,0,0
       defb 0,0,0,96,146,21,53,21,53,21,53,21,53,21,53,21
       defb 223,251,223,251,223,251,223,251,223,251,223,246,70,0,0,0
       defb 0,0,0,98,111,251,223,251,223,251,223,251,223,251,223,251
       defb 255,254,255,254,255,254,255,254,255,254,255,255,214,0,0,0
       defb 0,0,0,107,255,254,255,254,255,254,255,254,255,254,255,254
       defb 255,253,255,253,255,253,255,253,255,253,255,255,246,0,0,0
       defb 0,0,0,111,255,253,255,253,255,253,255,253,255,253,255,253
       defb 235,191,235,191,235,191,235,191,235,191,235,191,246,0,0,0
       defb 0,0,0,111,253,191,235,191,235,191,235,191,235,191,235,191
       defb 65,23,65,23,65,23,65,23,65,23,65,159,246,0,0,0
       defb 0,0,0,111,249,23,65,23,65,23,65,23,65,23,65,23
       defb 0,2,0,2,0,2,0,2,0,2,0,10,214,0,0,0
       defb 0,0,0,107,80,2,0,2,0,2,0,2,0,2,0,2
       defb 0,0,0,0,0,0,0,0,0,0,84,0,230,0,0,0
       defb 0,0,0,103,0,0,0,0,0,0,0,0,0,0,0,0
       defb 40,0,0,0,0,0,20,64,20,64,0,0,247,255,255,255
       defb 255,255,255,239,0,0,20,64,20,0,0,0,0,0,0,64
       defb 124,255,255,255,255,40,190,232,190,232,186,0,247,255,255,255
       defb 255,255,255,239,0,0,62,232,190,0,0,0,0,0,0,104
       defb 188,255,255,255,255,53,223,245,223,244,186,0,112,0,0,0
       defb 0,0,0,14,0,0,31,245,222,0,0,0,0,0,0,117
       defb 220,255,255,255,255,43,238,235,238,232,186,0,111,255,255,255
       defb 255,255,255,246,0,0,46,235,238,0,0,0,0,0,0,107
       defb 188,255,255,255,255,63,223,255,223,252,186,0,111,191,235,191
       defb 235,191,235,246,0,0,31,255,222,0,0,0,0,0,0,127
       defb 252,255,255,255,255,43,254,235,254,232,0,0,69,23,65,23
       defb 65,23,65,162,0,0,62,235,254,0,0,0,0,0,0,107
       defb 188,0,0,0,0,63,223,255,223,254,84,0,1,2,0,2
       defb 0,2,0,128,0,0,95,255,223,0,0,0,0,0,0,255
       defb 68,68,68,68,68,68,68,68,68,68,68,68,4,71,71,71
       defb 71,71,71,4,68,68,68,68,68,68,68,68,68,68,68,68
       defb 71,69,69,69,69,71,71,71,71,71,66,71,4,4,4,4
       defb 4,4,4,4,71,67,71,71,71,71,71,71,71,71,71,71

; Tiny fruit.

tinyf  defb 4,40,114,249,187,183,94,60,152,32,120,190,191,31,29,14
       defb 64,148,62,95,42,85,42,21,96,96,224,176,255,237,126,56
       defb 32,66,214,24,2,90,105,3,0,173,189,153,189,195,60,0
       defb 0,220,190,118,94,37,27,0,6,8,20,0,54,123,91,54
       defb 60,118,255,255,191,191,78,60,0,52,126,127,95,95,39,30


; Tiny sine table with just 16 angles.

sine   defb 1,1,237,0,183,0,99,0
       defb 0,0,98,0,180,0,236,0
       defb 0,1,236,0,180,0,96,0
       defb 0,0,99,0,183,0,237,0



; Screen compression stuff.
; Look, I know it's basic, it's been ripped from a collection of old screen
; compression routines I wrote around 1995, when I didn't comment anything.
; Kindly avert your eyes.

look   ld a,(23295)
       cpl
       ld (23296),a
       ld hl,65024
       ld de,65025
       ld bc,511
       ld (hl),l
       ldir
       ld hl,16384
       ld bc,27
look0  ld a,(hl)
       and 7           ;
       out (254),a     ; border.
       ld a,(hl)       ;
       ld d,0
       rla
       rl d
       ld e,a
       push hl
       ld hl,65024
       add hl,de
       inc (hl)
       ld a,(hl)
       cp 0
       jr nz,look1
       inc hl
       inc (hl)
look1  pop hl
       inc hl
       djnz look0
       dec c
       jr nz,look0
       ret


pkb    defb 0
comp   ld hl,65024
       ld b,0
comp1  ld a,(hl)
       inc hl
       or (hl)
       inc hl
       jr z,compfb     ; found byte.
       djnz comp1
       rst 0           ; failed; reset.
compfb dec hl
       dec hl
       ld de,65024
       and a
       sbc hl,de
       ld a,l
       rr h
       rra
       ld (pkb),a
       ld hl,16384
       ld de,40000
comp0  ld c,0          ; c=count.
       ld b,(hl)       ; b=byte.
comp9  ld a,(hl)       ; count.
       inc hl
       cp b
       jr nz,comp5     ; not same - end count.
       inc c
       jr z,comp4      ; 256 occurrences.
       jr comp9
comp4  dec c
       jr comp6
comp5  ld a,c
       cp 4
       jr c,comp8      ; not worth compressing.
comp6  ld a,(pkb)      ; compress.
       ld (de),a
       inc de
       ld a,b
       ld (de),a
       inc de
       ld a,c
       ld (de),a
       inc de
       dec hl
       jr comp7
comp8  ld a,b          ; store one byte.
       ld (de),a
       inc de
       ld b,0          ; back again.
       dec c
       sbc hl,bc
comp7  ld a,h
       cp 91           ; reached end of file?
       jr c,comp0
       push de
       pop bc
       ret


expand ld de,16384
       ld hl,screen    ; compressed screen data.
;       ld a,(pkb)
       ld c,9
expd0  ld a,(hl)
       cp c
       jr z,expd1
       ld (de),a
       inc de
expd2  inc hl
       ld a,d
       cp 91
       jr c,expd0
       ret
expd1  inc hl
       ld a,(hl)
       inc hl
       ld b,(hl)
expd3  ld (de),a
       inc de
       djnz expd3
       jr expd2


; Compressed menu screen.

screen defb 9,0,15,126,252,9,0,4,3,254,9,0,12,7,253,85
       defb 85,9,0,8,60,123,238,123,222,240,1,227,222,30,0,247
       defb 192,9,0,6,29,127,255,255,245,85,9,0,25,1,215,9
       defb 255,4,213,64,9,0,24,5,127,9,255,4,93,80,9,0
       defb 24,4,23,9,255,5,80,9,0,24,5,0,4,23,7,255
       defb 255,20,4,9,0,26,125,71,255,255,64,14,64,9,0,36
       defb 126,252,9,0,4,3,255,9,0,12,15,254,170,170,128,9
       defb 0,7,126,253,254,251,222,127,131,255,191,15,240,227,128,9
       defb 0,6,58,255,255,255,250,170,128,9,0,24,2,175,9,255
       defb 4,234,160,9,0,24,10,191,255,255,254,170,10,160,9,0
       defb 25,63,254,255,255,224,0,8,9,0,24,10,128,42,58,171
       defb 255,255,170,14,9,0,23,139,235,234,126,163,255,255,160,14
       defb 32,9,0,36,60,120,9,0,4,1,231,128,9,0,11,63
       defb 253,85,85,64,9,0,7,126,252,120,251,222,30,3,255,63
       defb 3,192,65,9,0,7,53,127,255,255,253,85,9,0,25,1
       defb 95,9,255,4,245,80,9,0,24,5,127,255,255,255,65,5
       defb 80,9,0,24,16,95,255,95,240,0,16,9,0,25,5,5
       defb 125,31,87,255,255,52,4,64,9,0,22,71,255,212,127,67
       defb 255,255,208,15,64,9,0,36,60,120,120,251,156,30,1,231
       defb 158,195,207,223,120,9,0,7,127,254,170,170,160,9,0,26
       defb 122,255,255,255,254,170,128,9,0,24,2,175,9,255,4,254
       defb 168,9,0,24,2,254,175,255,254,160,128,160,9,0,25,254
       defb 191,191,128,128,15,128,9,0,24,2,11,250,62,139,255,255
       defb 40,8,32,9,0,22,131,255,168,126,163,255,255,160,14,9
       defb 0,37,63,248,254,255,254,63,1,255,63,231,231,223,112,9
       defb 0,6,1,127,255,213,85,80,9,0,26,117,9,255,4,85
       defb 64,9,0,24,5,95,9,255,4,253,84,9,0,24,1,84
       defb 23,255,255,215,208,64,9,0,24,16,0,0,4,31,132,71
       defb 128,9,0,24,69,23,245,93,79,255,252,84,0,64,9,0
       defb 22,65,255,221,125,81,255,255,208,14,64,9,0,26,170,9
       defb 0,9,63,249,222,123,222,115,129,255,159,238,115,223,96,9
       defb 0,6,2,255,255,250,170,168,9,0,26,235,9,255,4,170
       defb 128,9,0,24,2,191,9,255,4,250,168,9,0,24,10,128
       defb 15,255,255,255,232,32,9,0,26,3,128,46,64,35,128,9
       defb 0,24,2,47,254,62,167,254,0,168,0,96,9,0,22,131
       defb 255,248,58,43,255,255,160,10,9,0,26,85,85,64,9,0
       defb 8,60,123,207,123,222,255,129,227,222,223,243,255,224,9,0
       defb 6,5,127,255,255,213,84,9,0,26,213,9,255,4,213,64
       defb 9,0,24,5,95,9,255,4,253,84,9,0,24,5,1,87
       defb 255,255,255,252,9,0,26,64,5,151,76,80,69,128,9,0
       defb 24,1,23,252,125,80,0,63,84,12,80,9,0,22,1,255
       defb 253,0,17,255,255,80,14,9,0,26,250,170,168,9,0,8
       defb 60,123,207,123,222,255,129,227,222,31,241,255,192,9,0,6
       defb 10,255,255,255,234,170,9,0,25,1,235,9,255,4,170,128
       defb 9,0,24,10,191,9,255,4,250,170,9,0,24,2,2,175
       defb 255,255,255,250,128,9,0,24,10,0,70,138,10,191,254,152
       defb 9,0,24,128,139,224,190,171,255,255,160,14,96,9,0,22
       defb 2,255,248,2,131,255,255,168,2,9,0,23,1,127,253,85
       defb 83,255,255,84,9,0,25,95,209,85,245,87,255,80,9,0
       defb 25,23,80,0,5,127,253,64,16,9,0,24,5,85,125,255
       defb 245,84,0,80,9,0,26,21,213,0,0,5,85,15,63,9
       defb 255,6,250,128,9,0,19,23,127,192,0,175,9,255,6,254
       defb 9,0,18,127,255,255,0,0,8,0,0,0,190,128,9,0
       defb 26,32,32,56,9,0,21,191,254,175,131,255,255,168,8,9
       defb 0,24,191,226,191,250,171,255,160,9,0,25,10,168,3,187
       defb 255,254,128,9,0,25,2,138,234,255,234,168,0,160,0,1
       defb 159,127,255,240,248,9,0,19,2,170,0,0,10,170,31,227
       defb 9,255,6,246,9,0,19,2,190,254,128,0,1,85,9,255
       defb 4,213,80,9,0,16,32,2,255,255,252,0,0,20,0,0
       defb 0,93,9,0,27,88,0,23,9,0,20,1,95,253,29,71
       defb 255,255,80,12,9,0,24,95,193,127,29,93,255,64,9,0
       defb 25,23,240,85,95,255,252,0,16,9,0,24,1,85,87,255
       defb 245,64,0,80,0,31,9,255,5,250,9,0,18,5,64,0
       defb 0,29,85,15,255,245,188,127,254,127,243,190,128,9,0,18
       defb 5,127,255,9,0,4,11,42,254,168,9,0,18,16,1,127
       defb 255,240,0,0,35,128,0,0,42,128,9,0,26,40,0,35
       defb 224,10,9,0,19,255,250,143,170,255,254,168,10,9,0,24
       defb 175,170,128,7,239,255,160,9,0,25,7,234,40,11,255,250
       defb 0,32,9,0,25,34,175,255,170,128,0,162,0,9,255,7
       defb 128,9,0,21,58,170,23,9,255,7,252,64,9,0,18,42
       defb 191,252,9,0,5,20,9,0,20,40,2,191,255,192,0,0
       defb 17,192,0,1,93,9,0,27,64,1,0,92,9,0,19,1
       defb 127,245,21,213,255,255,240,4,9,0,24,95,85,4,1,255
       defb 255,64,9,0,25,7,245,0,95,255,253,0,16,9,0,25
       defb 1,95,253,208,0,1,81,3,9,255,7,224,9,0,20,1
       defb 117,92,15,9,255,7,254,128,9,0,17,21,87,127,248,9
       defb 0,5,42,128,9,0,19,28,5,127,255,0,0,0,32,32
       defb 0,0,42,9,0,27,42,32,128,143,128,9,0,19,191,250
       defb 174,234,191,255,232,14,9,0,24,46,170,0,0,191,254,128
       defb 9,0,25,10,234,170,255,255,234,0,32,9,0,26,175,190
       defb 160,0,2,170,7,9,255,7,248,9,0,20,2,234,184,23
       defb 9,255,7,245,64,9,0,17,170,191,255,240,9,0,5,21
       defb 9,0,20,2,0,191,248,0,0,0,16,0,0,1,21,9
       defb 0,27,80,0,124,1,244,9,0,18,1,127,244,95,213,95
       defb 255,208,12,9,0,24,95,84,0,0,23,255,64,9,0,25
       defb 5,85,125,255,253,212,0,64,9,0,26,21,213,80,0,3
       defb 85,15,9,255,7,252,9,0,20,5,221,112,11,9,255,7
       defb 250,128,9,0,16,4,85,255,255,192,9,0,5,10,128,9
       defb 0,20,1,95,64,0,0,0,32,130,128,128,42,9,0,27
       defb 34,0,11,160,42,128,9,0,18,191,234,186,250,191,255,168
       defb 9,0,25,46,168,0,2,175,254,128,9,0,25,2,170,255
       defb 255,250,160,0,32,9,0,26,42,170,128,0,2,170,22,9
       defb 255,8,9,0,20,43,187,160,5,95,9,255,6,245,128,9
       defb 0,16,8,175,255,255,128,0,16,0,0,0,20,9,0,22
       defb 168,9,0,4,80,64,80,4,20,9,0,27,84,64,0,93
       defb 113,112,9,0,26,46,128,0,2,136,8,9,0,26,42,170
       defb 142,136,136,130,8,0,130,63,254,9,0,14,254,9,0,6
       defb 42,171,170,254,170,170,191,191,0,255,232,9,0,12,1,255
       defb 255,31,249,255,224,0,170,170,170,254,170,170,170,239,255,63
       defb 254,168,9,0,11,1,252,255,255,127,240,63,248,0,10,10
       defb 170,170,174,187,186,170,174,254,175,232,224,9,0,10,4,16
       defb 32,0,239,255,255,192,0,0,0,32,42,187,186,186,170,0
       defb 191,250,138,9,0,11,127,255,255,255,15,32,64,9,0,9
       defb 3,255,254,143,160,9,0,27,15,234,139,250,160,9,0,21
       defb 85,0,5,64,9,0,4,1,64,9,0,22,21,85,84,81
       defb 193,69,0,21,21,127,254,9,0,13,3,255,0,31,192,0
       defb 0,0,23,255,85,95,85,85,95,255,1,255,208,9,0,12
       defb 3,255,255,63,240,255,224,1,93,85,85,213,95,117,87,125
       defb 213,127,255,84,9,0,11,3,248,255,255,127,248,63,248,0
       defb 0,5,85,85,85,87,255,85,85,117,95,244,80,9,0,10
       defb 6,48,32,0,231,255,255,128,9,0,4,5,85,245,117,85
       defb 5,255,245,21,9,0,13,255,255,8,128,64,9,0,9,7
       defb 255,253,31,208,9,0,27,7,245,7,253,4,9,0,21,40
       defb 32,128,160,8,2,0,0,10,40,9,0,22,42,170,170,184
       defb 160,162,162,170,42,191,250,9,0,13,7,255,0,255,240,0
       defb 0,0,11,187,174,191,170,170,191,255,3,255,170,9,0,12
       defb 7,255,255,63,240,127,240,0,250,170,170,234,171,250,235,174
       defb 234,127,174,136,9,0,11,7,240,255,255,127,248,63,248,0
       defb 0,2,170,170,170,175,254,170,170,170,191,232,160,9,0,10
       defb 6,55,36,206,227,255,255,9,0,6,9,170,4,15,255,250
       defb 42,9,0,13,255,255,8,170,252,9,0,9,15,255,250,63
       defb 160,9,0,27,11,170,15,250,128,9,0,21,84,0,68,65
       defb 93,0,0,0,5,84,9,0,22,21,85,85,93,9,85,5
       defb 127,253,9,0,13,15,255,1,255,252,0,0,0,21,215,213
       defb 119,213,85,127,255,7,255,80,9,0,12,15,255,255,63,240
       defb 127,240,1,125,5,85,85,87,245,85,85,87,255,213,68,9
       defb 0,11,15,224,255,255,63,248,63,240,0,0,1,81,85,87
       defb 255,255,213,95,81,127,213,64,9,0,10,5,80,169,41,224
       defb 255,252,9,0,6,85,85,85,69,31,255,244,85,9,0,13
       defb 255,255,8,173,82,9,0,9,15,255,213,127,208,9,0,27
       defb 5,220,23,245,9,0,22,40,0,2,138,138,128,0,0,2
       defb 170,128,9,0,21,42,9,170,4,42,250,170,170,255,250,9
       defb 0,13,31,255,7,255,254,0,0,0,11,235,234,175,250,168
       defb 190,251,15,254,162,9,0,12,31,255,255,63,240,127,240,0
       defb 186,138,170,174,170,190,170,174,174,255,170,224,128,9,0,10
       defb 31,192,255,255,63,248,63,240,0,0,0,42,42,175,255,255
       defb 234,191,8,255,168,128,9,0,10,5,83,177,228,224,127,248
       defb 9,0,6,10,162,128,0,63,255,232,170,32,9,0,12,255
       defb 255,15,41,72,9,0,9,15,255,170,255,160,9,0,27,2
       defb 168,175,234,40,9,0,21,68,1,5,85,21,64,0,0,21
       defb 87,192,9,0,21,21,85,85,85,117,85,253,93,85,127,253
       defb 9,0,13,63,255,15,255,255,128,0,1,85,213,247,87,253
       defb 85,95,221,7,255,68,9,0,12,63,255,255,127,240,127,248
       defb 1,9,85,5,93,85,85,221,255,213,208,64,9,0,10,63
       defb 128,255,255,63,252,63,240,0,0,1,80,21,95,255,255,245
       defb 127,87,255,209,64,9,0,10,5,84,169,2,224,15,128,9
       defb 0,6,16,64,0,1,127,253,81,84,9,0,13,255,255,8
       defb 41,68,9,0,9,31,255,81,255,192,9,0,28,65,127,240
       defb 64,9,0,21,10,128,138,138,139,34,0,0,2,47,224,9
       defb 0,21,46,170,170,170,187,170,234,186,170,255,234,9,0,13
       defb 127,255,15,255,255,128,0,14,174,239,238,234,190,174,191,235
       defb 15,255,168,9,0,12,127,255,255,127,240,63,248,0,170,138
       defb 170,170,170,191,170,170,170,255,235,232,160,9,0,10,127,0
       defb 255,255,31,254,127,224,0,0,0,32,42,175,190,250,186,190
       defb 191,254,162,128,9,0,10,5,84,165,41,224,9,0,9,128
       defb 0,0,127,250,162,168,32,9,0,12,255,255,8,41,82,9
       defb 0,9,15,254,163,254,160,9,0,28,2,255,168,128,9,0
       defb 21,85,80,85,85,17,68,0,0,1,95,252,9,0,21,21
       defb 85,87,223,93,85,127,253,125,127,244,9,0,13,255,255,31
       defb 255,255,192,0,93,21,87,247,87,85,85,127,213,31,255,68
       defb 9,0,12,255,255,255,127,240,63,248,0,85,5,85,85,93
       defb 223,85,85,85,255,87,212,96,9,0,10,127,255,255,255,15
       defb 255,255,192,9,0,4,21,127,255,245,85,127,127,253,69,9
       defb 0,11,4,147,164,198,224,9,0,11,1,255,253,69,80,9
       defb 0,13,255,255,8,41,108,9,0,9,23,245,71,253,80,9
       defb 0,28,23,253,81,9,0,16,71,71,71,9,0,8,9,71
       defb 13,9,0,6,9,71,6,9,0,4,7,0,0,9,71,13
       defb 9,0,5,7,9,71,7,0,0,7,7,7,9,0,13,7
       defb 9,0,5,7,9,71,6,7,0,0,0,7,0,0,9,7
       defb 13,9,0,5,7,9,71,6,7,0,0,0,7,0,0,9
       defb 7,13,9,0,5,7,7,7,71,71,7,7,7,0,0,0
       defb 7,0,0,9,7,13,9,0,5,7,7,7,9,71,4,7
       defb 7,7,0,7,0,0,9,7,11,0,7,9,0,5,7,71
       defb 7,9,71,4,7,7,7,0,7,0,0,9,7,13,9,0
       defb 5,7,71,7,7,71,71,71,7,7,0,0,7,9,0,21
       defb 71,7,7,71,71,71,7,0,0,0,7,9,0,21,7,7
       defb 7,71,71,71,7,7,0,0,7,9,0,21,7,7,7,71
       defb 71,9,7,4,9,71,7,7,9,0,17,7,7,0,71,71
       defb 7,7,9,111,8,5,9,0,16,7,9,71,4,7,2,7
       defb 7,7,111,9,7,4,9,0,16,7,7,71,71,71,0,0
       defb 9,2,4,69,5,9,0,26,9,2,6,9,0,26,9,2
       defb 6,0,0,2,7,7,9,0,21,9,2,9,71,71,9,0
       defb 13,9,66,6,0,7,9,2,8,23,71,7,9,0,12,9
       defb 66,7,7,71,9,2,8,87,71,7,7,9,0,10,9,66
       defb 8,0,7,9,2,8,71,71,7,7,9,0,10,9,87,4
       defb 66,66,66,2,0,0,0,9,2,6,87,71,71,7,7,9
       defb 0,10,9,66,4,71,71,71,9,0,9,7,71,71,71,7
       defb 9,0,27,7,7,71,71,7,9,0,11




